@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: apic.cc,v 1.144 2010/05/15 09:23:50 vruppert Exp $
+// $Id: apic.cc 10891 2011-12-29 21:41:56Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (c) 2002-2009 Zwane Mwaikambo, Stanislav Shwartsman
@@ -151,7 +151,7 @@ static void apic_bus_broadcast_eoi(Bit8u vector)
 #endif
 
 // available even if APIC is not compiled in
-void apic_bus_deliver_smi(void)
+BOCHSAPI_MSVCONLY void apic_bus_deliver_smi(void)
 {
   BX_CPU(0)->deliver_SMI();
 }
@@ -184,8 +184,17 @@ bx_local_apic_c::bx_local_apic_c(BX_CPU_C *mycpu, unsigned id)
 
   // Register a non-active timer for use when the timer is started.
   timer_handle = bx_pc_system.register_timer_ticks(this,
-            BX_CPU(0)->lapic.periodic_smf, 0, 0, 0, "lapic");
+            bx_local_apic_c::periodic_smf, 0, 0, 0, "lapic");
   timer_active = 0;
+  
+#if BX_SUPPORT_VMX >= 2
+  // Register a non-active timer for VMX preemption timer.
+  vmx_timer_handle = bx_pc_system.register_timer_ticks(this,
+            bx_local_apic_c::vmx_preemption_timer_expired, 0, 0, 0, "vmx_preemtion");
+  BX_DEBUG(("vmx_timer is = %d", vmx_timer_handle));
+  vmx_preemption_timer_rate = VMX_MISC_PREEMPTION_TIMER_RATE;
+  vmx_timer_active = 0;
+#endif
 
   xapic = simulate_xapic; // xAPIC or legacy APIC
 
@@ -213,12 +222,20 @@ void bx_local_apic_c::reset(unsigned type)
   timer_divide_factor = 1;
   timer_initial = 0;
   timer_current = 0;
+  ticksInitial = 0;
 
   if(timer_active) {
     bx_pc_system.deactivate_timer(timer_handle);
     timer_active = 0;
   }
 
+#if BX_SUPPORT_VMX >= 2
+  if(vmx_timer_active) {
+    bx_pc_system.deactivate_timer(vmx_timer_handle);
+    vmx_timer_active = 0;
+  }
+#endif
+
   for(i=0; i<APIC_LVT_ENTRIES; i++) {
     lvt[i] = 0x10000;	// all LVT are masked
   }
@@ -240,7 +257,7 @@ void bx_local_apic_c::reset(unsigned type)
 
 void bx_local_apic_c::set_base(bx_phy_address newbase)
 {
-#if BX_SUPPORT_X2APIC
+#if BX_CPU_LEVEL >= 6
   if (mode == BX_APIC_X2APIC_MODE)
     ldr = ((apic_id & 0xfffffff0) << 16) | (1 << (apic_id & 0xf));
 #endif
@@ -305,6 +322,7 @@ void bx_local_apic_c::write(bx_phy_address addr, void *data, unsigned len)
 #define BX_LAPIC_ARBITRATION_PRIORITY 0x090
 #define BX_LAPIC_PPR                  0x0A0
 #define BX_LAPIC_EOI                  0x0B0
+#define BX_LAPIC_RRD                  0x0C0
 #define BX_LAPIC_LDR                  0x0D0
 #define BX_LAPIC_DESTINATION_FORMAT   0x0E0
 #define BX_LAPIC_SPURIOUS_VECTOR      0x0F0
@@ -347,12 +365,27 @@ void bx_local_apic_c::write(bx_phy_address addr, void *data, unsigned len)
 #define BX_LAPIC_TIMER_DIVIDE_CFG     0x3E0
 #define BX_LAPIC_SELF_IPI             0x3F0
 
+// extended AMD 
+#define BX_LAPIC_EXT_APIC_FEATURE     0x400
+#define BX_LAPIC_EXT_APIC_CONTROL     0x410
+#define BX_LAPIC_SPECIFIC_EOI         0x420
+#define BX_LAPIC_IER1                 0x480
+#define BX_LAPIC_IER2                 0x490
+#define BX_LAPIC_IER3                 0x4A0
+#define BX_LAPIC_IER4                 0x4B0
+#define BX_LAPIC_IER5                 0x4C0
+#define BX_LAPIC_IER6                 0x4D0
+#define BX_LAPIC_IER7                 0x4E0
+#define BX_LAPIC_IER8                 0x4F0
+
 // APIC write: 4 byte write to 16-byte aligned APIC address
 void bx_local_apic_c::write_aligned(bx_phy_address addr, Bit32u value)
 {
   BX_ASSERT((addr & 0xf) == 0);
+
   unsigned apic_reg = addr & 0xff0;
   BX_DEBUG(("LAPIC write 0x%08x to register 0x%04x", value, apic_reg));
+
   switch(apic_reg) {
     case BX_LAPIC_TPR: // task priority
       set_tpr(value & 0xff);
@@ -392,20 +425,8 @@ void bx_local_apic_c::write_aligned(bx_phy_address addr, Bit32u value)
     case BX_LAPIC_LVT_LINT0:   // LVT LINT0 Reg
     case BX_LAPIC_LVT_LINT1:   // LVT LINT1 Reg
     case BX_LAPIC_LVT_ERROR:   // LVT Error Reg
-      {
-         static Bit32u lvt_mask[] = {
-           0x000310ff, /* TIMER */
-           0x000117ff, /* THERMAL */
-           0x000117ff, /* PERFMON */
-           0x0001f7ff, /* LINT0 */
-           0x0001f7ff, /* LINT1 */
-           0x000110ff  /* ERROR */
-         };
-         unsigned lvt_entry = (apic_reg - BX_LAPIC_LVT_TIMER) >> 4;
-         lvt[lvt_entry] = value & lvt_mask[lvt_entry];
-         if(! software_enabled) lvt[lvt_entry] |= 0x10000;
-         break;
-      }
+      set_lvt_entry(apic_reg, value);
+      break;
     case BX_LAPIC_TIMER_INITIAL_COUNT:
       set_initial_timer_count(value);
       break;
@@ -414,10 +435,12 @@ void bx_local_apic_c::write_aligned(bx_phy_address addr, Bit32u value)
       timer_divconf = value & 0xb;
       set_divide_configuration(timer_divconf);
       break;
+      break;
     /* all read-only registers go here */
     case BX_LAPIC_ID:      // local APIC id
     case BX_LAPIC_VERSION: // local APIC version
     case BX_LAPIC_ARBITRATION_PRIORITY:
+    case BX_LAPIC_RRD:
     case BX_LAPIC_PPR:     // processor priority
     // ISRs not writable
     case BX_LAPIC_ISR1: case BX_LAPIC_ISR2:
@@ -442,7 +465,42 @@ void bx_local_apic_c::write_aligned(bx_phy_address addr, Bit32u value)
     default:
       shadow_error_status |= APIC_ERR_ILLEGAL_ADDR;
       // but for now I want to know about it in case I missed some.
-      BX_PANIC(("APIC register %x not implemented", apic_reg));
+      BX_ERROR(("APIC write: register %x not implemented", apic_reg));
+  }
+}
+
+void bx_local_apic_c::set_lvt_entry(unsigned apic_reg, Bit32u value)
+{
+  static Bit32u lvt_mask[] = {
+    0x000710ff, /* TIMER */
+    0x000117ff, /* THERMAL */
+    0x000117ff, /* PERFMON */
+    0x0001f7ff, /* LINT0 */
+    0x0001f7ff, /* LINT1 */
+    0x000110ff  /* ERROR */
+  };
+
+  unsigned lvt_entry = (apic_reg - BX_LAPIC_LVT_TIMER) >> 4;
+#if BX_CPU_LEVEL >= 6
+  if (apic_reg == BX_LAPIC_LVT_TIMER) {
+    if (! cpu->bx_cpuid_support_tsc_deadline()) {
+      value &= ~0x40000; // cannot enable TSC-Deadline when not supported
+    }
+    else {
+      if ((value ^ lvt[lvt_entry]) & 0x40000) {
+         // Transition between TSC-Deadline and other timer modes disarm the timer
+         if(timer_active) {
+            bx_pc_system.deactivate_timer(timer_handle);
+            timer_active = 0;
+         }
+      }
+    }
+  }
+#endif
+
+  lvt[lvt_entry] = value & lvt_mask[lvt_entry];
+  if(! software_enabled) {
+    lvt[lvt_entry] |= 0x10000;
   }
 }
 
@@ -543,8 +601,10 @@ Bit32u bx_local_apic_c::read_aligned(bx_phy_address addr)
 {
   BX_ASSERT((addr & 0xf) == 0);
   Bit32u data = 0;  // default value for unimplemented registers
+
   unsigned apic_reg = addr & 0xff0;
   BX_DEBUG(("LAPIC read from register 0x%04x", apic_reg));
+
   switch(apic_reg) {
   case BX_LAPIC_ID:      // local APIC id
     data = apic_id << 24; break;
@@ -638,22 +698,15 @@ Bit32u bx_local_apic_c::read_aligned(bx_phy_address addr)
     data = timer_initial;
     break;
   case BX_LAPIC_TIMER_CURRENT_COUNT: // current count for timer
-    if(timer_active==0) {
-      data = timer_current;
-    } else {
-      Bit64u delta64 = (bx_pc_system.time_ticks() - ticksInitial) / timer_divide_factor;
-      Bit32u delta32 = (Bit32u) delta64;
-      if(delta32 > timer_initial)
-        BX_PANIC(("APIC: R(curr timer count): delta < initial"));
-      timer_current = timer_initial - delta32;
-      data = timer_current;
-    }
+    data = get_current_timer_count();
     break;
   case BX_LAPIC_TIMER_DIVIDE_CFG:   // timer divide configuration
     data = timer_divconf;
     break;
   default:
-    BX_INFO(("APIC register %08x not implemented", apic_reg));
+      shadow_error_status |= APIC_ERR_ILLEGAL_ADDR;
+      // but for now I want to know about it in case I missed some.
+      BX_ERROR(("APIC read: register %x not implemented", apic_reg));
   }
 
   BX_DEBUG(("read from APIC address 0x" FMT_PHY_ADDRX " = %08x", addr, data));
@@ -811,7 +864,7 @@ bx_bool bx_local_apic_c::match_logical_addr(apic_dest_t address)
 {
   bx_bool match = 0;
 
-#if BX_SUPPORT_X2APIC
+#if BX_CPU_LEVEL >= 6
   if (mode == BX_APIC_X2APIC_MODE) {
     // only cluster model supported in x2apic mode
     if (address == 0xffffffff) // // broadcast all
@@ -908,10 +961,8 @@ void bx_local_apic_c::periodic(void)
     return;
   }
 
-  // timer reached zero since the last call to periodic.
   Bit32u timervec = lvt[APIC_LVT_TIMER];
-  if(timervec & 0x20000) {
-    // Periodic mode.
+
     // If timer is not masked, trigger interrupt.
     if((timervec & 0x10000)==0) {
       trigger_irq(timervec & 0xff, APIC_EDGE_TRIGGERED);
@@ -919,21 +970,17 @@ void bx_local_apic_c::periodic(void)
     else {
       BX_DEBUG(("local apic timer LVT masked"));
     }
-    // Reload timer values.
+
+  // timer reached zero since the last call to periodic.
+  if(timervec & 0x20000) {
+    // Periodic mode - reload timer values
     timer_current = timer_initial;
-    ticksInitial = bx_pc_system.time_ticks(); // Take a reading.
+    ticksInitial = bx_pc_system.time_ticks(); // timer value when it started to count
     BX_DEBUG(("local apic timer(periodic) triggered int, reset counter to 0x%08x", timer_current));
   }
   else {
     // one-shot mode
     timer_current = 0;
-    // If timer is not masked, trigger interrupt.
-    if((timervec & 0x10000)==0) {
-      trigger_irq(timervec & 0xff, APIC_EDGE_TRIGGERED);
-    }
-    else {
-      BX_DEBUG(("local apic timer LVT masked"));
-    }
     timer_active = 0;
     BX_DEBUG(("local apic timer(one-shot) triggered int"));
     bx_pc_system.deactivate_timer(timer_handle);
@@ -952,7 +999,14 @@ void bx_local_apic_c::set_divide_configuration(Bit32u value)
 
 void bx_local_apic_c::set_initial_timer_count(Bit32u value)
 {
-  // If active before, deactive the current timer before changing it.
+  Bit32u timervec = lvt[APIC_LVT_TIMER];
+
+#if BX_CPU_LEVEL >= 6
+  // in TSC-deadline mode writes to initial time count are ignored
+  if (timervec & 0x40000) return;
+#endif
+
+  // If active before, deactivate the current timer before changing it.
   if(timer_active) {
     bx_pc_system.deactivate_timer(timer_handle);
     timer_active = 0;
@@ -968,15 +1022,108 @@ void bx_local_apic_c::set_initial_timer_count(Bit32u value)
     BX_DEBUG(("APIC: Initial Timer Count Register = %u", value));
     timer_current = timer_initial;
     timer_active = 1;
-    Bit32u timervec = lvt[APIC_LVT_TIMER];
     bx_bool continuous = (timervec & 0x20000) > 0;
-    ticksInitial = bx_pc_system.time_ticks(); // Take a reading.
+    ticksInitial = bx_pc_system.time_ticks(); // timer value when it started to count
     bx_pc_system.activate_timer_ticks(timer_handle,
             Bit64u(timer_initial) * Bit64u(timer_divide_factor), continuous);
   }
 }
 
-#if BX_SUPPORT_X2APIC
+Bit32u bx_local_apic_c::get_current_timer_count(void)
+{
+#if BX_CPU_LEVEL >= 6
+  Bit32u timervec = lvt[APIC_LVT_TIMER];
+
+  // in TSC-deadline mode current timer count always reads 0
+  if (timervec & 0x40000) return 0;
+#endif
+
+  if(timer_active==0) {
+    return timer_current;
+  } else {
+    Bit64u delta64 = (bx_pc_system.time_ticks() - ticksInitial) / timer_divide_factor;
+    Bit32u delta32 = (Bit32u) delta64;
+    if(delta32 > timer_initial)
+      BX_PANIC(("APIC: R(curr timer count): delta < initial"));
+    timer_current = timer_initial - delta32;
+    return timer_current;
+  }
+}
+
+#if BX_CPU_LEVEL >= 6
+void bx_local_apic_c::set_tsc_deadline(Bit64u deadline)
+{
+  Bit32u timervec = lvt[APIC_LVT_TIMER];
+
+  if ((timervec & 0x40000) == 0) {
+    BX_ERROR(("APIC: TSC-Deadline timer is disabled"));
+    return;
+  }
+
+  // If active before, deactivate the current timer before changing it.
+  if(timer_active) {
+    bx_pc_system.deactivate_timer(timer_handle);
+    timer_active = 0;
+  }
+
+  ticksInitial = deadline;
+  if (deadline != 0) {
+    BX_INFO(("APIC: TSC-Deadline is set to " FMT_LL "d", deadline));
+    Bit64u currtime = bx_pc_system.time_ticks();
+    timer_active = 1;
+    bx_pc_system.activate_timer_ticks(timer_handle, (deadline > currtime) ? (deadline - currtime) : 1 , 0);
+  }
+}
+
+Bit64u bx_local_apic_c::get_tsc_deadline(void)
+{
+  Bit32u timervec = lvt[APIC_LVT_TIMER];
+
+  // read as zero if TSC-deadline timer is disabled
+  if ((timervec & 0x40000) == 0) return 0;
+
+  return ticksInitial; /* also holds TSC-deadline value */
+}
+#endif
+
+#if BX_SUPPORT_VMX >= 2
+Bit32u bx_local_apic_c::read_vmx_preemption_timer(void)
+{
+  Bit32u diff = (bx_pc_system.time_ticks() >> vmx_preemption_timer_rate) - (vmx_preemption_timer_initial >> vmx_preemption_timer_rate);
+  if (vmx_preemption_timer_value < diff)
+    return 0;
+  else
+    return vmx_preemption_timer_value - diff;
+}
+
+void bx_local_apic_c::set_vmx_preemption_timer(Bit32u value)
+{
+  vmx_preemption_timer_value = value;
+  vmx_preemption_timer_initial = bx_pc_system.time_ticks();
+  vmx_preemption_timer_fire = ((vmx_preemption_timer_initial >> vmx_preemption_timer_rate) + value) << vmx_preemption_timer_rate;
+  BX_DEBUG(("VMX Preemption timer. value = %u, rate = %u, init = %u, fire = %u", value, vmx_preemption_timer_rate, vmx_preemption_timer_initial, vmx_preemption_timer_fire));
+  bx_pc_system.activate_timer_ticks(vmx_timer_handle, vmx_preemption_timer_fire - vmx_preemption_timer_initial, 0);
+  vmx_timer_active = 1;
+}
+
+void bx_local_apic_c::deactivate_vmx_preemption_timer(void)
+{
+  if (! vmx_timer_active) return;
+  bx_pc_system.deactivate_timer(vmx_timer_handle);
+  vmx_timer_active = 0;
+}
+
+// Invoked when VMX preemption timer expired
+void bx_local_apic_c::vmx_preemption_timer_expired(void *this_ptr)
+{
+  bx_local_apic_c *class_ptr = (bx_local_apic_c *) this_ptr;
+  class_ptr->cpu->pending_vmx_timer_expired = 1;
+  class_ptr->cpu->async_event = 1;
+  class_ptr->deactivate_vmx_preemption_timer();
+}
+#endif
+  
+#if BX_CPU_LEVEL >= 6
 // return false when x2apic is not supported/not readable
 bx_bool bx_local_apic_c::read_x2apic(unsigned index, Bit64u *val_64)
 {
@@ -1142,7 +1289,7 @@ void bx_local_apic_c::register_state(bx_param_c *parent)
   unsigned i;
   char name[6];
 
-  bx_list_c *lapic = new bx_list_c(parent, "local_apic", 25);
+  bx_list_c *lapic = new bx_list_c(parent, "local_apic", 31);
 
   BXRS_HEX_PARAM_SIMPLE(lapic, base_addr);
   BXRS_HEX_PARAM_SIMPLE(lapic, apic_id);
@@ -1183,6 +1330,15 @@ void bx_local_apic_c::register_state(bx_param_c *parent)
   BXRS_PARAM_BOOL(lapic, timer_active, timer_active);
   BXRS_HEX_PARAM_SIMPLE(lapic, ticksInitial);
   BXRS_PARAM_BOOL(lapic, INTR, INTR);
+
+#if BX_SUPPORT_VMX >= 2
+  BXRS_DEC_PARAM_SIMPLE(lapic, vmx_timer_handle);
+  BXRS_HEX_PARAM_SIMPLE(lapic, vmx_preemption_timer_initial);
+  BXRS_HEX_PARAM_SIMPLE(lapic, vmx_preemption_timer_fire);
+  BXRS_HEX_PARAM_SIMPLE(lapic, vmx_preemption_timer_value);
+  BXRS_HEX_PARAM_SIMPLE(lapic, vmx_preemption_timer_rate);
+  BXRS_PARAM_BOOL(lapic, vmx_timer_active, vmx_timer_active);
+#endif
 }
 
 #endif /* if BX_SUPPORT_APIC */
