@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: bit32.cc,v 1.19 2010/10/18 22:19:45 sshwarts Exp $
+// $Id: bit32.cc 10798 2011-11-27 13:23:26Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2009  The Bochs Project
+//  Copyright (C) 2001-2011  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,7 +26,7 @@
 
 #if BX_CPU_LEVEL >= 3
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BSF_GdEdR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BSF_GdEdR(bxInstruction_c *i)
 {
   Bit32u op2_32 = BX_READ_32BIT_REG(i->rm());
 
@@ -46,9 +46,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BSF_GdEdR(bxInstruction_c *i)
     /* now write result back to destination */
     BX_WRITE_32BIT_REGZ(i->nnn(), op1_32);
   }
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BSR_GdEdR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BSR_GdEdR(bxInstruction_c *i)
 {
   Bit32u op2_32 = BX_READ_32BIT_REG(i->rm());
 
@@ -68,9 +70,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BSR_GdEdR(bxInstruction_c *i)
     /* now write result back to destination */
     BX_WRITE_32BIT_REGZ(i->nnn(), op1_32);
   }
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EdGdM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EdGdM(bxInstruction_c *i)
 {
   bx_address op1_addr;
   Bit32u op1_32, op2_32, index;
@@ -87,9 +91,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EdGdM(bxInstruction_c *i)
   op1_32 = read_virtual_dword(i->seg(), op1_addr & i->asize_mask());
 
   set_CF((op1_32 >> index) & 0x01);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EdGdR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EdGdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
@@ -98,9 +104,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EdGdR(bxInstruction_c *i)
   op2_32 &= 0x1f;
 
   set_CF((op1_32 >> op2_32) & 0x01);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EdGdM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EdGdM(bxInstruction_c *i)
 {
   bx_address op1_addr;
   Bit32u op1_32, op2_32, index;
@@ -118,14 +126,16 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EdGdM(bxInstruction_c *i)
   op1_32 = read_RMW_virtual_dword(i->seg(), op1_addr & i->asize_mask());
 
   bit_i = (op1_32 >> index) & 0x01;
-  op1_32 |= (((Bit32u) 1) << index);
+  op1_32 |= (1 << index);
 
   write_RMW_virtual_dword(op1_32);
 
   set_CF(bit_i);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EdGdR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EdGdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
@@ -133,13 +143,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EdGdR(bxInstruction_c *i)
   op2_32 = BX_READ_32BIT_REG(i->nnn());
   op2_32 &= 0x1f;
   set_CF((op1_32 >> op2_32) & 0x01);
-  op1_32 |= (((Bit32u) 1) << op2_32);
+  op1_32 |= (1 << op2_32);
 
   /* now write result back to the destination */
   BX_WRITE_32BIT_REGZ(i->rm(), op1_32);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EdGdM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EdGdM(bxInstruction_c *i)
 {
   bx_address op1_addr;
   Bit32u op1_32, op2_32, index;
@@ -156,15 +168,17 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EdGdM(bxInstruction_c *i)
   op1_32 = read_RMW_virtual_dword(i->seg(), op1_addr & i->asize_mask());
 
   bx_bool temp_cf = (op1_32 >> index) & 0x01;
-  op1_32 &= ~(((Bit32u) 1) << index);
+  op1_32 &= ~(1 << index);
 
   /* now write back to destination */
   write_RMW_virtual_dword(op1_32);
 
   set_CF(temp_cf);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EdGdR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EdGdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
@@ -172,13 +186,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EdGdR(bxInstruction_c *i)
   op2_32 = BX_READ_32BIT_REG(i->nnn());
   op2_32 &= 0x1f;
   set_CF((op1_32 >> op2_32) & 0x01);
-  op1_32 &= ~(((Bit32u) 1) << op2_32);
+  op1_32 &= ~(1 << op2_32);
 
   /* now write result back to the destination */
   BX_WRITE_32BIT_REGZ(i->rm(), op1_32);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EdGdM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EdGdM(bxInstruction_c *i)
 {
   bx_address op1_addr;
   Bit32u op1_32, op2_32, index_32;
@@ -194,13 +210,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EdGdM(bxInstruction_c *i)
 
   op1_32 = read_RMW_virtual_dword(i->seg(), op1_addr & i->asize_mask());
   bx_bool temp_CF = (op1_32 >> index_32) & 0x01;
-  op1_32 ^= (((Bit32u) 1) << index_32);  /* toggle bit */
+  op1_32 ^= (1 << index_32);  /* toggle bit */
   set_CF(temp_CF);
 
   write_RMW_virtual_dword(op1_32);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EdGdR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EdGdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
@@ -209,13 +227,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EdGdR(bxInstruction_c *i)
   op2_32 &= 0x1f;
 
   bx_bool temp_CF = (op1_32 >> op2_32) & 0x01;
-  op1_32 ^= (((Bit32u) 1) << op2_32);  /* toggle bit */
+  op1_32 ^= (1 << op2_32);  /* toggle bit */
   set_CF(temp_CF);
 
   BX_WRITE_32BIT_REGZ(i->rm(), op1_32);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EdIbM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EdIbM(bxInstruction_c *i)
 {
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
@@ -223,17 +243,21 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EdIbM(bxInstruction_c *i)
   Bit8u  op2_8  = i->Ib() & 0x1f;
 
   set_CF((op1_32 >> op2_8) & 0x01);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EdIbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EdIbR(bxInstruction_c *i)
 {
   Bit32u op1_32 = BX_READ_32BIT_REG(i->rm());
   Bit8u  op2_8  = i->Ib() & 0x1f;
 
   set_CF((op1_32 >> op2_8) & 0x01);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EdIbM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EdIbM(bxInstruction_c *i)
 {
   Bit8u op2_8 = i->Ib() & 0x1f;
 
@@ -241,25 +265,29 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EdIbM(bxInstruction_c *i)
 
   Bit32u op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   bx_bool temp_CF = (op1_32 >> op2_8) & 0x01;
-  op1_32 |= (((Bit32u) 1) << op2_8);
+  op1_32 |= (1 << op2_8);
   write_RMW_virtual_dword(op1_32);
 
   set_CF(temp_CF);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EdIbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EdIbR(bxInstruction_c *i)
 {
   Bit8u op2_8 = i->Ib() & 0x1f;
 
   Bit32u op1_32 = BX_READ_32BIT_REG(i->rm());
   bx_bool temp_CF = (op1_32 >> op2_8) & 0x01;
-  op1_32 |= (((Bit32u) 1) << op2_8);
+  op1_32 |= (1 << op2_8);
   BX_WRITE_32BIT_REGZ(i->rm(), op1_32);
 
   set_CF(temp_CF);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EdIbM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EdIbM(bxInstruction_c *i)
 {
   Bit8u op2_8 = i->Ib() & 0x1f;
 
@@ -267,25 +295,29 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EdIbM(bxInstruction_c *i)
 
   Bit32u op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   bx_bool temp_CF = (op1_32 >> op2_8) & 0x01;
-  op1_32 ^= (((Bit32u) 1) << op2_8);  /* toggle bit */
+  op1_32 ^= (1 << op2_8);  /* toggle bit */
   write_RMW_virtual_dword(op1_32);
 
   set_CF(temp_CF);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EdIbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EdIbR(bxInstruction_c *i)
 {
   Bit8u op2_8 = i->Ib() & 0x1f;
 
   Bit32u op1_32 = BX_READ_32BIT_REG(i->rm());
   bx_bool temp_CF = (op1_32 >> op2_8) & 0x01;
-  op1_32 ^= (((Bit32u) 1) << op2_8);  /* toggle bit */
+  op1_32 ^= (1 << op2_8);  /* toggle bit */
   BX_WRITE_32BIT_REGZ(i->rm(), op1_32);
 
   set_CF(temp_CF);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EdIbM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EdIbM(bxInstruction_c *i)
 {
   Bit8u op2_8 = i->Ib() & 0x1f;
 
@@ -293,26 +325,30 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EdIbM(bxInstruction_c *i)
 
   Bit32u op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   bx_bool temp_CF = (op1_32 >> op2_8) & 0x01;
-  op1_32 &= ~(((Bit32u) 1) << op2_8);
+  op1_32 &= ~(1 << op2_8);
   write_RMW_virtual_dword(op1_32);
 
   set_CF(temp_CF);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EdIbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EdIbR(bxInstruction_c *i)
 {
   Bit8u op2_8 = i->Ib() & 0x1f;
 
   Bit32u op1_32 = BX_READ_32BIT_REG(i->rm());
   bx_bool temp_CF = (op1_32 >> op2_8) & 0x01;
-  op1_32 &= ~(((Bit32u) 1) << op2_8);
+  op1_32 &= ~(1 << op2_8);
   BX_WRITE_32BIT_REGZ(i->rm(), op1_32);
 
   set_CF(temp_CF);
+
+  BX_NEXT_INSTR(i);
 }
 
 /* F3 0F B8 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::POPCNT_GdEdR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::POPCNT_GdEdR(bxInstruction_c *i)
 {
   Bit32u op2_32 = BX_READ_32BIT_REG(i->rm());
 
@@ -325,8 +361,47 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::POPCNT_GdEdR(bxInstruction_c *i)
   Bit32u flags = op1_32 ? 0 : EFlagsZFMask;
   setEFlagsOSZAPC(flags);
 
-  /* now write result back to destination */
   BX_WRITE_32BIT_REGZ(i->nnn(), op1_32);
+
+  BX_NEXT_INSTR(i);
+}
+
+/* F3 0F BC */
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::TZCNT_GdEdR(bxInstruction_c *i)
+{
+  Bit32u op1_32 = BX_READ_32BIT_REG(i->rm());
+  Bit32u mask = 0x1, result_32 = 0;
+
+  while ((op1_32 & mask) == 0 && mask) {
+    mask <<= 1;
+    result_32++;
+  }
+
+  set_CF(! op1_32);
+  set_ZF(! result_32);
+  
+  BX_WRITE_32BIT_REGZ(i->nnn(), result_32);
+
+  BX_NEXT_INSTR(i);
+}
+
+/* F3 0F BD */
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::LZCNT_GdEdR(bxInstruction_c *i)
+{
+  Bit32u op1_32 = BX_READ_32BIT_REG(i->rm());
+  Bit32u mask = 0x80000000, result_32 = 0;
+
+  while ((op1_32 & mask) == 0 && mask) {
+    mask >>= 1;
+    result_32++;
+  }
+
+  set_CF(! op1_32);
+  set_ZF(! result_32);
+  
+  BX_WRITE_32BIT_REGZ(i->nnn(), result_32);
+
+  BX_NEXT_INSTR(i);
 }
 
 #endif // (BX_CPU_LEVEL >= 3)
