@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: cpu.cc,v 1.319 2011/01/26 11:48:13 sshwarts Exp $
+// $Id: cpu.cc 10683 2011-09-22 19:38:52Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2009  The Bochs Project
+//  Copyright (C) 2001-2011  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,12 +26,6 @@
 
 #include "iodev/iodev.h"
 
-// Make code more tidy with a few macros.
-#if BX_SUPPORT_X86_64==0
-#define RIP EIP
-#define RCX ECX
-#endif
-
 #define InstrumentICACHE 0
 
 #if InstrumentICACHE
@@ -55,24 +49,7 @@ static unsigned iCacheMisses=0;
 #define InstrICache_Increment(v)
 #endif
 
-// The CHECK_MAX_INSTRUCTIONS macro allows cpu_loop to execute a few
-// instructions and then return so that the other processors have a chance to
-// run.  This is used by bochs internal debugger or when simulating
-// multiple processors.
-//
-// If maximum instructions have been executed, return. The zero-count
-// means run forever.
-#if BX_SUPPORT_SMP || BX_DEBUGGER
-  #define CHECK_MAX_INSTRUCTIONS(count) \
-    if ((count) > 0) {                  \
-      (count)--;                        \
-      if ((count) == 0) return;         \
-    }
-#else
-  #define CHECK_MAX_INSTRUCTIONS(count)
-#endif
-
-void BX_CPU_C::cpu_loop(Bit32u max_instr_count)
+void BX_CPU_C::cpu_loop(void)
 {
 #if BX_DEBUGGER
   BX_CPU_THIS_PTR break_point = 0;
@@ -81,13 +58,14 @@ void BX_CPU_C::cpu_loop(Bit32u max_instr_count)
 #endif
 
   if (setjmp(BX_CPU_THIS_PTR jmp_buf_env)) {
-    // only from exception function we can get here ...
-    BX_INSTR_NEW_INSTRUCTION(BX_CPU_ID);
-    BX_TICK1_IF_SINGLE_PROCESSOR();
+    // can get here only from exception function or VMEXIT
+#if BX_SUPPORT_HANDLERS_CHAINING_SPEEDUPS
+    BX_CPU_THIS_PTR icount++;
+#endif
+    BX_SYNC_TIME_IF_SINGLE_PROCESSOR(0);
 #if BX_DEBUGGER || BX_GDBSTUB
     if (dbg_instruction_epilog()) return;
 #endif
-    CHECK_MAX_INSTRUCTIONS(max_instr_count);
 #if BX_GDBSTUB
     if (bx_dbg.gdbstub_enabled) return;
 #endif
@@ -120,47 +98,48 @@ void BX_CPU_C::cpu_loop(Bit32u max_instr_count)
     }
 
     bxICacheEntry_c *entry = getICacheEntry();
-
     bxInstruction_c *i = entry->i;
 
-#if BX_SUPPORT_TRACE_CACHE
+#if BX_SUPPORT_HANDLERS_CHAINING_SPEEDUPS
+    for(;;) {
+
+      // want to allow changing of the instruction inside instrumentation callback
+      BX_INSTR_BEFORE_EXECUTION(BX_CPU_ID, i);
+      RIP += i->ilen();
+      // when handlers chaining is enabled this single call will execute entire trace
+      BX_CPU_CALL_METHOD(i->execute, (i)); // might iterate repeat instruction
+
+      BX_SYNC_TIME_IF_SINGLE_PROCESSOR(0);
+
+      if (BX_CPU_THIS_PTR async_event) break;
+
+      i = getICacheEntry()->i;
+    }
+#else // BX_SUPPORT_HANDLERS_CHAINING_SPEEDUPS == 0
+
     bxInstruction_c *last = i + (entry->tlen);
 
     for(;;) {
-#endif
 
-#if BX_DISASM
-      if (BX_CPU_THIS_PTR trace) {
-        // print the instruction that is about to be executed
+#if BX_DEBUGGER
+      if (BX_CPU_THIS_PTR trace)
         debug_disasm_instruction(BX_CPU_THIS_PTR prev_rip);
-      }
 #endif
 
-      // instruction decoding completed -> continue with execution
       // want to allow changing of the instruction inside instrumentation callback
       BX_INSTR_BEFORE_EXECUTION(BX_CPU_ID, i);
       RIP += i->ilen();
       BX_CPU_CALL_METHOD(i->execute, (i)); // might iterate repeat instruction
       BX_CPU_THIS_PTR prev_rip = RIP; // commit new RIP
       BX_INSTR_AFTER_EXECUTION(BX_CPU_ID, i);
-      BX_TICK1_IF_SINGLE_PROCESSOR();
-
-      // inform instrumentation about new instruction
-      BX_INSTR_NEW_INSTRUCTION(BX_CPU_ID);
+      BX_SYNC_TIME_IF_SINGLE_PROCESSOR(0);
 
       // note instructions generating exceptions never reach this point
 #if BX_DEBUGGER || BX_GDBSTUB
       if (dbg_instruction_epilog()) return;
 #endif
 
-      CHECK_MAX_INSTRUCTIONS(max_instr_count);
-
-#if BX_SUPPORT_TRACE_CACHE
-      if (BX_CPU_THIS_PTR async_event) {
-        // clear stop trace magic indication that probably was set by repeat or branch32/64
-        BX_CPU_THIS_PTR async_event &= ~BX_ASYNC_EVENT_STOP_TRACE;
-        break;
-      }
+      if (BX_CPU_THIS_PTR async_event) break;
 
       if (++i == last) {
         entry = getICacheEntry();
@@ -169,9 +148,71 @@ void BX_CPU_C::cpu_loop(Bit32u max_instr_count)
       }
     }
 #endif
+
+    // clear stop trace magic indication that probably was set by repeat or branch32/64
+    BX_CPU_THIS_PTR async_event &= ~BX_ASYNC_EVENT_STOP_TRACE;
+
   }  // while (1)
 }
 
+#if BX_SUPPORT_SMP
+
+void BX_CPU_C::cpu_run_trace(void)
+{
+  if (setjmp(BX_CPU_THIS_PTR jmp_buf_env)) {
+    // can get here only from exception function or VMEXIT
+    BX_CPU_THIS_PTR icount++;
+    return;
+  }
+
+  // check on events which occurred for previous instructions (traps)
+  // and ones which are asynchronous to the CPU (hardware interrupts)
+  if (BX_CPU_THIS_PTR async_event) {
+    if (handleAsyncEvent()) {
+      // If request to return to caller ASAP.
+      return;
+    }
+  }
+
+  bxICacheEntry_c *entry = getICacheEntry();
+  bxInstruction_c *i = entry->i;
+
+#if BX_SUPPORT_HANDLERS_CHAINING_SPEEDUPS
+  // want to allow changing of the instruction inside instrumentation callback
+  BX_INSTR_BEFORE_EXECUTION(BX_CPU_ID, i);
+  RIP += i->ilen();
+  // when handlers chaining is enabled this single call will execute entire trace
+  BX_CPU_CALL_METHOD(i->execute, (i)); // might iterate repeat instruction
+
+  if (BX_CPU_THIS_PTR async_event) {
+    // clear stop trace magic indication that probably was set by repeat or branch32/64
+    BX_CPU_THIS_PTR async_event &= ~BX_ASYNC_EVENT_STOP_TRACE;
+  }
+#else
+  bxInstruction_c *last = i + (entry->tlen);
+
+  for(;;) {
+    // want to allow changing of the instruction inside instrumentation callback
+    BX_INSTR_BEFORE_EXECUTION(BX_CPU_ID, i);
+    RIP += i->ilen();
+    BX_CPU_CALL_METHOD(i->execute, (i)); // might iterate repeat instruction
+    BX_CPU_THIS_PTR prev_rip = RIP; // commit new RIP
+    BX_INSTR_AFTER_EXECUTION(BX_CPU_ID, i);
+    BX_CPU_THIS_PTR icount++;
+
+    if (BX_CPU_THIS_PTR async_event) {
+      // clear stop trace magic indication that probably was set by repeat or branch32/64
+      BX_CPU_THIS_PTR async_event &= ~BX_ASYNC_EVENT_STOP_TRACE;
+      break;
+    }
+
+    if (++i == last) break;
+  }
+#endif // BX_SUPPORT_HANDLERS_CHAINING_SPEEDUPS
+}
+
+#endif
+
 bxICacheEntry_c* BX_CPU_C::getICacheEntry(void)
 {
   bx_address eipBiased = RIP + BX_CPU_THIS_PTR eipPageBias;
@@ -192,17 +233,19 @@ bxICacheEntry_c* BX_CPU_C::getICacheEntry(void)
     // iCache miss. No validated instruction with matching fetch parameters
     // is in the iCache.
     InstrICache_Increment(iCacheMisses);
-    serveICacheMiss(entry, (Bit32u) eipBiased, pAddr);
+    entry = serveICacheMiss(entry, (Bit32u) eipBiased, pAddr);
   }
 
   return entry;
 }
 
-void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat(bxInstruction_c *i, BxExecutePtr_tR execute)
+#define BX_REPEAT_TIME_UPDATE_INTERVAL 15
+
+void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat(bxInstruction_c *i, BxRepIterationPtr_tR execute)
 {
   // non repeated instruction
   if (! i->repUsedL()) {
-    BX_CPU_CALL_METHOD(execute, (i));
+    BX_CPU_CALL_REP_ITERATION(execute, (i));
     return;
   }
 
@@ -214,7 +257,7 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat(bxInstruction_c *i, BxExecutePtr_tR
   if (i->as64L()) {
     while(1) {
       if (RCX != 0) {
-        BX_CPU_CALL_METHOD(execute, (i));
+        BX_CPU_CALL_REP_ITERATION(execute, (i));
         BX_INSTR_REPEAT_ITERATION(BX_CPU_ID, i);
         RCX --;
       }
@@ -225,7 +268,10 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat(bxInstruction_c *i, BxExecutePtr_tR
 #endif
         break; // exit always if debugger enabled
 
-      BX_TICK1_IF_SINGLE_PROCESSOR();
+#if BX_SUPPORT_HANDLERS_CHAINING_SPEEDUPS || BX_SUPPORT_SMP
+      BX_CPU_THIS_PTR icount++;
+#endif
+      BX_SYNC_TIME_IF_SINGLE_PROCESSOR(BX_REPEAT_TIME_UPDATE_INTERVAL);
     }
   }
   else
@@ -233,7 +279,7 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat(bxInstruction_c *i, BxExecutePtr_tR
   if (i->as32L()) {
     while(1) {
       if (ECX != 0) {
-        BX_CPU_CALL_METHOD(execute, (i));
+        BX_CPU_CALL_REP_ITERATION(execute, (i));
         BX_INSTR_REPEAT_ITERATION(BX_CPU_ID, i);
         RCX = ECX - 1;
       }
@@ -244,14 +290,17 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat(bxInstruction_c *i, BxExecutePtr_tR
 #endif
         break; // exit always if debugger enabled
 
-      BX_TICK1_IF_SINGLE_PROCESSOR();
+#if BX_SUPPORT_HANDLERS_CHAINING_SPEEDUPS || BX_SUPPORT_SMP
+      BX_CPU_THIS_PTR icount++;
+#endif
+      BX_SYNC_TIME_IF_SINGLE_PROCESSOR(BX_REPEAT_TIME_UPDATE_INTERVAL);
     }
   }
   else  // 16bit addrsize
   {
     while(1) {
       if (CX != 0) {
-        BX_CPU_CALL_METHOD(execute, (i));
+        BX_CPU_CALL_REP_ITERATION(execute, (i));
         BX_INSTR_REPEAT_ITERATION(BX_CPU_ID, i);
         CX --;
       }
@@ -262,7 +311,10 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat(bxInstruction_c *i, BxExecutePtr_tR
 #endif
         break; // exit always if debugger enabled
 
-      BX_TICK1_IF_SINGLE_PROCESSOR();
+#if BX_SUPPORT_HANDLERS_CHAINING_SPEEDUPS || BX_SUPPORT_SMP
+      BX_CPU_THIS_PTR icount++;
+#endif
+      BX_SYNC_TIME_IF_SINGLE_PROCESSOR(BX_REPEAT_TIME_UPDATE_INTERVAL);
     }
   }
 
@@ -272,19 +324,17 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat(bxInstruction_c *i, BxExecutePtr_tR
 
   RIP = BX_CPU_THIS_PTR prev_rip; // repeat loop not done, restore RIP
 
-#if BX_SUPPORT_TRACE_CACHE
   // assert magic async_event to stop trace execution
   BX_CPU_THIS_PTR async_event |= BX_ASYNC_EVENT_STOP_TRACE;
-#endif
 }
 
-void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat_ZF(bxInstruction_c *i, BxExecutePtr_tR execute)
+void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat_ZF(bxInstruction_c *i, BxRepIterationPtr_tR execute)
 {
   unsigned rep = i->repUsedValue();
 
   // non repeated instruction
   if (! rep) {
-    BX_CPU_CALL_METHOD(execute, (i));
+    BX_CPU_CALL_REP_ITERATION(execute, (i));
     return;
   }
 
@@ -297,7 +347,7 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat_ZF(bxInstruction_c *i, BxExecutePtr
     if (i->as64L()) {
       while(1) {
         if (RCX != 0) {
-          BX_CPU_CALL_METHOD(execute, (i));
+          BX_CPU_CALL_REP_ITERATION(execute, (i));
           BX_INSTR_REPEAT_ITERATION(BX_CPU_ID, i);
           RCX --;
         }
@@ -308,7 +358,10 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat_ZF(bxInstruction_c *i, BxExecutePtr
 #endif
           break; // exit always if debugger enabled
 
-        BX_TICK1_IF_SINGLE_PROCESSOR();
+#if BX_SUPPORT_HANDLERS_CHAINING_SPEEDUPS || BX_SUPPORT_SMP
+        BX_CPU_THIS_PTR icount++;
+#endif
+        BX_SYNC_TIME_IF_SINGLE_PROCESSOR(BX_REPEAT_TIME_UPDATE_INTERVAL);
       }
     }
     else
@@ -316,7 +369,7 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat_ZF(bxInstruction_c *i, BxExecutePtr
     if (i->as32L()) {
       while(1) {
         if (ECX != 0) {
-          BX_CPU_CALL_METHOD(execute, (i));
+          BX_CPU_CALL_REP_ITERATION(execute, (i));
           BX_INSTR_REPEAT_ITERATION(BX_CPU_ID, i);
           RCX = ECX - 1;
         }
@@ -327,14 +380,17 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat_ZF(bxInstruction_c *i, BxExecutePtr
 #endif
           break; // exit always if debugger enabled
 
-        BX_TICK1_IF_SINGLE_PROCESSOR();
+#if BX_SUPPORT_HANDLERS_CHAINING_SPEEDUPS || BX_SUPPORT_SMP
+        BX_CPU_THIS_PTR icount++;
+#endif
+        BX_SYNC_TIME_IF_SINGLE_PROCESSOR(BX_REPEAT_TIME_UPDATE_INTERVAL);
       }
     }
     else  // 16bit addrsize
     {
       while(1) {
         if (CX != 0) {
-          BX_CPU_CALL_METHOD(execute, (i));
+          BX_CPU_CALL_REP_ITERATION(execute, (i));
           BX_INSTR_REPEAT_ITERATION(BX_CPU_ID, i);
           CX --;
         }
@@ -345,7 +401,10 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat_ZF(bxInstruction_c *i, BxExecutePtr
 #endif
           break; // exit always if debugger enabled
 
-        BX_TICK1_IF_SINGLE_PROCESSOR();
+#if BX_SUPPORT_HANDLERS_CHAINING_SPEEDUPS || BX_SUPPORT_SMP
+        BX_CPU_THIS_PTR icount++;
+#endif
+        BX_SYNC_TIME_IF_SINGLE_PROCESSOR(BX_REPEAT_TIME_UPDATE_INTERVAL);
       }
     }
   }
@@ -354,7 +413,7 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat_ZF(bxInstruction_c *i, BxExecutePtr
     if (i->as64L()) {
       while(1) {
         if (RCX != 0) {
-          BX_CPU_CALL_METHOD(execute, (i));
+          BX_CPU_CALL_REP_ITERATION(execute, (i));
           BX_INSTR_REPEAT_ITERATION(BX_CPU_ID, i);
           RCX --;
         }
@@ -365,7 +424,10 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat_ZF(bxInstruction_c *i, BxExecutePtr
 #endif
           break; // exit always if debugger enabled
 
-        BX_TICK1_IF_SINGLE_PROCESSOR();
+#if BX_SUPPORT_HANDLERS_CHAINING_SPEEDUPS || BX_SUPPORT_SMP
+        BX_CPU_THIS_PTR icount++;
+#endif
+        BX_SYNC_TIME_IF_SINGLE_PROCESSOR(BX_REPEAT_TIME_UPDATE_INTERVAL);
       }
     }
     else
@@ -373,7 +435,7 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat_ZF(bxInstruction_c *i, BxExecutePtr
     if (i->as32L()) {
       while(1) {
         if (ECX != 0) {
-          BX_CPU_CALL_METHOD(execute, (i));
+          BX_CPU_CALL_REP_ITERATION(execute, (i));
           BX_INSTR_REPEAT_ITERATION(BX_CPU_ID, i);
           RCX = ECX - 1;
         }
@@ -384,14 +446,17 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat_ZF(bxInstruction_c *i, BxExecutePtr
 #endif
           break; // exit always if debugger enabled
 
-        BX_TICK1_IF_SINGLE_PROCESSOR();
+#if BX_SUPPORT_HANDLERS_CHAINING_SPEEDUPS || BX_SUPPORT_SMP
+        BX_CPU_THIS_PTR icount++;
+#endif
+        BX_SYNC_TIME_IF_SINGLE_PROCESSOR(BX_REPEAT_TIME_UPDATE_INTERVAL);
       }
     }
     else  // 16bit addrsize
     {
       while(1) {
         if (CX != 0) {
-          BX_CPU_CALL_METHOD(execute, (i));
+          BX_CPU_CALL_REP_ITERATION(execute, (i));
           BX_INSTR_REPEAT_ITERATION(BX_CPU_ID, i);
           CX --;
         }
@@ -402,7 +467,10 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat_ZF(bxInstruction_c *i, BxExecutePtr
 #endif
           break; // exit always if debugger enabled
 
-        BX_TICK1_IF_SINGLE_PROCESSOR();
+#if BX_SUPPORT_HANDLERS_CHAINING_SPEEDUPS || BX_SUPPORT_SMP
+        BX_CPU_THIS_PTR icount++;
+#endif
+        BX_SYNC_TIME_IF_SINGLE_PROCESSOR(BX_REPEAT_TIME_UPDATE_INTERVAL);
       }
     }
   }
@@ -413,10 +481,8 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat_ZF(bxInstruction_c *i, BxExecutePtr
 
   RIP = BX_CPU_THIS_PTR prev_rip; // repeat loop not done, restore RIP
 
-#if BX_SUPPORT_TRACE_CACHE
   // assert magic async_event to stop trace execution
   BX_CPU_THIS_PTR async_event |= BX_ASYNC_EVENT_STOP_TRACE;
-#endif
 }
 
 unsigned BX_CPU_C::handleAsyncEvent(void)
@@ -429,8 +495,10 @@ unsigned BX_CPU_C::handleAsyncEvent(void)
     // an interrupt wakes up the CPU.
     while (1)
     {
-      if ((BX_CPU_INTR && (BX_CPU_THIS_PTR get_IF() || 
-          (BX_CPU_THIS_PTR activity_state == BX_ACTIVITY_STATE_MWAIT_IF))) ||
+      if ((BX_CPU_INTR && (BX_CPU_THIS_PTR get_IF() || BX_CPU_THIS_PTR activity_state == BX_ACTIVITY_STATE_MWAIT_IF)) ||
+#if BX_SUPPORT_VMX >= 2
+           BX_CPU_THIS_PTR pending_vmx_timer_expired ||
+#endif
            BX_CPU_THIS_PTR pending_NMI || BX_CPU_THIS_PTR pending_SMI || BX_CPU_THIS_PTR pending_INIT)
       {
         // interrupt ends the HALT condition
@@ -513,20 +581,44 @@ unsigned BX_CPU_C::handleAsyncEvent(void)
     }
 #endif
     // reset will clear pending INIT
-    BX_CPU_THIS_PTR reset(BX_RESET_SOFTWARE);
+    reset(BX_RESET_SOFTWARE);
+
+#if BX_SUPPORT_SMP
+    if (BX_SMP_PROCESSORS > 1) {
+      // if HALT condition remains, return so other CPUs have a chance
+      if (BX_CPU_THIS_PTR activity_state) {
+#if BX_DEBUGGER
+        BX_CPU_THIS_PTR stop_reason = STOP_CPU_HALTED;
+#endif
+        return 1; // Return to caller of cpu_loop.
+      }
+    }
+#endif
   }
 
   // Priority 4: Traps on Previous Instruction
   //   Breakpoints
   //   Debug Trap Exceptions (TF flag set or data/IO breakpoint)
-  if (BX_CPU_THIS_PTR debug_trap &&
-       !(BX_CPU_THIS_PTR inhibit_mask & BX_INHIBIT_DEBUG_SHADOW))
-  {
+  if (! (BX_CPU_THIS_PTR inhibit_mask & BX_INHIBIT_DEBUG_SHADOW)) {
     // A trap may be inhibited on this boundary due to an instruction
     // which loaded SS.  If so we clear the inhibit_mask below
     // and don't execute this code until the next boundary.
-    exception(BX_DB_EXCEPTION, 0); // no error, not interrupt
+#if BX_X86_DEBUGGER
+    code_breakpoint_match(get_laddr(BX_SEG_REG_CS, BX_CPU_THIS_PTR prev_rip));
+#endif
+    if (BX_CPU_THIS_PTR debug_trap)
+      exception(BX_DB_EXCEPTION, 0); // no error, not interrupt
   }
+  
+  // Priority 4.5: VMX Preemption Timer Expired. FIXME: is it a kind of external interrupt?
+#if BX_SUPPORT_VMX >= 2
+  if (BX_CPU_THIS_PTR in_vmx_guest) {
+    if (BX_CPU_THIS_PTR pending_vmx_timer_expired) {
+      BX_CPU_THIS_PTR pending_vmx_timer_expired = 0;
+      VMexit_PreemptionTimerExpired();
+    }
+  }
+#endif
 
   // Priority 5: External Interrupts
   //   NMI Interrupts
@@ -604,6 +696,14 @@ unsigned BX_CPU_C::handleAsyncEvent(void)
     DEV_dma_raise_hlda();
   }
 
+  if (BX_CPU_THIS_PTR get_TF())
+  {
+    // TF is set before execution of next instruction.  Schedule
+    // a debug trap (#DB) after execution.  After completion of
+    // next instruction, the code above will invoke the trap.
+    BX_CPU_THIS_PTR debug_trap |= BX_DEBUG_SINGLE_STEP_BIT;
+  }
+
   // Priority 6: Faults from fetching next instruction
   //   Code breakpoint fault
   //   Code segment limit violation (priority 7 on 486/Pentium)
@@ -628,38 +728,6 @@ unsigned BX_CPU_C::handleAsyncEvent(void)
   //   Alignment check
   // (handled by rest of the code)
 
-  // Now we can handle things which are synchronous to instruction
-  // execution.
-  if (BX_CPU_THIS_PTR get_RF()) {
-    BX_CPU_THIS_PTR clear_RF();
-  }
-#if BX_X86_DEBUGGER
-  else {
-    // only bother comparing if any breakpoints enabled and
-    // debug events are not inhibited on this boundary.
-    if (! (BX_CPU_THIS_PTR inhibit_mask & BX_INHIBIT_DEBUG_SHADOW) && ! BX_CPU_THIS_PTR in_repeat) {
-      if (BX_CPU_THIS_PTR dr7 & 0x000000ff) {
-        bx_address iaddr = get_laddr(BX_SEG_REG_CS, BX_CPU_THIS_PTR prev_rip);
-        Bit32u dr6_bits = hwdebug_compare(iaddr, 1, BX_HWDebugInstruction, BX_HWDebugInstruction);
-        if (dr6_bits) {
-          // Add to the list of debug events thus far.
-          BX_CPU_THIS_PTR debug_trap |= dr6_bits;
-          BX_ERROR(("#DB: x86 code breakpoint catched"));
-          exception(BX_DB_EXCEPTION, 0); // no error, not interrupt
-        }
-      }
-    }
-  }
-#endif
-
-  if (BX_CPU_THIS_PTR get_TF())
-  {
-    // TF is set before execution of next instruction.  Schedule
-    // a debug trap (#DB) after execution.  After completion of
-    // next instruction, the code above will invoke the trap.
-    BX_CPU_THIS_PTR debug_trap |= BX_DEBUG_SINGLE_STEP_BIT;
-  }
-
   if (!((BX_CPU_INTR && BX_CPU_THIS_PTR get_IF()) ||
         BX_CPU_THIS_PTR debug_trap ||
 //      BX_CPU_THIS_PTR get_TF() // implies debug_trap is set
@@ -667,13 +735,12 @@ unsigned BX_CPU_C::handleAsyncEvent(void)
 #if BX_SUPPORT_VMX
      || BX_CPU_THIS_PTR vmx_interrupt_window || BX_CPU_THIS_PTR inhibit_mask
 #endif
+#if BX_SUPPORT_VMX >= 2
+     || BX_CPU_THIS_PTR pending_vmx_timer_expired
+#endif
 #if BX_X86_DEBUGGER
-       // any debug code breakpoint is set
-     || ((BX_CPU_THIS_PTR dr7 & 0xff) &&
-           (((BX_CPU_THIS_PTR dr7 >> 16) & 3) == 0 ||
-            ((BX_CPU_THIS_PTR dr7 >> 20) & 3) == 0 ||
-            ((BX_CPU_THIS_PTR dr7 >> 24) & 3) == 0 ||
-            ((BX_CPU_THIS_PTR dr7 >> 28) & 3) == 0))
+     // a debug code breakpoint is set in current page
+     || BX_CPU_THIS_PTR codebp
 #endif
         ))
     BX_CPU_THIS_PTR async_event = 0;
@@ -732,6 +799,29 @@ void BX_CPU_C::prefetch(void)
     }
   }
 
+#if BX_X86_DEBUGGER
+  if (hwbreakpoint_check(laddr, BX_HWDebugInstruction, BX_HWDebugInstruction)) {
+    BX_CPU_THIS_PTR async_event = 1;
+    BX_CPU_THIS_PTR codebp = 1;
+    if (! (BX_CPU_THIS_PTR inhibit_mask & BX_INHIBIT_DEBUG_SHADOW)) {
+       // The next instruction could already hit a code breakpoint but
+       // async_event won't take effect immediatelly.
+       // Check if the next executing instruction hits code breakpoint
+
+       // check only if not fetching page cross instruction
+       // this check is 32-bit wrap safe as well
+       if (EIP == (Bit32u) BX_CPU_THIS_PTR prev_rip) {
+         if (code_breakpoint_match(laddr)) exception(BX_DB_EXCEPTION, 0);
+       }
+    }
+  }
+  else {
+    BX_CPU_THIS_PTR codebp = 0;
+  }
+#endif
+
+  BX_CPU_THIS_PTR clear_RF();
+
   bx_address lpf = LPFOf(laddr);
   unsigned TLB_index = BX_TLB_INDEX_OF(lpf, 0);
   bx_TLB_entry *tlbEntry = &BX_CPU_THIS_PTR TLB.entry[TLB_index];
@@ -742,7 +832,7 @@ void BX_CPU_C::prefetch(void)
     fetchPtr = (Bit8u*) tlbEntry->hostPageAddr;
   }  
   else {
-    bx_phy_address pAddr = translate_linear(laddr, CPL, BX_EXECUTE);
+    bx_phy_address pAddr = translate_linear(laddr, USER_PL, BX_EXECUTE);
     BX_CPU_THIS_PTR pAddrPage = PPFOf(pAddr);
   }
 
@@ -813,7 +903,8 @@ bx_bool BX_CPU_C::dbg_instruction_epilog(void)
   bx_address debug_eip = RIP;
   Bit16u cs = BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value;
 
-  BX_CPU_THIS_PTR guard_found.icount++;
+  BX_CPU_THIS_PTR icount++;
+
   BX_CPU_THIS_PTR guard_found.cs  = cs;
   BX_CPU_THIS_PTR guard_found.eip = debug_eip;
   BX_CPU_THIS_PTR guard_found.laddr = BX_CPU_THIS_PTR get_laddr(BX_SEG_REG_CS, debug_eip);
@@ -849,6 +940,13 @@ bx_bool BX_CPU_C::dbg_instruction_epilog(void)
     return(1); // on a breakpoint
   }
 
+  // see if debugger requesting icount guard 
+  if (bx_guard.guard_for & BX_DBG_GUARD_ICOUNT) {
+    if (BX_CPU_THIS_PTR icount >= BX_CPU_THIS_PTR guard_found.icount_max) {
+      return(1);
+    }
+  }
+
   // convenient point to see if user requested debug break or typed Ctrl-C
   if (bx_guard.interrupt_requested) {
     return(1);
@@ -873,7 +971,6 @@ bx_bool BX_CPU_C::dbg_instruction_epilog(void)
         {
           BX_CPU_THIS_PTR guard_found.guard_found = BX_DBG_GUARD_IADDR_VIR;
           BX_CPU_THIS_PTR guard_found.iaddr_index = n;
-          BX_CPU_THIS_PTR guard_found.time_tick = tt;
           return(1); // on a breakpoint
         }
       }
@@ -887,7 +984,6 @@ bx_bool BX_CPU_C::dbg_instruction_epilog(void)
         {
           BX_CPU_THIS_PTR guard_found.guard_found = BX_DBG_GUARD_IADDR_LIN;
           BX_CPU_THIS_PTR guard_found.iaddr_index = n;
-          BX_CPU_THIS_PTR guard_found.time_tick = tt;
           return(1); // on a breakpoint
         }
       }
@@ -903,7 +999,6 @@ bx_bool BX_CPU_C::dbg_instruction_epilog(void)
           {
             BX_CPU_THIS_PTR guard_found.guard_found = BX_DBG_GUARD_IADDR_PHY;
             BX_CPU_THIS_PTR guard_found.iaddr_index = n;
-            BX_CPU_THIS_PTR guard_found.time_tick = tt;
             return(1); // on a breakpoint
           }
         }
