@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: icache.h,v 1.61 2011/01/23 17:21:34 sshwarts Exp $
+// $Id: icache.h 10613 2011-08-21 16:44:02Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2007-2011 Stanislav Shwartsman
@@ -104,22 +104,15 @@ extern bxPageWriteStampTable pageWriteStampTable;
 #define BxICacheEntries (64 * 1024)  // Must be a power of 2.
 #define BxICacheMemPool (384 * 1024)
 
-#if BX_SUPPORT_TRACE_CACHE
-  #define BX_MAX_TRACE_LENGTH 32
-#endif
+#define BX_MAX_TRACE_LENGTH 32
 
 struct bxICacheEntry_c
 {
   bx_phy_address pAddr; // Physical address of the instruction
   Bit32u traceMask;
 
-#if BX_SUPPORT_TRACE_CACHE
   Bit32u tlen;          // Trace length in instructions
   bxInstruction_c *i;
-#else
-  // ... define as array of 1 to simplify merge with trace cache code
-  bxInstruction_c i[1];
-#endif
 };
 
 #define BX_ICACHE_INVALID_PHY_ADDRESS (bx_phy_address(-1))
@@ -127,7 +120,6 @@ struct bxICacheEntry_c
 class BOCHSAPI bxICache_c {
 public:
   bxICacheEntry_c entry[BxICacheEntries];
-#if BX_SUPPORT_TRACE_CACHE
   bxInstruction_c mpool[BxICacheMemPool];
   unsigned mpindex;
 
@@ -137,7 +129,13 @@ public:
     bxICacheEntry_c *e; // Pointer to icache entry
   } pageSplitIndex[BX_ICACHE_PAGE_SPLIT_ENTRIES];
   int nextPageSplitIndex;
-#endif
+
+#define BX_ICACHE_VICTIM_ENTRIES 8 /* must be power of two */
+  struct bxVictimCacheEntry {
+    Bit32u fetchModeMask;
+    bxICacheEntry_c vc_entry;
+  } victimCache[BX_ICACHE_VICTIM_ENTRIES];
+  int nextVictimCacheIndex;
 
 public:
   bxICache_c() { flushICacheEntries(); }
@@ -148,10 +146,10 @@ public:
     return ((pAddr) & (BxICacheEntries-1)) ^ fetchModeMask;
   }
 
-#if BX_SUPPORT_TRACE_CACHE
   BX_CPP_INLINE void alloc_trace(bxICacheEntry_c *e)
   {
-    if (mpindex + BX_MAX_TRACE_LENGTH > BxICacheMemPool) {
+    // took +1 garbend for instruction chaining speedup (end-of-trace opcode)
+    if ((mpindex + BX_MAX_TRACE_LENGTH + 1) > BxICacheMemPool) {
       flushICacheEntries();
     }
     e->i = &mpool[mpindex];
@@ -162,7 +160,7 @@ public:
 
   BX_CPP_INLINE void commit_page_split_trace(bx_phy_address paddr, bxICacheEntry_c *entry)
   {
-    mpindex++;  // commit_trace(1)
+    mpindex += entry->tlen;
 
     // register page split entry
     if (pageSplitIndex[nextPageSplitIndex].ppf != BX_ICACHE_INVALID_PHY_ADDRESS)
@@ -173,11 +171,30 @@ public:
 
     nextPageSplitIndex = (nextPageSplitIndex+1) & (BX_ICACHE_PAGE_SPLIT_ENTRIES-1);
   }
-#endif
+
+  BX_CPP_INLINE bxICacheEntry_c *lookup_victim_cache(bx_phy_address pAddr, Bit32u fetchModeMask)
+  {
+    for (int i=0; i < BX_ICACHE_VICTIM_ENTRIES;i++) {
+      bxVictimCacheEntry *e = &victimCache[i];
+      if (e->vc_entry.pAddr == pAddr && e->fetchModeMask == fetchModeMask) {
+        return &e->vc_entry;
+      }
+    }
+
+    return NULL;
+  }
+
+  BX_CPP_INLINE void victim_entry(bxICacheEntry_c *entry, Bit32u fetchModeMask)
+  {
+    if (entry->pAddr != BX_ICACHE_INVALID_PHY_ADDRESS) {
+      victimCache[nextVictimCacheIndex].fetchModeMask = fetchModeMask;
+      victimCache[nextVictimCacheIndex].vc_entry = *entry;
+      nextVictimCacheIndex = (nextVictimCacheIndex+1) & (BX_ICACHE_VICTIM_ENTRIES-1);
+    }
+  }
 
   BX_CPP_INLINE void handleSMC(bx_phy_address pAddr, Bit32u mask);
 
-  BX_CPP_INLINE void purgeICacheEntries(void);
   BX_CPP_INLINE void flushICacheEntries(void);
 
   BX_CPP_INLINE bxICacheEntry_c* get_entry(bx_phy_address pAddr, unsigned fetchModeMask)
@@ -197,32 +214,37 @@ BX_CPP_INLINE void bxICache_c::flushICacheEntries(void)
     e->traceMask = 0;
   }
 
-#if BX_SUPPORT_TRACE_CACHE
+  nextPageSplitIndex = 0;
   for (i=0;i<BX_ICACHE_PAGE_SPLIT_ENTRIES;i++)
     pageSplitIndex[i].ppf = BX_ICACHE_INVALID_PHY_ADDRESS;
 
-  nextPageSplitIndex = 0;
+  nextVictimCacheIndex = 0;
+  for (i=0;i<BX_ICACHE_VICTIM_ENTRIES;i++)
+    victimCache[i].vc_entry.pAddr = BX_ICACHE_INVALID_PHY_ADDRESS;
+
   mpindex = 0;
-#endif
 }
 
 BX_CPP_INLINE void bxICache_c::handleSMC(bx_phy_address pAddr, Bit32u mask)
 {
-  // TODO: invalidate only entries in same page as pAddr
-
   pAddr = LPFOf(pAddr);
 
-#if BX_SUPPORT_TRACE_CACHE
+  unsigned i;
+
   if (mask & 0x1) {
     // the store touched 1st cache line in the page, check for
     // page split traces to invalidate.
-    for (unsigned i=0;i<BX_ICACHE_PAGE_SPLIT_ENTRIES;i++) {
+    for (i=0;i<BX_ICACHE_PAGE_SPLIT_ENTRIES;i++) {
       if (pAddr == pageSplitIndex[i].ppf) {
         pageSplitIndex[i].ppf = BX_ICACHE_INVALID_PHY_ADDRESS;
       }
     }
   }
-#endif
+
+  for (i=0;i < BX_ICACHE_VICTIM_ENTRIES; i++) {
+    bxVictimCacheEntry *e = &victimCache[i];
+    e->vc_entry.pAddr = BX_ICACHE_INVALID_PHY_ADDRESS;
+  }
 
   bxICacheEntry_c *e = get_entry(pAddr, 0);
 
