@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: io.cc,v 1.81 2010/03/14 15:51:26 sshwarts Exp $
+// $Id: io.cc 10888 2011-12-29 20:52:44Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2009  The Bochs Project
+//  Copyright (C) 2001-2011  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -27,19 +27,11 @@
 
 #include "iodev/iodev.h"
 
-#if BX_SUPPORT_X86_64==0
-// Make life easier for merging cpu64 and cpu32 code.
-#define RDI EDI
-#define RSI ESI
-#define RAX EAX
-#define RCX ECX
-#endif
-
 //
 // Repeat Speedups methods
 //
 
-#if BX_SupportRepeatSpeedups
+#if BX_SUPPORT_REPEAT_SPEEDUPS
 Bit32u BX_CPU_C::FastRepINSW(bxInstruction_c *i, bx_address dstOff, Bit16u port, Bit32u wordCount)
 {
   Bit32u wordsFitDst;
@@ -194,7 +186,7 @@ Bit32u BX_CPU_C::FastRepOUTSW(bxInstruction_c *i, unsigned srcSeg, bx_address sr
 // REP INS methods
 //
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_INSB_YbDX(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_INSB_YbDX(bxInstruction_c *i)
 {
   if (! allow_io(i, DX, 1)) {
     BX_DEBUG(("INSB_YbDX: I/O access not allowed !"));
@@ -214,6 +206,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_INSB_YbDX(bxInstruction_c *i)
   else {
     BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::INSB16_YbDX);
   }
+
+  BX_NEXT_INSTR(i);
 }
 
 // 16-bit address size
@@ -271,7 +265,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSB64_YbDX(bxInstruction_c *i)
 
 #endif
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_INSW_YwDX(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_INSW_YwDX(bxInstruction_c *i)
 {
   if (! allow_io(i, DX, 2)) {
     BX_DEBUG(("INSW_YwDX: I/O access not allowed !"));
@@ -291,6 +285,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_INSW_YwDX(bxInstruction_c *i)
   else {
     BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::INSW16_YwDX);
   }
+
+  BX_NEXT_INSTR(i);
 }
 
 // 16-bit operand size, 16-bit address size
@@ -316,7 +312,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSW32_YwDX(bxInstruction_c *i)
   Bit32u edi = EDI;
   unsigned incr = 2;
 
-#if (BX_SupportRepeatSpeedups) && (BX_DEBUGGER == 0)
+#if (BX_SUPPORT_REPEAT_SPEEDUPS) && (BX_DEBUGGER == 0)
   /* If conditions are right, we can transfer IO to physical memory
    * in a batch, rather than one instruction at a time.
    */
@@ -380,7 +376,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSW64_YwDX(bxInstruction_c *i)
 
 #endif
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_INSD_YdDX(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_INSD_YdDX(bxInstruction_c *i)
 {
   if (! allow_io(i, DX, 4)) {
     BX_DEBUG(("INSD_YdDX: I/O access not allowed !"));
@@ -400,6 +396,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_INSD_YdDX(bxInstruction_c *i)
   else {
     BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::INSD16_YdDX);
   }
+
+  BX_NEXT_INSTR(i);
 }
 
 // 32-bit operand size, 16-bit address size
@@ -458,7 +456,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSD64_YdDX(bxInstruction_c *i)
 // REP OUTS methods
 //
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_OUTSB_DXXb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_OUTSB_DXXb(bxInstruction_c *i)
 {
   if (! allow_io(i, DX, 1)) {
     BX_DEBUG(("OUTSB_DXXb: I/O access not allowed !"));
@@ -478,6 +476,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_OUTSB_DXXb(bxInstruction_c *i)
   else {
     BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::OUTSB16_DXXb);
   }
+
+  BX_NEXT_INSTR(i);
 }
 
 // 16-bit address size
@@ -520,7 +520,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUTSB64_DXXb(bxInstruction_c *i)
 
 #endif
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_OUTSW_DXXw(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_OUTSW_DXXw(bxInstruction_c *i)
 {
   if (! allow_io(i, DX, 2)) {
     BX_DEBUG(("OUTSW_DXXw: I/O access not allowed !"));
@@ -540,6 +540,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_OUTSW_DXXw(bxInstruction_c *i)
   else {
     BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::OUTSW16_DXXw);
   }
+
+  BX_NEXT_INSTR(i);
 }
 
 // 16-bit operand size, 16-bit address size
@@ -561,7 +563,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUTSW32_DXXw(bxInstruction_c *i)
   Bit32u esi = ESI;
   unsigned incr = 2;
 
-#if (BX_SupportRepeatSpeedups) && (BX_DEBUGGER == 0)
+#if (BX_SUPPORT_REPEAT_SPEEDUPS) && (BX_DEBUGGER == 0)
   /* If conditions are right, we can transfer IO to physical memory
    * in a batch, rather than one instruction at a time.
    */
@@ -609,7 +611,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUTSW64_DXXw(bxInstruction_c *i)
 
 #endif
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_OUTSD_DXXd(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_OUTSD_DXXd(bxInstruction_c *i)
 {
   if (! allow_io(i, DX, 4)) {
     BX_DEBUG(("OUTSD_DXXd: I/O access not allowed !"));
@@ -629,6 +631,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_OUTSD_DXXd(bxInstruction_c *i)
   else {
     BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::OUTSD16_DXXd);
   }
+
+  BX_NEXT_INSTR(i);
 }
 
 // 32-bit operand size, 16-bit address size
@@ -675,7 +679,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUTSD64_DXXd(bxInstruction_c *i)
 // non repeatable IN/OUT methods
 //
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_ALIb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_ALIb(bxInstruction_c *i)
 {
   unsigned port = i->Ib();
 
@@ -685,9 +689,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_ALIb(bxInstruction_c *i)
   }
 
   AL = BX_INP(port, 1);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_AXIb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_AXIb(bxInstruction_c *i)
 {
   unsigned port = i->Ib();
 
@@ -697,9 +703,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_AXIb(bxInstruction_c *i)
   }
 
   AX = BX_INP(port, 2);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_EAXIb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_EAXIb(bxInstruction_c *i)
 {
   unsigned port = i->Ib();
 
@@ -709,9 +717,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_EAXIb(bxInstruction_c *i)
   }
 
   RAX = BX_INP(port, 4);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_IbAL(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_IbAL(bxInstruction_c *i)
 {
   unsigned port = i->Ib();
 
@@ -721,9 +731,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_IbAL(bxInstruction_c *i)
   }
 
   BX_OUTP(port, AL, 1);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_IbAX(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_IbAX(bxInstruction_c *i)
 {
   unsigned port = i->Ib();
 
@@ -733,9 +745,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_IbAX(bxInstruction_c *i)
   }
 
   BX_OUTP(port, AX, 2);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_IbEAX(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_IbEAX(bxInstruction_c *i)
 {
   unsigned port = i->Ib();
 
@@ -745,9 +759,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_IbEAX(bxInstruction_c *i)
   }
 
   BX_OUTP(port, EAX, 4);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_ALDX(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_ALDX(bxInstruction_c *i)
 {
   unsigned port = DX;
 
@@ -757,9 +773,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_ALDX(bxInstruction_c *i)
   }
 
   AL = BX_INP(port, 1);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_AXDX(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_AXDX(bxInstruction_c *i)
 {
   unsigned port = DX;
 
@@ -769,9 +787,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_AXDX(bxInstruction_c *i)
   }
 
   AX = BX_INP(port, 2);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_EAXDX(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_EAXDX(bxInstruction_c *i)
 {
   unsigned port = DX;
 
@@ -781,9 +801,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_EAXDX(bxInstruction_c *i)
   }
 
   RAX = BX_INP(port, 4);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_DXAL(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_DXAL(bxInstruction_c *i)
 {
   unsigned port = DX;
 
@@ -793,9 +815,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_DXAL(bxInstruction_c *i)
   }
 
   BX_OUTP(port, AL, 1);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_DXAX(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_DXAX(bxInstruction_c *i)
 {
   unsigned port = DX;
 
@@ -805,9 +829,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_DXAX(bxInstruction_c *i)
   }
 
   BX_OUTP(port, AX, 2);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_DXEAX(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_DXEAX(bxInstruction_c *i)
 {
   unsigned port = DX;
 
@@ -817,6 +843,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_DXEAX(bxInstruction_c *i)
   }
 
   BX_OUTP(port, EAX, 4);
+
+  BX_NEXT_INSTR(i);
 }
 
 bx_bool BX_CPP_AttrRegparmN(3) BX_CPU_C::allow_io(bxInstruction_c *i, Bit16u port, unsigned len)
@@ -825,7 +853,7 @@ bx_bool BX_CPP_AttrRegparmN(3) BX_CPU_C::allow_io(bxInstruction_c *i, Bit16u por
    * Otherwise, must check the IO permission map on >286.
    * On the 286, there is no IO permissions map */
 
-  if (BX_CPU_THIS_PTR cr0.get_PE() && (BX_CPU_THIS_PTR get_VM() || (CPL>BX_CPU_THIS_PTR get_IOPL())))
+  if (BX_CPU_THIS_PTR cr0.get_PE() && (BX_CPU_THIS_PTR get_VM() || (CPL > BX_CPU_THIS_PTR get_IOPL())))
   {
     if (BX_CPU_THIS_PTR tr.cache.valid==0 ||
        (BX_CPU_THIS_PTR tr.cache.type != BX_SYS_SEGMENT_AVAIL_386_TSS &&
@@ -857,10 +885,11 @@ bx_bool BX_CPP_AttrRegparmN(3) BX_CPU_C::allow_io(bxInstruction_c *i, Bit16u por
   }
 
 #if BX_SUPPORT_VMX
-  VMexit_IO(i, port, len);
+  if (BX_CPU_THIS_PTR in_vmx_guest)
+    VMexit_IO(i, port, len);
 #endif
 
-#if BX_X86_DEBUGGER
+#if BX_X86_DEBUGGER && BX_CPU_LEVEL >= 5
   iobreakpoint_match(port, len);
 #endif
 
