@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: logical8.cc,v 1.45 2011/01/25 20:59:26 sshwarts Exp $
+// $Id: logical8.cc 10451 2011-07-06 20:01:18Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2009  The Bochs Project
+//  Copyright (C) 2001-2011  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,7 +24,7 @@
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EbGbM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EbGbM(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
@@ -36,9 +36,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EbGbM(bxInstruction_c *i)
   write_RMW_virtual_byte(op1);
 
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_GbEbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_GbEbR(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
@@ -48,18 +50,21 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_GbEbR(bxInstruction_c *i)
   BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), op1);
 
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_ALIb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_ALIb(bxInstruction_c *i)
 {
-  Bit8u op1 = AL;
-  op1 ^= i->Ib();
-  AL = op1;
+  Bit8u op_8 = AL ^ i->Ib();
+  AL = op_8;
 
-  SET_FLAGS_OSZAPC_LOGIC_8(op1);
+  SET_FLAGS_OSZAPC_LOGIC_8(op_8);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EbIbM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EbIbM(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib();
 
@@ -70,9 +75,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EbIbM(bxInstruction_c *i)
   write_RMW_virtual_byte(op1);
 
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EbIbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EbIbR(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib();
 
@@ -81,9 +88,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EbIbR(bxInstruction_c *i)
   BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), op1);
 
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EbIbM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EbIbM(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib();
 
@@ -94,9 +103,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EbIbM(bxInstruction_c *i)
   write_RMW_virtual_byte(op1);
 
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EbIbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EbIbR(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib();
 
@@ -105,25 +116,31 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EbIbR(bxInstruction_c *i)
   BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), op1);
 
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::NOT_EbM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::NOT_EbM(bxInstruction_c *i)
 {
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
   Bit8u op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
   op1_8 = ~op1_8;
   write_RMW_virtual_byte(op1_8);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::NOT_EbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::NOT_EbR(bxInstruction_c *i)
 {
   Bit8u op1_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
   op1_8 = ~op1_8;
   BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), op1_8);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EbGbM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EbGbM(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
@@ -135,9 +152,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EbGbM(bxInstruction_c *i)
   write_RMW_virtual_byte(op1);
 
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_GbEbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_GbEbR(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
@@ -147,21 +166,21 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_GbEbR(bxInstruction_c *i)
   BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), op1);
 
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_ALIb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_ALIb(bxInstruction_c *i)
 {
-  Bit8u op1, op2;
+  Bit8u op_8 = AL | i->Ib();
+  AL = op_8;
 
-  op1 = AL;
-  op2 = i->Ib();
-  op1 |= op2;
-  AL = op1;
+  SET_FLAGS_OSZAPC_LOGIC_8(op_8);
 
-  SET_FLAGS_OSZAPC_LOGIC_8(op1);
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EbGbM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EbGbM(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
@@ -173,9 +192,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EbGbM(bxInstruction_c *i)
   write_RMW_virtual_byte(op1);
 
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_GbEbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_GbEbR(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
@@ -185,21 +206,21 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_GbEbR(bxInstruction_c *i)
   BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), op1);
 
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_ALIb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_ALIb(bxInstruction_c *i)
 {
-  Bit8u op1, op2;
+  Bit8u op_8 = AL & i->Ib();
+  AL = op_8;
 
-  op1 = AL;
-  op2 = i->Ib();
-  op1 &= op2;
-  AL = op1;
+  SET_FLAGS_OSZAPC_LOGIC_8(op_8);
 
-  SET_FLAGS_OSZAPC_LOGIC_8(op1);
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EbIbM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EbIbM(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib();
 
@@ -210,9 +231,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EbIbM(bxInstruction_c *i)
   write_RMW_virtual_byte(op1);
 
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EbIbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EbIbR(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib();
 
@@ -221,9 +244,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EbIbR(bxInstruction_c *i)
   BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), op1);
 
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EbGbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EbGbR(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
@@ -232,22 +257,24 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EbGbR(bxInstruction_c *i)
   op1 &= op2;
 
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_ALIb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_ALIb(bxInstruction_c *i)
 {
-  Bit8u op1, op2;
+  Bit8u op_8 = AL & i->Ib();
 
-  op1 = AL;
-  op2 = i->Ib();
-  op1 &= op2;
+  SET_FLAGS_OSZAPC_LOGIC_8(op_8);
 
-  SET_FLAGS_OSZAPC_LOGIC_8(op1);
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EbIbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EbIbR(bxInstruction_c *i)
 {
   Bit8u op1 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
   op1 &= i->Ib();
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
+
+  BX_NEXT_INSTR(i);
 }
