@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: msr.cc,v 1.50 2010/10/07 20:40:01 sshwarts Exp $
+// $Id: msr.cc 10906 2011-12-31 12:38:46Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2008-2010 Stanislav Shwartsman
+//   Copyright (c) 2008-2011 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -26,26 +26,19 @@
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-#if BX_SUPPORT_X86_64==0
-// Make life easier for merging code.
-#define RAX EAX
-#define RDX EDX
-#endif
-
 #if BX_CPU_LEVEL >= 5
 bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
 {
   Bit64u val64 = 0;
 
-  if ((index & 0x3FFFFFFF) >= BX_MSR_MAX_INDEX)
-    return 0;
-
-#if BX_SUPPORT_X2APIC
-  if (index >= 0x800 && index <= 0xBFF) {
-    if (BX_CPU_THIS_PTR msr.apicbase & 0x400)  // X2APIC mode
-      return BX_CPU_THIS_PTR lapic.read_x2apic(index, msr);
-    else
-      return 0;
+#if BX_CPU_LEVEL >= 6
+  if (bx_cpuid_support_x2apic()) {
+    if (index >= 0x800 && index <= 0xBFF) {
+      if (BX_CPU_THIS_PTR msr.apicbase & 0x400)  // X2APIC mode
+        return BX_CPU_THIS_PTR lapic.read_x2apic(index, msr);
+      else
+        return 0;
+    }
   }
 #endif
 
@@ -54,30 +47,24 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
 #if BX_CPU_LEVEL >= 6
     case BX_MSR_SYSENTER_CS:
       if (! bx_cpuid_support_sep()) {
-        // failed to find the MSR, could #GP or ignore it silently
         BX_ERROR(("RDMSR MSR_SYSENTER_CS: SYSENTER/SYSEXIT feature not enabled !"));
-        if (! BX_CPU_THIS_PTR ignore_bad_msrs)
-          return 0; // will result in #GP fault due to unknown MSR
+        return handle_unknown_rdmsr(index, msr);
       }
       val64 = BX_CPU_THIS_PTR msr.sysenter_cs_msr;
       break;
 
     case BX_MSR_SYSENTER_ESP:
       if (! bx_cpuid_support_sep()) {
-        // failed to find the MSR, could #GP or ignore it silently
         BX_ERROR(("RDMSR MSR_SYSENTER_ESP: SYSENTER/SYSEXIT feature not enabled !"));
-        if (! BX_CPU_THIS_PTR ignore_bad_msrs)
-          return 0; // will result in #GP fault due to unknown MSR
+        return handle_unknown_rdmsr(index, msr);
       }
       val64 = BX_CPU_THIS_PTR msr.sysenter_esp_msr;
       break;
 
     case BX_MSR_SYSENTER_EIP:
       if (! bx_cpuid_support_sep()) {
-        // failed to find the MSR, could #GP or ignore it silently
         BX_ERROR(("RDMSR MSR_SYSENTER_EIP: SYSENTER/SYSEXIT feature not enabled !"));
-        if (! BX_CPU_THIS_PTR ignore_bad_msrs)
-          return 0; // will result in #GP fault due to unknown MSR
+        return handle_unknown_rdmsr(index, msr);
       }
       val64 = BX_CPU_THIS_PTR msr.sysenter_eip_msr;
       break;
@@ -146,6 +133,16 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
       break;
 #endif
 
+#if BX_CPU_LEVEL >= 6
+    case BX_MSR_TSC_DEADLINE:
+      if (! bx_cpuid_support_tsc_deadline()) {
+        BX_ERROR(("RDMSR BX_MSR_TSC_DEADLINE: TSC-Deadline not enabled !"));
+        return handle_unknown_rdmsr(index, msr);
+      }
+      val64 = BX_CPU_THIS_PTR lapic.get_tsc_deadline();
+      break;
+#endif
+
 #if BX_SUPPORT_VMX
 /*
     case BX_MSR_IA32_SMM_MONITOR_CTL:
@@ -170,10 +167,13 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
     case BX_MSR_VMX_VMENTRY_CTRLS:
       val64 = VMX_MSR_VMX_VMENTRY_CTRLS;
       break;
-#if BX_SUPPORT_VMX >= 2
     case BX_MSR_VMX_PROCBASED_CTRLS2:
-      val64 = VMX_MSR_VMX_PROCBASED_CTRLS2;
-      break;
+      if (BX_CPU_THIS_PTR vmx_cap.vmx_vmexec_ctrl2_supported_bits) {
+        val64 = VMX_MSR_VMX_PROCBASED_CTRLS2;
+        break;
+      }
+      return 0;
+#if BX_SUPPORT_VMX >= 2
     case BX_MSR_VMX_TRUE_PINBASED_CTRLS:
       val64 = VMX_MSR_VMX_TRUE_PINBASED_CTRLS;
       break;
@@ -186,9 +186,18 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
     case BX_MSR_VMX_TRUE_VMENTRY_CTRLS:
       val64 = VMX_MSR_VMX_TRUE_VMENTRY_CTRLS;
       break;
-    case BX_MSR_VMX_MSR_VMX_EPT_VPID_CAP:
-      val64 = VMX_MSR_VMX_EPT_VPID_CAP;
-      break;
+    case BX_MSR_VMX_EPT_VPID_CAP:
+      if (BX_SUPPORT_VMX_EXTENSION(BX_VMX_EPT) || BX_SUPPORT_VMX_EXTENSION(BX_VMX_VPID)) {
+        val64 = VMX_MSR_VMX_EPT_VPID_CAP;
+        break;
+      }
+      return 0;
+    case BX_MSR_VMX_VMFUNC:
+      if (BX_CPU_THIS_PTR vmx_cap.vmx_vmfunc_supported_bits) {
+        val64 = BX_CPU_THIS_PTR vmx_cap.vmx_vmfunc_supported_bits;
+        break;
+      }
+      return 0;
 #endif
     case BX_MSR_VMX_MISC:
       val64 = VMX_MSR_MISC;
@@ -210,56 +219,82 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
       break;
 #endif
 
-#if BX_SUPPORT_X86_64
     case BX_MSR_EFER:
+      if (! BX_CPU_THIS_PTR efer_suppmask) {
+        BX_ERROR(("RDMSR MSR_EFER: EFER MSR is not supported !"));
+        return handle_unknown_rdmsr(index, msr);
+      }
       val64 = BX_CPU_THIS_PTR efer.get32();
       break;
 
     case BX_MSR_STAR:
+      if ((BX_CPU_THIS_PTR efer_suppmask & BX_EFER_SCE_MASK) == 0) {
+        BX_ERROR(("RDMSR MSR_STAR: SYSCALL/SYSRET support not enabled !"));
+        return handle_unknown_rdmsr(index, msr);
+      }
       val64 = MSR_STAR;
       break;
 
+#if BX_SUPPORT_X86_64
     case BX_MSR_LSTAR:
+      if (! bx_cpuid_support_x86_64()) {
+        BX_ERROR(("RDMSR MSR_LSTAR: long mode support not enabled !"));
+        return handle_unknown_rdmsr(index, msr);
+      }
       val64 = MSR_LSTAR;
       break;
 
     case BX_MSR_CSTAR:
+      if (! bx_cpuid_support_x86_64()) {
+        BX_ERROR(("RDMSR MSR_CSTAR: long mode support not enabled !"));
+        return handle_unknown_rdmsr(index, msr);
+      }
       val64 = MSR_CSTAR;
       break;
 
     case BX_MSR_FMASK:
+      if (! bx_cpuid_support_x86_64()) {
+        BX_ERROR(("RDMSR MSR_FMASK: long mode support not enabled !"));
+        return handle_unknown_rdmsr(index, msr);
+      }
       val64 = MSR_FMASK;
       break;
 
     case BX_MSR_FSBASE:
+      if (! bx_cpuid_support_x86_64()) {
+        BX_ERROR(("RDMSR MSR_FSBASE: long mode support not enabled !"));
+        return handle_unknown_rdmsr(index, msr);
+      }
       val64 = MSR_FSBASE;
       break;
 
     case BX_MSR_GSBASE:
+      if (! bx_cpuid_support_x86_64()) {
+        BX_ERROR(("RDMSR MSR_GSBASE: long mode support not enabled !"));
+        return handle_unknown_rdmsr(index, msr);
+      }
       val64 = MSR_GSBASE;
       break;
 
     case BX_MSR_KERNELGSBASE:
+      if (! bx_cpuid_support_x86_64()) {
+        BX_ERROR(("RDMSR MSR_KERNELGSBASE: long mode support not enabled !"));
+        return handle_unknown_rdmsr(index, msr);
+      }
       val64 = MSR_KERNELGSBASE;
       break;
 
     case BX_MSR_TSC_AUX:
+      if (! bx_cpuid_support_rdtscp()) {
+        BX_ERROR(("RDMSR MSR_TSC_AUX: RTDSCP feature not enabled !"));
+        return handle_unknown_rdmsr(index, msr);
+      }
       val64 = MSR_TSC_AUX;   // 32 bit MSR
       break;
-#endif  // #if BX_SUPPORT_X86_64
+#endif
 
     default:
-#if BX_CONFIGURE_MSRS
-      if (index < BX_MSR_MAX_INDEX && BX_CPU_THIS_PTR msrs[index]) {
-        val64 = BX_CPU_THIS_PTR msrs[index]->get64();
-        break;
-      }
-#endif
-      // failed to find the MSR, could #GP or ignore it silently
-      BX_ERROR(("RDMSR: Unknown register %#x", index));
-
-      if (! BX_CPU_THIS_PTR ignore_bad_msrs)
-        return 0; // will result in #GP fault due to unknown MSR
+      return handle_unknown_rdmsr(index, msr);
   }
 
   BX_DEBUG(("RDMSR: read %08x:%08x from MSR %x", GET32H(val64), GET32L(val64), index));
@@ -267,12 +302,44 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
   *msr = val64;
   return 1;
 }
+
+bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::handle_unknown_rdmsr(Bit32u index, Bit64u *msr)
+{
+  Bit64u val_64 = 0;
+
+  // Try to check cpuid_t first (can implement some MSRs)
+  int result = BX_CPU_THIS_PTR cpuid->rdmsr(index, &val_64);
+  if (result == 0)
+    return 0; // #GP fault due to not supported MSR
+
+  if (result < 0) {
+    // cpuid_t have no idea about this MSR
+#if BX_CONFIGURE_MSRS
+    if (index < BX_MSR_MAX_INDEX && BX_CPU_THIS_PTR msrs[index]) {
+      val_64 = BX_CPU_THIS_PTR msrs[index]->get64();
+    }
+    else
+#endif
+    {
+      // failed to find the MSR, could #GP or ignore it silently
+      BX_ERROR(("RDMSR: Unknown register %#x", index));
+
+      if (! BX_CPU_THIS_PTR ignore_bad_msrs)
+        return 0; // will result in #GP fault due to unknown MSR
+    }
+  }
+
+  *msr = val_64;
+  return 1;
+}
+
 #endif // BX_CPU_LEVEL >= 5
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::RDMSR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RDMSR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 5
-  if (!real_mode() && CPL != 0) {
+  // CPL is always 0 in real mode
+  if (/* !real_mode() && */ CPL!=0) {
     BX_ERROR(("RDMSR: CPL != 0 not in real mode"));
     exception(BX_GP_EXCEPTION, 0);
   }
@@ -281,7 +348,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RDMSR(bxInstruction_c *i)
   Bit64u val64 = 0;
 
 #if BX_SUPPORT_VMX
-  VMexit_MSR(i, VMX_VMEXIT_RDMSR, index);
+  if (BX_CPU_THIS_PTR in_vmx_guest)
+    VMexit_MSR(i, VMX_VMEXIT_RDMSR, index);
 #endif
 
 #if BX_SUPPORT_VMX >= 2
@@ -289,7 +357,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RDMSR(bxInstruction_c *i)
     if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_VIRTUALIZE_X2APIC_MODE)) {
       RAX = VMX_Read_VTPR() & 0xff;
       RDX = 0;
-      return;
+      BX_NEXT_INSTR(i);
     }
   }
 #endif
@@ -300,6 +368,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RDMSR(bxInstruction_c *i)
   RAX = GET32L(val64);
   RDX = GET32H(val64);
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 #if BX_CPU_LEVEL >= 6
@@ -361,15 +431,14 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
 
   BX_DEBUG(("WRMSR: write %08x:%08x to MSR %x", val32_hi, val32_lo, index));
 
-  if ((index & 0x3FFFFFFF) >= BX_MSR_MAX_INDEX)
-    return 0;
-
-#if BX_SUPPORT_X2APIC
-  if (index >= 0x800 && index <= 0xBFF) {
-    if (BX_CPU_THIS_PTR msr.apicbase & 0x400)  // X2APIC mode
-      return BX_CPU_THIS_PTR lapic.write_x2apic(index, val_64);
-    else
-      return 0;
+#if BX_CPU_LEVEL >= 6
+  if (bx_cpuid_support_x2apic()) {
+    if (index >= 0x800 && index <= 0xBFF) {
+      if (BX_CPU_THIS_PTR msr.apicbase & 0x400)  // X2APIC mode
+        return BX_CPU_THIS_PTR lapic.write_x2apic(index, val_64);
+      else
+        return 0;
+    }
   }
 #endif
 
@@ -378,20 +447,16 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
 #if BX_CPU_LEVEL >= 6
     case BX_MSR_SYSENTER_CS:
       if (! bx_cpuid_support_sep()) {
-        // failed to find the MSR, could #GP or ignore it silently
         BX_ERROR(("WRMSR MSR_SYSENTER_CS: SYSENTER/SYSEXIT feature not enabled !"));
-        if (! BX_CPU_THIS_PTR ignore_bad_msrs)
-          return 0; // will result in #GP fault due to unknown MSR
+        return handle_unknown_wrmsr(index, val_64);
       }
       BX_CPU_THIS_PTR msr.sysenter_cs_msr = val32_lo;
       break;
 
     case BX_MSR_SYSENTER_ESP:
       if (! bx_cpuid_support_sep()) {
-        // failed to find the MSR, could #GP or ignore it silently
         BX_ERROR(("WRMSR MSR_SYSENTER_ESP: SYSENTER/SYSEXIT feature not enabled !"));
-        if (! BX_CPU_THIS_PTR ignore_bad_msrs)
-          return 0; // will result in #GP fault due to unknown MSR
+        return handle_unknown_wrmsr(index, val_64);
       }
 #if BX_SUPPORT_X86_64
       if (! IsCanonical(val_64)) {
@@ -404,10 +469,8 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
 
     case BX_MSR_SYSENTER_EIP:
       if (! bx_cpuid_support_sep()) {
-        // failed to find the MSR, could #GP or ignore it silently
         BX_ERROR(("WRMSR MSR_SYSENTER_EIP: SYSENTER/SYSEXIT feature not enabled !"));
-        if (! BX_CPU_THIS_PTR ignore_bad_msrs)
-          return 0; // will result in #GP fault due to unknown MSR
+        return handle_unknown_wrmsr(index, val_64);
       }
 #if BX_SUPPORT_X86_64
       if (! IsCanonical(val_64)) {
@@ -526,6 +589,16 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
       return relocate_apic(val_64);
 #endif
 
+#if BX_CPU_LEVEL >= 6
+    case BX_MSR_TSC_DEADLINE:
+      if (! bx_cpuid_support_tsc_deadline()) {
+        BX_ERROR(("WRMSR BX_MSR_TSC_DEADLINE: TSC-Deadline not enabled !"));
+        return handle_unknown_wrmsr(index, val_64);
+      }
+      BX_CPU_THIS_PTR lapic.set_tsc_deadline(val_64);
+      break;
+#endif
+
 #if BX_SUPPORT_VMX
     // Support only two bits: lock bit (bit 0) and VMX enable (bit 2)
     case BX_MSR_IA32_FEATURE_CONTROL:
@@ -554,39 +627,34 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
     case BX_MSR_VMX_CR4_FIXED0:
     case BX_MSR_VMX_CR4_FIXED1:
     case BX_MSR_VMX_VMCS_ENUM:
-    case BX_MSR_VMX_MSR_VMX_EPT_VPID_CAP:
+    case BX_MSR_VMX_EPT_VPID_CAP:
     case BX_MSR_VMX_TRUE_PINBASED_CTRLS:
     case BX_MSR_VMX_TRUE_PROCBASED_CTRLS:
     case BX_MSR_VMX_TRUE_VMEXIT_CTRLS:
     case BX_MSR_VMX_TRUE_VMENTRY_CTRLS:
+    case BX_MSR_VMX_VMFUNC:
       BX_ERROR(("WRMSR: VMX read only MSR"));
       return 0;
 #endif
 
-#if BX_SUPPORT_X86_64
     case BX_MSR_EFER:
-      if (val_64 & ~BX_EFER_SUPPORTED_BITS) {
-        BX_ERROR(("WRMSR: attempt to set reserved bits of EFER MSR !"));
-        return 0;
-      }
-
-      /* #GP(0) if changing EFER.LME when cr0.pg = 1 */
-      if ((BX_CPU_THIS_PTR efer.get_LME() != ((val32_lo >> 8) & 1)) &&
-           BX_CPU_THIS_PTR  cr0.get_PG())
-      {
-        BX_ERROR(("WRMSR: attempt to change LME when CR0.PG=1"));
-        return 0;
-      }
-
-      BX_CPU_THIS_PTR efer.set32((val32_lo & BX_EFER_SUPPORTED_BITS & ~BX_EFER_LMA_MASK)
-              | (BX_CPU_THIS_PTR efer.get32() & BX_EFER_LMA_MASK)); // keep LMA untouched
+      if (! SetEFER(val_64)) return 0;
       break;
 
     case BX_MSR_STAR:
+      if ((BX_CPU_THIS_PTR efer_suppmask & BX_EFER_SCE_MASK) == 0) {
+        BX_ERROR(("RDMSR MSR_STAR: SYSCALL/SYSRET support not enabled !"));
+        return handle_unknown_wrmsr(index, val_64);
+      }
       MSR_STAR = val_64;
       break;
 
+#if BX_SUPPORT_X86_64
     case BX_MSR_LSTAR:
+      if (! bx_cpuid_support_x86_64()) {
+        BX_ERROR(("WRMSR MSR_LSTAR: long mode support not enabled !"));
+        return handle_unknown_wrmsr(index, val_64);
+      }
       if (! IsCanonical(val_64)) {
         BX_ERROR(("WRMSR: attempt to write non-canonical value to MSR_LSTAR !"));
         return 0;
@@ -595,6 +663,10 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
       break;
 
     case BX_MSR_CSTAR:
+      if (! bx_cpuid_support_x86_64()) {
+        BX_ERROR(("WRMSR MSR_CSTAR: long mode support not enabled !"));
+        return handle_unknown_wrmsr(index, val_64);
+      }
       if (! IsCanonical(val_64)) {
         BX_ERROR(("WRMSR: attempt to write non-canonical value to MSR_CSTAR !"));
         return 0;
@@ -603,10 +675,18 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
       break;
 
     case BX_MSR_FMASK:
+      if (! bx_cpuid_support_x86_64()) {
+        BX_ERROR(("WRMSR MSR_FMASK: long mode support not enabled !"));
+        return handle_unknown_wrmsr(index, val_64);
+      }
       MSR_FMASK = (Bit32u) val_64;
       break;
 
     case BX_MSR_FSBASE:
+      if (! bx_cpuid_support_x86_64()) {
+        BX_ERROR(("WRMSR MSR_FSBASE: long mode support not enabled !"));
+        return handle_unknown_wrmsr(index, val_64);
+      }
       if (! IsCanonical(val_64)) {
         BX_ERROR(("WRMSR: attempt to write non-canonical value to MSR_FSBASE !"));
         return 0;
@@ -615,6 +695,10 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
       break;
 
     case BX_MSR_GSBASE:
+      if (! bx_cpuid_support_x86_64()) {
+        BX_ERROR(("WRMSR MSR_GSBASE: long mode support not enabled !"));
+        return handle_unknown_wrmsr(index, val_64);
+      }
       if (! IsCanonical(val_64)) {
         BX_ERROR(("WRMSR: attempt to write non-canonical value to MSR_GSBASE !"));
         return 0;
@@ -623,6 +707,10 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
       break;
 
     case BX_MSR_KERNELGSBASE:
+      if (! bx_cpuid_support_x86_64()) {
+        BX_ERROR(("WRMSR MSR_KERNELGSBASE: long mode support not enabled !"));
+        return handle_unknown_wrmsr(index, val_64);
+      }
       if (! IsCanonical(val_64)) {
         BX_ERROR(("WRMSR: attempt to write non-canonical value to MSR_KERNELGSBASE !"));
         return 0;
@@ -631,28 +719,48 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
       break;
 
     case BX_MSR_TSC_AUX:
+      if (! bx_cpuid_support_rdtscp()) {
+        BX_ERROR(("WRMSR MSR_TSC_AUX: RTDSCP feature not enabled !"));
+        return handle_unknown_wrmsr(index, val_64);
+      }
       MSR_TSC_AUX = val32_lo;
       break;
 #endif  // #if BX_SUPPORT_X86_64
 
     default:
-#if BX_CONFIGURE_MSRS
-      if (index < BX_MSR_MAX_INDEX && BX_CPU_THIS_PTR msrs[index]) {
-        if (! BX_CPU_THIS_PTR msrs[index]->set64(val_64)) {
-          BX_ERROR(("WRMSR: Write failed to MSR %#x - #GP fault", index));
-          return 0;
-        }
-        break;
-      }
-#endif
-      // failed to find the MSR, could #GP or ignore it silently
-      BX_ERROR(("WRMSR: Unknown register %#x", index));
-      if (! BX_CPU_THIS_PTR ignore_bad_msrs)
-        return 0; // will result in #GP fault due to unknown MSR
+      return handle_unknown_wrmsr(index, val_64);
   }
 
   return 1;
 }
+
+bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::handle_unknown_wrmsr(Bit32u index, Bit64u val_64)
+{
+  // Try to check cpuid_t first (can implement some MSRs)
+  int result = BX_CPU_THIS_PTR cpuid->wrmsr(index, val_64);
+  if (result == 0)
+    return 0; // #GP fault due to not supported MSR
+
+  if (result < 0) {
+    // cpuid_t have no idea about this MSR
+#if BX_CONFIGURE_MSRS
+    if (index < BX_MSR_MAX_INDEX && BX_CPU_THIS_PTR msrs[index]) {
+      if (! BX_CPU_THIS_PTR msrs[index]->set64(val_64)) {
+        BX_ERROR(("WRMSR: Write failed to MSR %#x - #GP fault", index));
+        return 0;
+      }
+      return 1;
+    }
+#endif
+    // failed to find the MSR, could #GP or ignore it silently
+    BX_ERROR(("WRMSR: Unknown register %#x", index));
+    if (! BX_CPU_THIS_PTR ignore_bad_msrs)
+      return 0; // will result in #GP fault due to unknown MSR
+  }
+
+  return 1;
+}
+
 #endif // BX_CPU_LEVEL >= 5
 
 #if BX_SUPPORT_APIC
@@ -668,32 +776,32 @@ bx_bool BX_CPU_C::relocate_apic(Bit64u val_64)
    * [M:63]  Reserved
    */
 
-#define BX_MSR_APICBASE_RESERVED_BITS (0x2ff | (BX_SUPPORT_X2APIC ? 0 : 0x400))
+#define BX_MSR_APICBASE_RESERVED_BITS (0x2ff | (bx_cpuid_support_x2apic() ? 0 : 0x400))
 
   if (BX_CPU_THIS_PTR msr.apicbase & 0x800) {
     Bit32u val32_hi = GET32H(val_64), val32_lo = GET32L(val_64);
     BX_INFO(("WRMSR: wrote %08x:%08x to MSR_APICBASE", val32_hi, val32_lo));
-#if BX_SUPPORT_X86_64
     if (! IsValidPhyAddr(val_64)) {
       BX_ERROR(("relocate_apic: invalid physical address"));
       return 0;
     }
-#endif
     if (val32_lo & BX_MSR_APICBASE_RESERVED_BITS) {
       BX_ERROR(("relocate_apic: attempt to set reserved bits"));
       return 0;
     }
 
-#if BX_SUPPORT_X2APIC
-    unsigned apic_state = (BX_CPU_THIS_PTR msr.apicbase >> 10) & 3;
-    unsigned new_state = (val32_lo >> 10) & 3;
-    if (new_state == BX_APIC_STATE_INVALID) {
-      BX_ERROR(("relocate_apic: attempt to set invalid apic state"));
-      return 0;
-    }
-    if (apic_state == BX_APIC_X2APIC_MODE && new_state != BX_APIC_GLOBALLY_DISABLED) {
-      BX_ERROR(("relocate_apic: attempt to switch from x2apic -> xapic"));
-      return 0;
+#if BX_CPU_LEVEL >= 6
+    if (bx_cpuid_support_x2apic()) {
+      unsigned apic_state = (BX_CPU_THIS_PTR msr.apicbase >> 10) & 3;
+      unsigned new_state = (val32_lo >> 10) & 3;
+      if (new_state == BX_APIC_STATE_INVALID) {
+        BX_ERROR(("relocate_apic: attempt to set invalid apic state"));
+        return 0;
+      }
+      if (apic_state == BX_APIC_X2APIC_MODE && new_state != BX_APIC_GLOBALLY_DISABLED) {
+        BX_ERROR(("relocate_apic: attempt to switch from x2apic -> xapic"));
+        return 0;
+      }
     }
 #endif
 
@@ -701,12 +809,6 @@ bx_bool BX_CPU_C::relocate_apic(Bit64u val_64)
     BX_CPU_THIS_PTR lapic.set_base(BX_CPU_THIS_PTR msr.apicbase);
     // TLB flush is required for emulation correctness
     TLB_flush();  // don't care about performance of apic relocation
-
-    if ((val32_lo & 0x800) == 0) {
-      // APIC global enable bit cleared, clear APIC on chip CPUID feature flag
-      BX_CPU_THIS_PTR cpuid_std_function[0x1].edx &= ~(1<<9);
-      BX_CPU_THIS_PTR cpuid_ext_function[0x1].edx &= ~(1<<9);
-    }
   }
   else {
     BX_INFO(("WRMSR: MSR_APICBASE APIC global enable bit cleared !"));
@@ -716,10 +818,11 @@ bx_bool BX_CPU_C::relocate_apic(Bit64u val_64)
 }
 #endif
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::WRMSR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::WRMSR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 5
-  if (!real_mode() && CPL != 0) {
+  // CPL is always 0 in real mode
+  if (/* !real_mode() && */ CPL!=0) {
     BX_ERROR(("WRMSR: CPL != 0 not in real mode"));
     exception(BX_GP_EXCEPTION, 0);
   }
@@ -728,14 +831,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::WRMSR(bxInstruction_c *i)
   Bit32u index = ECX;
 
 #if BX_SUPPORT_VMX
-  VMexit_MSR(i, VMX_VMEXIT_WRMSR, index);
+  if (BX_CPU_THIS_PTR in_vmx_guest)
+    VMexit_MSR(i, VMX_VMEXIT_WRMSR, index);
 #endif
 
 #if BX_SUPPORT_VMX >= 2
   if (BX_CPU_THIS_PTR in_vmx_guest && index == 0x808) {
     if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_VIRTUALIZE_X2APIC_MODE)) {
       VMX_Write_VTPR(AL);
-      return;
+      BX_NEXT_INSTR(i);
     }
   }
 #endif
@@ -743,6 +847,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::WRMSR(bxInstruction_c *i)
   if (! wrmsr(index, val_64))
     exception(BX_GP_EXCEPTION, 0);
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 #if BX_CONFIGURE_MSRS
