@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: shift64.cc,v 1.42 2010/05/18 07:28:05 sshwarts Exp $
+// $Id: shift64.cc 10451 2011-07-06 20:01:18Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2010  The Bochs Project
+//  Copyright (C) 2001-2011  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,7 +26,7 @@
 
 #if BX_SUPPORT_X86_64
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHLD_EqGqM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHLD_EqGqM(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, result_64;
   unsigned count;
@@ -43,22 +43,25 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHLD_EqGqM(bxInstruction_c *i)
     count = CL;
 
   count &= 0x3f; // use only 6 LSB's
-  if (!count) return;
 
-  op2_64 = BX_READ_64BIT_REG(i->nnn());
+  if (count) {
+    op2_64 = BX_READ_64BIT_REG(i->nnn());
 
-  result_64 = (op1_64 << count) | (op2_64 >> (64 - count));
+    result_64 = (op1_64 << count) | (op2_64 >> (64 - count));
 
-  write_RMW_virtual_qword(result_64);
+    write_RMW_virtual_qword(result_64);
 
-  SET_FLAGS_OSZAPC_LOGIC_64(result_64);
+    SET_FLAGS_OSZAPC_LOGIC_64(result_64);
 
-  cf = (op1_64 >> (64 - count)) & 0x1;
-  of = cf ^ (result_64 >> 63); // of = cf ^ result63
-  SET_FLAGS_OxxxxC(of, cf);
+    cf = (op1_64 >> (64 - count)) & 0x1;
+    of = cf ^ (result_64 >> 63); // of = cf ^ result63
+    SET_FLAGS_OxxxxC(of, cf);
+  }
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHLD_EqGqR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHLD_EqGqR(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, result_64;
   unsigned count;
@@ -70,23 +73,26 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHLD_EqGqR(bxInstruction_c *i)
     count = CL;
 
   count &= 0x3f; // use only 6 LSB's
-  if (!count) return;
 
-  op1_64 = BX_READ_64BIT_REG(i->rm());
-  op2_64 = BX_READ_64BIT_REG(i->nnn());
+  if (count) {
+    op1_64 = BX_READ_64BIT_REG(i->rm());
+    op2_64 = BX_READ_64BIT_REG(i->nnn());
 
-  result_64 = (op1_64 << count) | (op2_64 >> (64 - count));
+    result_64 = (op1_64 << count) | (op2_64 >> (64 - count));
 
-  BX_WRITE_64BIT_REG(i->rm(), result_64);
+    BX_WRITE_64BIT_REG(i->rm(), result_64);
 
-  SET_FLAGS_OSZAPC_LOGIC_64(result_64);
+    SET_FLAGS_OSZAPC_LOGIC_64(result_64);
 
-  cf = (op1_64 >> (64 - count)) & 0x1;
-  of = cf ^ (result_64 >> 63); // of = cf ^ result63
-  SET_FLAGS_OxxxxC(of, cf);
+    cf = (op1_64 >> (64 - count)) & 0x1;
+    of = cf ^ (result_64 >> 63); // of = cf ^ result63
+    SET_FLAGS_OxxxxC(of, cf);
+  }
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EqGqM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EqGqM(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, result_64;
   unsigned count;
@@ -103,22 +109,25 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EqGqM(bxInstruction_c *i)
     count = CL;
 
   count &= 0x3f; // use only 6 LSB's
-  if (!count) return;
 
-  op2_64 = BX_READ_64BIT_REG(i->nnn());
+  if (count) {
+    op2_64 = BX_READ_64BIT_REG(i->nnn());
 
-  result_64 = (op2_64 << (64 - count)) | (op1_64 >> count);
+    result_64 = (op2_64 << (64 - count)) | (op1_64 >> count);
 
-  write_RMW_virtual_qword(result_64);
+    write_RMW_virtual_qword(result_64);
 
-  SET_FLAGS_OSZAPC_LOGIC_64(result_64);
+    SET_FLAGS_OSZAPC_LOGIC_64(result_64);
 
-  cf = (op1_64 >> (count - 1)) & 0x1;
-  of = ((result_64 << 1) ^ result_64) >> 63; // of = result62 ^ result63
-  SET_FLAGS_OxxxxC(of, cf);
+    cf = (op1_64 >> (count - 1)) & 0x1;
+    of = ((result_64 << 1) ^ result_64) >> 63; // of = result62 ^ result63
+    SET_FLAGS_OxxxxC(of, cf);
+  }
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EqGqR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EqGqR(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, result_64;
   unsigned count;
@@ -130,23 +139,26 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EqGqR(bxInstruction_c *i)
     count = CL;
 
   count &= 0x3f; // use only 6 LSB's
-  if (!count) return;
 
-  op1_64 = BX_READ_64BIT_REG(i->rm());
-  op2_64 = BX_READ_64BIT_REG(i->nnn());
+  if (count) {
+    op1_64 = BX_READ_64BIT_REG(i->rm());
+    op2_64 = BX_READ_64BIT_REG(i->nnn());
 
-  result_64 = (op2_64 << (64 - count)) | (op1_64 >> count);
+    result_64 = (op2_64 << (64 - count)) | (op1_64 >> count);
 
-  BX_WRITE_64BIT_REG(i->rm(), result_64);
+    BX_WRITE_64BIT_REG(i->rm(), result_64);
 
-  SET_FLAGS_OSZAPC_LOGIC_64(result_64);
+    SET_FLAGS_OSZAPC_LOGIC_64(result_64);
 
-  cf = (op1_64 >> (count - 1)) & 0x1;
-  of = ((result_64 << 1) ^ result_64) >> 63; // of = result62 ^ result63
-  SET_FLAGS_OxxxxC(of, cf);
+    cf = (op1_64 >> (count - 1)) & 0x1;
+    of = ((result_64 << 1) ^ result_64) >> 63; // of = result62 ^ result63
+    SET_FLAGS_OxxxxC(of, cf);
+  }
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROL_EqM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ROL_EqM(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
@@ -163,19 +175,22 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROL_EqM(bxInstruction_c *i)
     count = i->Ib();
 
   count &= 0x3f;
-  if (! count) return;
 
-  result_64 = (op1_64 << count) | (op1_64 >> (64 - count));
+  if (count) {
+    result_64 = (op1_64 << count) | (op1_64 >> (64 - count));
 
-  write_RMW_virtual_qword(result_64);
+    write_RMW_virtual_qword(result_64);
 
-  bit0  = (result_64 & 0x1);
-  bit63 = (result_64 >> 63);
-  // of = cf ^ result63
-  SET_FLAGS_OxxxxC(bit0 ^ bit63, bit0);
+    bit0  = (result_64 & 0x1);
+    bit63 = (result_64 >> 63);
+    // of = cf ^ result63
+    SET_FLAGS_OxxxxC(bit0 ^ bit63, bit0);
+  }
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROL_EqR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ROL_EqR(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
@@ -187,19 +202,22 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROL_EqR(bxInstruction_c *i)
     count = i->Ib();
 
   count &= 0x3f;
-  if (! count) return;
+  
+  if (count) {
+    op1_64 = BX_READ_64BIT_REG(i->rm());
+    result_64 = (op1_64 << count) | (op1_64 >> (64 - count));
+    BX_WRITE_64BIT_REG(i->rm(), result_64);
 
-  op1_64 = BX_READ_64BIT_REG(i->rm());
-  result_64 = (op1_64 << count) | (op1_64 >> (64 - count));
-  BX_WRITE_64BIT_REG(i->rm(), result_64);
+    bit0  = (result_64 & 0x1);
+    bit63 = (result_64 >> 63);
+    // of = cf ^ result63
+    SET_FLAGS_OxxxxC(bit0 ^ bit63, bit0);
+  }
 
-  bit0  = (result_64 & 0x1);
-  bit63 = (result_64 >> 63);
-  // of = cf ^ result63
-  SET_FLAGS_OxxxxC(bit0 ^ bit63, bit0);
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROR_EqM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ROR_EqM(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
@@ -216,19 +234,22 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROR_EqM(bxInstruction_c *i)
     count = i->Ib();
 
   count &= 0x3f;
-  if (! count) return;
 
-  result_64 = (op1_64 >> count) | (op1_64 << (64 - count));
+  if (count) {
+    result_64 = (op1_64 >> count) | (op1_64 << (64 - count));
 
-  write_RMW_virtual_qword(result_64);
+    write_RMW_virtual_qword(result_64);
 
-  bit63 = (result_64 >> 63) & 1;
-  bit62 = (result_64 >> 62) & 1;
-  // of = result62 ^ result63
-  SET_FLAGS_OxxxxC(bit62 ^ bit63, bit63);
+    bit63 = (result_64 >> 63) & 1;
+    bit62 = (result_64 >> 62) & 1;
+    // of = result62 ^ result63
+    SET_FLAGS_OxxxxC(bit62 ^ bit63, bit63);
+  }
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROR_EqR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ROR_EqR(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
@@ -240,19 +261,22 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROR_EqR(bxInstruction_c *i)
     count = i->Ib();
 
   count &= 0x3f;
-  if (! count) return;
 
-  op1_64 = BX_READ_64BIT_REG(i->rm());
-  result_64 = (op1_64 >> count) | (op1_64 << (64 - count));
-  BX_WRITE_64BIT_REG(i->rm(), result_64);
+  if (count) {
+    op1_64 = BX_READ_64BIT_REG(i->rm());
+    result_64 = (op1_64 >> count) | (op1_64 << (64 - count));
+    BX_WRITE_64BIT_REG(i->rm(), result_64);
 
-  bit63 = (result_64 >> 63) & 1;
-  bit62 = (result_64 >> 62) & 1;
-  // of = result62 ^ result63
-  SET_FLAGS_OxxxxC(bit62 ^ bit63, bit63);
+    bit63 = (result_64 >> 63) & 1;
+    bit62 = (result_64 >> 62) & 1;
+    // of = result62 ^ result63
+    SET_FLAGS_OxxxxC(bit62 ^ bit63, bit63);
+  }
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EqM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EqM(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
@@ -269,7 +293,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EqM(bxInstruction_c *i)
     count = i->Ib();
 
   count &= 0x3f;
-  if (!count) return;
+
+  if (!count) {
+    BX_NEXT_INSTR(i);
+  }
 
   if (count==1) {
     result_64 = (op1_64 << 1) | getB_CF();
@@ -284,9 +311,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EqM(bxInstruction_c *i)
   cf = (op1_64 >> (64 - count)) & 0x1;
   of = cf ^ (result_64 >> 63); // of = cf ^ result63
   SET_FLAGS_OxxxxC(of, cf);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EqR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EqR(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
@@ -298,7 +327,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EqR(bxInstruction_c *i)
     count = i->Ib();
 
   count &= 0x3f;
-  if (!count) return;
+
+  if (!count) {
+    BX_NEXT_INSTR(i);
+  }
 
   op1_64 = BX_READ_64BIT_REG(i->rm());
 
@@ -315,9 +347,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EqR(bxInstruction_c *i)
   cf = (op1_64 >> (64 - count)) & 0x1;
   of = cf ^ (result_64 >> 63); // of = cf ^ result63
   SET_FLAGS_OxxxxC(of, cf);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EqM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EqM(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
@@ -334,7 +368,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EqM(bxInstruction_c *i)
     count = i->Ib();
 
   count &= 0x3f;
-  if (!count) return;
+
+  if (!count) {
+    BX_NEXT_INSTR(i);
+  }
 
   if (count==1) {
     result_64 = (op1_64 >> 1) | (((Bit64u) getB_CF()) << 63);
@@ -349,9 +386,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EqM(bxInstruction_c *i)
   cf = (op1_64 >> (count - 1)) & 0x1;
   of = ((result_64 << 1) ^ result_64) >> 63;
   SET_FLAGS_OxxxxC(of, cf);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EqR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EqR(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
@@ -363,7 +402,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EqR(bxInstruction_c *i)
     count = i->Ib();
 
   count &= 0x3f;
-  if (!count) return;
+
+  if (!count) {
+    BX_NEXT_INSTR(i);
+  }
 
   op1_64 = BX_READ_64BIT_REG(i->rm());
 
@@ -380,9 +422,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EqR(bxInstruction_c *i)
   cf = (op1_64 >> (count - 1)) & 0x1;
   of = ((result_64 << 1) ^ result_64) >> 63;
   SET_FLAGS_OxxxxC(of, cf);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EqM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EqM(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
@@ -399,20 +443,23 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EqM(bxInstruction_c *i)
     count = i->Ib();
 
   count &= 0x3f;
-  if (!count) return;
 
-  /* count < 64, since only lower 6 bits used */
-  result_64 = (op1_64 << count);
-  cf = (op1_64 >> (64 - count)) & 0x1;
-  of = cf ^ (result_64 >> 63);
+  if (count) {
+    /* count < 64, since only lower 6 bits used */
+    result_64 = (op1_64 << count);
+    cf = (op1_64 >> (64 - count)) & 0x1;
+    of = cf ^ (result_64 >> 63);
 
-  write_RMW_virtual_qword(result_64);
+    write_RMW_virtual_qword(result_64);
 
-  SET_FLAGS_OSZAPC_LOGIC_64(result_64);
-  SET_FLAGS_OxxxxC(of, cf);
+    SET_FLAGS_OSZAPC_LOGIC_64(result_64);
+    SET_FLAGS_OxxxxC(of, cf);
+  }
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EqR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EqR(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
@@ -424,20 +471,23 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EqR(bxInstruction_c *i)
     count = i->Ib();
 
   count &= 0x3f;
-  if (!count) return;
 
-  op1_64 = BX_READ_64BIT_REG(i->rm());
-  /* count < 64, since only lower 6 bits used */
-  result_64 = (op1_64 << count);
-  BX_WRITE_64BIT_REG(i->rm(), result_64);
+  if (count) {
+    op1_64 = BX_READ_64BIT_REG(i->rm());
+    /* count < 64, since only lower 6 bits used */
+    result_64 = (op1_64 << count);
+    BX_WRITE_64BIT_REG(i->rm(), result_64);
 
-  cf = (op1_64 >> (64 - count)) & 0x1;
-  of = cf ^ (result_64 >> 63);
-  SET_FLAGS_OSZAPC_LOGIC_64(result_64);
-  SET_FLAGS_OxxxxC(of, cf);
+    cf = (op1_64 >> (64 - count)) & 0x1;
+    of = cf ^ (result_64 >> 63);
+    SET_FLAGS_OSZAPC_LOGIC_64(result_64);
+    SET_FLAGS_OxxxxC(of, cf);
+  }
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_EqM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_EqM(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
@@ -454,22 +504,25 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_EqM(bxInstruction_c *i)
     count = i->Ib();
 
   count &= 0x3f;
-  if (!count) return;
 
-  result_64 = (op1_64 >> count);
+  if (count) {
+    result_64 = (op1_64 >> count);
 
-  write_RMW_virtual_qword(result_64);
+    write_RMW_virtual_qword(result_64);
 
-  cf = (op1_64 >> (count - 1)) & 0x1;
-  // note, that of == result63 if count == 1 and
-  //            of == 0        if count >= 2
-  of = ((result_64 << 1) ^ result_64) >> 63;
+    cf = (op1_64 >> (count - 1)) & 0x1;
+    // note, that of == result63 if count == 1 and
+    //            of == 0        if count >= 2
+    of = ((result_64 << 1) ^ result_64) >> 63;
 
-  SET_FLAGS_OSZAPC_LOGIC_64(result_64);
-  SET_FLAGS_OxxxxC(of, cf);
+    SET_FLAGS_OSZAPC_LOGIC_64(result_64);
+    SET_FLAGS_OxxxxC(of, cf);
+  }
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_EqR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_EqR(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
@@ -481,22 +534,25 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_EqR(bxInstruction_c *i)
     count = i->Ib();
 
   count &= 0x3f;
-  if (!count) return;
 
-  op1_64 = BX_READ_64BIT_REG(i->rm());
-  result_64 = (op1_64 >> count);
-  BX_WRITE_64BIT_REG(i->rm(), result_64);
+  if (count) {
+    op1_64 = BX_READ_64BIT_REG(i->rm());
+    result_64 = (op1_64 >> count);
+    BX_WRITE_64BIT_REG(i->rm(), result_64);
 
-  cf = (op1_64 >> (count - 1)) & 0x1;
-  // note, that of == result63 if count == 1 and
-  //            of == 0        if count >= 2
-  of = ((result_64 << 1) ^ result_64) >> 63;
+    cf = (op1_64 >> (count - 1)) & 0x1;
+    // note, that of == result63 if count == 1 and
+    //            of == 0        if count >= 2
+    of = ((result_64 << 1) ^ result_64) >> 63;
 
-  SET_FLAGS_OSZAPC_LOGIC_64(result_64);
-  SET_FLAGS_OxxxxC(of, cf);
+    SET_FLAGS_OSZAPC_LOGIC_64(result_64);
+    SET_FLAGS_OxxxxC(of, cf);
+  }
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_EqM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_EqM(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
@@ -512,19 +568,22 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_EqM(bxInstruction_c *i)
     count = i->Ib();
 
   count &= 0x3f;
-  if (!count) return;
 
-  /* count < 64, since only lower 6 bits used */
-  result_64 = ((Bit64s) op1_64) >> count;
+  if (count) {
+    /* count < 64, since only lower 6 bits used */
+    result_64 = ((Bit64s) op1_64) >> count;
 
-  write_RMW_virtual_qword(result_64);
+    write_RMW_virtual_qword(result_64);
 
-  SET_FLAGS_OSZAPC_LOGIC_64(result_64);
-  set_CF((op1_64 >> (count - 1)) & 1);
-  clear_OF();  /* signed overflow cannot happen in SAR instruction */
+    SET_FLAGS_OSZAPC_LOGIC_64(result_64);
+    set_CF((op1_64 >> (count - 1)) & 1);
+    clear_OF();  /* signed overflow cannot happen in SAR instruction */
+  }
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_EqR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_EqR(bxInstruction_c *i)
 {
   Bit64u op1_64, result_64;
   unsigned count;
@@ -535,18 +594,21 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_EqR(bxInstruction_c *i)
     count = i->Ib();
 
   count &= 0x3f;
-  if (!count) return;
 
-  op1_64 = BX_READ_64BIT_REG(i->rm());
+  if (count) {
+    op1_64 = BX_READ_64BIT_REG(i->rm());
 
-  /* count < 64, since only lower 6 bits used */
-  result_64 = ((Bit64s) op1_64) >> count;
+    /* count < 64, since only lower 6 bits used */
+    result_64 = ((Bit64s) op1_64) >> count;
 
-  BX_WRITE_64BIT_REG(i->rm(), result_64);
+    BX_WRITE_64BIT_REG(i->rm(), result_64);
 
-  SET_FLAGS_OSZAPC_LOGIC_64(result_64);
-  set_CF((op1_64 >> (count - 1)) & 1);
-  clear_OF();  /* signed overflow cannot happen in SAR instruction */
+    SET_FLAGS_OSZAPC_LOGIC_64(result_64);
+    set_CF((op1_64 >> (count - 1)) & 1);
+    clear_OF();  /* signed overflow cannot happen in SAR instruction */
+  }
+
+  BX_NEXT_INSTR(i);
 }
 
 #endif /* if BX_SUPPORT_X86_64 */
