@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: shift8.cc,v 1.43 2010/05/18 07:28:05 sshwarts Exp $
+// $Id: shift8.cc 10451 2011-07-06 20:01:18Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2010  The Bochs Project
+//  Copyright (C) 2001-2011  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,7 +24,7 @@
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROL_EbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ROL_EbR(bxInstruction_c *i)
 {
   unsigned count;
   unsigned bit0, bit7;
@@ -42,25 +42,27 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROL_EbR(bxInstruction_c *i)
       bit7 = (op1_8 >> 7);
       SET_FLAGS_OxxxxC(bit0 ^ bit7, bit0);
     }
-    return;
+  }
+  else {
+    count &= 0x7; // use only lowest 3 bits
+
+    Bit8u result_8 = (op1_8 << count) | (op1_8 >> (8 - count));
+
+    BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), result_8);
+
+    /* set eflags:
+     * ROL count affects the following flags: C, O
+     */
+    bit0 = (result_8 &  1);
+    bit7 = (result_8 >> 7);
+
+    SET_FLAGS_OxxxxC(bit0 ^ bit7, bit0);
   }
 
-  count &= 0x7; // use only lowest 3 bits
-
-  Bit8u result_8 = (op1_8 << count) | (op1_8 >> (8 - count));
-
-  BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), result_8);
-
-  /* set eflags:
-   * ROL count affects the following flags: C, O
-   */
-  bit0 = (result_8 &  1);
-  bit7 = (result_8 >> 7);
-
-  SET_FLAGS_OxxxxC(bit0 ^ bit7, bit0);
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROL_EbM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ROL_EbM(bxInstruction_c *i)
 {
   unsigned count;
   unsigned bit0, bit7;
@@ -80,25 +82,27 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROL_EbM(bxInstruction_c *i)
       bit7 = (op1_8 >> 7);
       SET_FLAGS_OxxxxC(bit0 ^ bit7, bit0);
     }
-    return;
+  }
+  else {
+    count &= 0x7; // use only lowest 3 bits
+
+    Bit8u result_8 = (op1_8 << count) | (op1_8 >> (8 - count));
+
+    write_RMW_virtual_byte(result_8);
+
+    /* set eflags:
+     * ROL count affects the following flags: C, O
+     */
+    bit0 = (result_8 &  1);
+    bit7 = (result_8 >> 7);
+
+    SET_FLAGS_OxxxxC(bit0 ^ bit7, bit0);
   }
 
-  count &= 0x7; // use only lowest 3 bits
-
-  Bit8u result_8 = (op1_8 << count) | (op1_8 >> (8 - count));
-
-  write_RMW_virtual_byte(result_8);
-
-  /* set eflags:
-   * ROL count affects the following flags: C, O
-   */
-  bit0 = (result_8 &  1);
-  bit7 = (result_8 >> 7);
-
-  SET_FLAGS_OxxxxC(bit0 ^ bit7, bit0);
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROR_EbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ROR_EbR(bxInstruction_c *i)
 {
   unsigned count;
   unsigned bit6, bit7;
@@ -117,25 +121,27 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROR_EbR(bxInstruction_c *i)
 
       SET_FLAGS_OxxxxC(bit6 ^ bit7, bit7);
     }
-    return;
+  }
+  else {
+    count &= 0x7; /* use only bottom 3 bits */
+
+    Bit8u result_8 = (op1_8 >> count) | (op1_8 << (8 - count));
+
+    BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), result_8);
+
+    /* set eflags:
+     * ROR count affects the following flags: C, O
+     */
+    bit6 = (result_8 >> 6) & 1;
+    bit7 = (result_8 >> 7) & 1;
+
+    SET_FLAGS_OxxxxC(bit6 ^ bit7, bit7);
   }
 
-  count &= 0x7; /* use only bottom 3 bits */
-
-  Bit8u result_8 = (op1_8 >> count) | (op1_8 << (8 - count));
-
-  BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), result_8);
-
-  /* set eflags:
-   * ROR count affects the following flags: C, O
-   */
-  bit6 = (result_8 >> 6) & 1;
-  bit7 = (result_8 >> 7) & 1;
-
-  SET_FLAGS_OxxxxC(bit6 ^ bit7, bit7);
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROR_EbM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ROR_EbM(bxInstruction_c *i)
 {
   unsigned count;
   unsigned bit6, bit7;
@@ -156,25 +162,27 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROR_EbM(bxInstruction_c *i)
 
       SET_FLAGS_OxxxxC(bit6 ^ bit7, bit7);
     }
-    return;
+  }
+  else {
+    count &= 0x7; /* use only bottom 3 bits */
+
+    Bit8u result_8 = (op1_8 >> count) | (op1_8 << (8 - count));
+
+    write_RMW_virtual_byte(result_8);
+
+    /* set eflags:
+     * ROR count affects the following flags: C, O
+     */
+    bit6 = (result_8 >> 6) & 1;
+    bit7 = (result_8 >> 7) & 1;
+
+    SET_FLAGS_OxxxxC(bit6 ^ bit7, bit7);
   }
 
-  count &= 0x7; /* use only bottom 3 bits */
-
-  Bit8u result_8 = (op1_8 >> count) | (op1_8 << (8 - count));
-
-  write_RMW_virtual_byte(result_8);
-
-  /* set eflags:
-   * ROR count affects the following flags: C, O
-   */
-  bit6 = (result_8 >> 6) & 1;
-  bit7 = (result_8 >> 7) & 1;
-
-  SET_FLAGS_OxxxxC(bit6 ^ bit7, bit7);
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EbR(bxInstruction_c *i)
 {
   Bit8u result_8;
   unsigned count;
@@ -187,7 +195,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EbR(bxInstruction_c *i)
 
   count = (count & 0x1f) % 9;
 
-  if (! count) return;
+  if (! count) {
+    BX_NEXT_INSTR(i);
+  }
 
   Bit8u op1_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
 
@@ -204,9 +214,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EbR(bxInstruction_c *i)
   cf = (op1_8 >> (8 - count)) & 0x01;
   of = cf ^ (result_8 >> 7);  // of = cf ^ result7
   SET_FLAGS_OxxxxC(of, cf);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EbM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EbM(bxInstruction_c *i)
 {
   Bit8u result_8;
   unsigned count;
@@ -223,7 +235,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EbM(bxInstruction_c *i)
 
   count = (count & 0x1f) % 9;
 
-  if (! count) return;
+  if (! count) {
+    BX_NEXT_INSTR(i);
+  }
 
   if (count==1) {
     result_8 = (op1_8 << 1) | getB_CF();
@@ -238,9 +252,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EbM(bxInstruction_c *i)
   cf = (op1_8 >> (8 - count)) & 0x01;
   of = cf ^ (result_8 >> 7);  // of = cf ^ result7
   SET_FLAGS_OxxxxC(of, cf);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EbR(bxInstruction_c *i)
 {
   unsigned count;
   unsigned cf, of;
@@ -252,21 +268,23 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EbR(bxInstruction_c *i)
 
   count = (count & 0x1f) % 9;
 
-  if (! count) return;
+  if (count) {
+    Bit8u op1_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
 
-  Bit8u op1_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
+    Bit8u result_8 = (op1_8 >> count) | (getB_CF() << (8 - count)) |
+                     (op1_8 << (9 - count));
 
-  Bit8u result_8 = (op1_8 >> count) | (getB_CF() << (8 - count)) |
-                   (op1_8 << (9 - count));
+    BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), result_8);
 
-  BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), result_8);
+    cf = (op1_8 >> (count - 1)) & 0x1;
+    of = (((result_8 << 1) ^ result_8) >> 7) & 0x1; // of = result6 ^ result7
+    SET_FLAGS_OxxxxC(of, cf);
+  }
 
-  cf = (op1_8 >> (count - 1)) & 0x1;
-  of = (((result_8 << 1) ^ result_8) >> 7) & 0x1; // of = result6 ^ result7
-  SET_FLAGS_OxxxxC(of, cf);
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EbM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EbM(bxInstruction_c *i)
 {
   unsigned count;
   unsigned cf, of;
@@ -282,19 +300,21 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EbM(bxInstruction_c *i)
 
   count = (count & 0x1f) % 9;
 
-  if (! count) return;
+  if (count) {
+    Bit8u result_8 = (op1_8 >> count) | (getB_CF() << (8 - count)) |
+                     (op1_8 << (9 - count));
 
-  Bit8u result_8 = (op1_8 >> count) | (getB_CF() << (8 - count)) |
-                   (op1_8 << (9 - count));
+    write_RMW_virtual_byte(result_8);
 
-  write_RMW_virtual_byte(result_8);
+    cf = (op1_8 >> (count - 1)) & 0x1;
+    of = (((result_8 << 1) ^ result_8) >> 7) & 0x1; // of = result6 ^ result7
+    SET_FLAGS_OxxxxC(of, cf);
+  }
 
-  cf = (op1_8 >> (count - 1)) & 0x1;
-  of = (((result_8 << 1) ^ result_8) >> 7) & 0x1; // of = result6 ^ result7
-  SET_FLAGS_OxxxxC(of, cf);
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EbR(bxInstruction_c *i)
 {
   Bit8u result_8;
   unsigned count;
@@ -307,7 +327,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EbR(bxInstruction_c *i)
 
   count &= 0x1f;
 
-  if (!count) return;
+  if (!count) {
+    BX_NEXT_INSTR(i);
+  }
 
   Bit8u op1_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
 
@@ -324,9 +346,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EbR(bxInstruction_c *i)
 
   SET_FLAGS_OSZAPC_LOGIC_8(result_8);
   SET_FLAGS_OxxxxC(of, cf);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EbM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EbM(bxInstruction_c *i)
 {
   Bit8u result_8;
   unsigned count;
@@ -343,7 +367,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EbM(bxInstruction_c *i)
   /* pointer, segment address pair */
   Bit8u op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
 
-  if (!count) return;
+  if (!count) {
+    BX_NEXT_INSTR(i);
+  }
 
   if (count <= 8) {
     result_8 = (op1_8 << count);
@@ -358,9 +384,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EbM(bxInstruction_c *i)
 
   SET_FLAGS_OSZAPC_LOGIC_8(result_8);
   SET_FLAGS_OxxxxC(of, cf);
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_EbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_EbR(bxInstruction_c *i)
 {
   unsigned count;
 
@@ -371,22 +399,24 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_EbR(bxInstruction_c *i)
 
   count &= 0x1f;
 
-  if (!count) return;
+  if (count) {
+    Bit8u op1_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
+    Bit8u result_8 = (op1_8 >> count);
+    BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), result_8);
 
-  Bit8u op1_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
-  Bit8u result_8 = (op1_8 >> count);
-  BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), result_8);
+    unsigned cf = (op1_8 >> (count - 1)) & 0x1;
+    // note, that of == result7 if count == 1 and
+    //            of == 0       if count >= 2
+    unsigned of = (((result_8 << 1) ^ result_8) >> 7) & 0x1;
 
-  unsigned cf = (op1_8 >> (count - 1)) & 0x1;
-  // note, that of == result7 if count == 1 and
-  //            of == 0       if count >= 2
-  unsigned of = (((result_8 << 1) ^ result_8) >> 7) & 0x1;
+    SET_FLAGS_OSZAPC_LOGIC_8(result_8);
+    SET_FLAGS_OxxxxC(of, cf);
+  }
 
-  SET_FLAGS_OSZAPC_LOGIC_8(result_8);
-  SET_FLAGS_OxxxxC(of, cf);
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_EbM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_EbM(bxInstruction_c *i)
 {
   unsigned count;
 
@@ -401,22 +431,24 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_EbM(bxInstruction_c *i)
   /* pointer, segment address pair */
   Bit8u op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
 
-  if (!count) return;
+  if (count) {
+    Bit8u result_8 = (op1_8 >> count);
 
-  Bit8u result_8 = (op1_8 >> count);
+    write_RMW_virtual_byte(result_8);
 
-  write_RMW_virtual_byte(result_8);
+    unsigned cf = (op1_8 >> (count - 1)) & 0x1;
+    // note, that of == result7 if count == 1 and
+    //            of == 0       if count >= 2
+    unsigned of = (((result_8 << 1) ^ result_8) >> 7) & 0x1;
 
-  unsigned cf = (op1_8 >> (count - 1)) & 0x1;
-  // note, that of == result7 if count == 1 and
-  //            of == 0       if count >= 2
-  unsigned of = (((result_8 << 1) ^ result_8) >> 7) & 0x1;
+    SET_FLAGS_OSZAPC_LOGIC_8(result_8);
+    SET_FLAGS_OxxxxC(of, cf);
+  }
 
-  SET_FLAGS_OSZAPC_LOGIC_8(result_8);
-  SET_FLAGS_OxxxxC(of, cf);
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_EbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_EbR(bxInstruction_c *i)
 {
   unsigned count;
 
@@ -427,20 +459,22 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_EbR(bxInstruction_c *i)
 
   count &= 0x1f;
 
-  if (!count) return;
+  if (count) {
+    Bit8u op1_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
+    Bit8u result_8 = ((Bit8s) op1_8) >> count;
+    BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), result_8);
 
-  Bit8u op1_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
-  Bit8u result_8 = ((Bit8s) op1_8) >> count;
-  BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), result_8);
+    unsigned cf = (((Bit8s) op1_8) >> (count - 1)) & 0x1;
 
-  unsigned cf = (((Bit8s) op1_8) >> (count - 1)) & 0x1;
+    SET_FLAGS_OSZAPC_LOGIC_8(result_8);
+    /* signed overflow cannot happen in SAR instruction */
+    SET_FLAGS_OxxxxC(0, cf);
+  }
 
-  SET_FLAGS_OSZAPC_LOGIC_8(result_8);
-  /* signed overflow cannot happen in SAR instruction */
-  SET_FLAGS_OxxxxC(0, cf);
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_EbM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_EbM(bxInstruction_c *i)
 {
   unsigned count;
 
@@ -455,15 +489,17 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_EbM(bxInstruction_c *i)
   /* pointer, segment address pair */
   Bit8u op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
 
-  if (!count) return;
+  if (count) {
+    Bit8u result_8 = ((Bit8s) op1_8) >> count;
 
-  Bit8u result_8 = ((Bit8s) op1_8) >> count;
+    write_RMW_virtual_byte(result_8);
 
-  write_RMW_virtual_byte(result_8);
+    unsigned cf = (((Bit8s) op1_8) >> (count - 1)) & 0x1;
 
-  unsigned cf = (((Bit8s) op1_8) >> (count - 1)) & 0x1;
+    SET_FLAGS_OSZAPC_LOGIC_8(result_8);
+    /* signed overflow cannot happen in SAR instruction */
+    SET_FLAGS_OxxxxC(0, cf);
+  }
 
-  SET_FLAGS_OSZAPC_LOGIC_8(result_8);
-  /* signed overflow cannot happen in SAR instruction */
-  SET_FLAGS_OxxxxC(0, cf);
+  BX_NEXT_INSTR(i);
 }
