@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: sse_move.cc,v 1.132 2011/02/11 10:08:42 sshwarts Exp $
+// $Id: sse_move.cc 10629 2011-08-27 13:47:16Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003-2011 Stanislav Shwartsman
@@ -26,13 +26,10 @@
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-// Make code more tidy with a few macros.
-#if BX_SUPPORT_X86_64==0
-#define RDI EDI
-#endif
-
 #if BX_CPU_LEVEL >= 6
 
+#include "simd_int.h"
+
 void BX_CPU_C::print_state_SSE(void)
 {
   BX_DEBUG(("MXCSR: 0x%08x\n", BX_MXCSR_REGISTER));
@@ -126,7 +123,7 @@ Bit16u BX_CPU_C::unpack_FPU_TW(Bit16u tag_byte)
 /* ************************************ */
 
 /* 0F AE Grp15 010 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::LDMXCSR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::LDMXCSR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
@@ -137,10 +134,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LDMXCSR(bxInstruction_c *i)
 
   BX_MXCSR_REGISTER = new_mxcsr;
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 0F AE Grp15 011 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::STMXCSR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::STMXCSR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   Bit32u mxcsr = BX_MXCSR_REGISTER & MXCSR_MASK;
@@ -149,10 +148,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::STMXCSR(bxInstruction_c *i)
 
   write_virtual_dword(i->seg(), eaddr, mxcsr);
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 0F AE Grp15 000 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::FXSAVE(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FXSAVE(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   unsigned index;
@@ -245,11 +246,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FXSAVE(bxInstruction_c *i)
   }
 
 #if BX_SUPPORT_X86_64
-  if (BX_CPU_THIS_PTR efer.get_FFXSR() && CPL == 0 && long64_mode())
-    return; // skip saving of the XMM state
+  if (BX_CPU_THIS_PTR efer.get_FFXSR() && CPL == 0 && long64_mode()) {
+    BX_NEXT_INSTR(i); // skip saving of the XMM state
+  }
 #endif
 
-
   if(BX_CPU_THIS_PTR cr4.get_OSFXSR() && bx_cpuid_support_sse())
   {
     /* store XMM register file */
@@ -265,10 +266,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FXSAVE(bxInstruction_c *i)
 
   /* do not touch reserved fields */
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 0F AE Grp15 001 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::FXRSTOR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FXRSTOR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister xmm;
@@ -289,6 +292,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FXRSTOR(bxInstruction_c *i)
   BX_CPU_THIS_PTR the_i387.swd =  xmm.xmm16u(1);
   BX_CPU_THIS_PTR the_i387.tos = (xmm.xmm16u(1) >> 11) & 0x07;
 
+  /* always set bit 6 as '1 */
+  BX_CPU_THIS_PTR the_i387.cwd =
+     (BX_CPU_THIS_PTR the_i387.cwd & ~FPU_CW_Reserved_Bits) | 0x0040;
+
   /* Restore x87 FPU Opcode */
   /* The lower 11 bits contain the FPU opcode, upper 5 bits are reserved */
   BX_CPU_THIS_PTR the_i387.foo = xmm.xmm16u(3) & 0x7FF;
@@ -357,8 +364,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FXRSTOR(bxInstruction_c *i)
   }
 
 #if BX_SUPPORT_X86_64
-  if (BX_CPU_THIS_PTR efer.get_FFXSR() && CPL == 0 && long64_mode())
-    return; // skip restore of the XMM state
+  if (BX_CPU_THIS_PTR efer.get_FFXSR() && CPL == 0 && long64_mode()) {
+    BX_NEXT_INSTR(i); // skip restore of the XMM state
+  }
 #endif
 
   /* If the OSFXSR bit in CR4 is not set, the FXRSTOR instruction does
@@ -377,6 +385,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FXRSTOR(bxInstruction_c *i)
     }
   }
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* *************************** */
@@ -389,71 +399,75 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FXRSTOR(bxInstruction_c *i)
 /* MOVUPD: 66 0F 10 */
 /* MOVDQU: F3 0F 6F */
 /* LDDQU:  F2 0F F0 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVUPS_VpsWpsM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVUPS_VpsWpsM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op;
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-  read_virtual_dqword(i->seg(), eaddr, (Bit8u *) &op);
-
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), op);
+  read_virtual_dqword(i->seg(), eaddr, &BX_XMM_REG(i->nnn()));
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVUPS_WpsVpsM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVUPS_WpsVpsM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
   write_virtual_dqword(i->seg(), eaddr, &BX_XMM_REG(i->nnn()));
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* MOVAPS:    0F 28 */
 /* MOVAPD: 66 0F 28 */
 /* MOVDQA: F3 0F 6F */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVAPS_VpsWpsR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVAPS_VpsWpsR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   BX_WRITE_XMM_REG(i->nnn(), BX_READ_XMM_REG(i->rm()));
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVAPS_VpsWpsM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVAPS_VpsWpsM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op;
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-  read_virtual_dqword_aligned(i->seg(), eaddr, (Bit8u *) &op);
-
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), op);
+  read_virtual_dqword_aligned(i->seg(), eaddr, &BX_XMM_REG(i->nnn()));
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* MOVAPS:     0F 29 */
 /* MOVNTPS:    0F 2B */
 /* MOVNTPD: 66 0F 2B */
 /* MOVNTDQ: 66 0F E7 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVAPS_WpsVpsM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVAPS_WpsVpsM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
   write_virtual_dqword_aligned(i->seg(), eaddr, &BX_XMM_REG(i->nnn()));
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* F3 0F 10 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSS_VssWssR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSS_VssWssR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   /* If the source operand is an XMM register, the high-order
           96 bits of the destination XMM register are not modified. */
   BX_WRITE_XMM_REG_LO_DWORD(i->nnn(), BX_READ_XMM_REG_LO_DWORD(i->rm()));
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSS_VssWssM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSS_VssWssM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister op;
@@ -465,144 +479,151 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSS_VssWssM(bxInstruction_c *i)
   op.xmm64u(0) = (Bit64u) read_virtual_dword(i->seg(), eaddr);
   op.xmm64u(1) = 0;
 
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), op);
+  BX_WRITE_XMM_REGZ(i->nnn(), op, i->getVL());
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* F3 0F 11 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSS_WssVssM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSS_WssVssM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
   write_virtual_dword(i->seg(), eaddr, BX_READ_XMM_REG_LO_DWORD(i->nnn()));
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* F2 0F 10 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSD_VsdWsdR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSD_VsdWsdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   /* If the source operand is an XMM register, the high-order
           64 bits of the destination XMM register are not modified. */
   BX_WRITE_XMM_REG_LO_QWORD(i->nnn(), BX_READ_XMM_REG_LO_QWORD(i->rm()));
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* MOVHLPS:   0F 12 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVHLPS_VpsWpsR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVHLPS_VpsWpsR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   BX_WRITE_XMM_REG_LO_QWORD(i->nnn(), BX_READ_XMM_REG_HI_QWORD(i->rm()));
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* MOVLPS:    0F 12 */
 /* MOVLPD: 66 0F 12 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVLPS_VpsMq(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVLPS_VpsMq(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
   /* pointer, segment address pair */
   Bit64u val64 = read_virtual_qword(i->seg(), eaddr);
 
-  /* now write result back to destination */
   BX_WRITE_XMM_REG_LO_QWORD(i->nnn(), val64);
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* F2 0F 12 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVDDUP_VpdWqR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVDDUP_VpdWqR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op;
-
-  Bit64u val64 = BX_READ_XMM_REG_LO_QWORD(i->rm());
-
-  op.xmm64u(0) = val64;
-  op.xmm64u(1) = val64;
-
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), op);
+  sse_pbroadcastq(&BX_XMM_REG(i->nnn()), BX_READ_XMM_REG_LO_QWORD(i->rm()));
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* F3 0F 12 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSLDUP_VpsWpsR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSLDUP_VpsWpsR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op;
-
-  op = BX_READ_XMM_REG(i->rm());
+  BxPackedXmmRegister op = BX_READ_XMM_REG(i->rm());
 
   op.xmm32u(1) = op.xmm32u(0);
   op.xmm32u(3) = op.xmm32u(2);
 
-  /* now write result back to destination */
   BX_WRITE_XMM_REG(i->nnn(), op);
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* F3 0F 16 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSHDUP_VpsWpsR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSHDUP_VpsWpsR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op;
-
-  op = BX_READ_XMM_REG(i->rm());
+  BxPackedXmmRegister op = BX_READ_XMM_REG(i->rm());
 
   op.xmm32u(0) = op.xmm32u(1);
   op.xmm32u(2) = op.xmm32u(3);
 
-  /* now write result back to destination */
   BX_WRITE_XMM_REG(i->nnn(), op);
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* MOVLPS:    0F 13 */
 /* MOVLPD: 66 0F 13 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVLPS_MqVps(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVLPS_MqVps(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
   write_virtual_qword(i->seg(), eaddr, BX_XMM_REG_LO_QWORD(i->nnn()));
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* MOVLHPS:   0F 16 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVLHPS_VpsWpsR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVLHPS_VpsWpsR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   BX_WRITE_XMM_REG_HI_QWORD(i->nnn(), BX_READ_XMM_REG_LO_QWORD(i->rm()));
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* MOVHPS:    0F 16 */
 /* MOVHPD: 66 0F 16 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVHPS_VpsMq(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVHPS_VpsMq(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
   /* pointer, segment address pair */
   Bit64u val64 = read_virtual_qword(i->seg(), eaddr);
 
-  /* now write result back to destination */
   BX_WRITE_XMM_REG_HI_QWORD(i->nnn(), val64);
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* MOVHPS:    0F 17 */
 /* MOVHPD: 66 0F 17 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVHPS_MqVps(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVHPS_MqVps(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
   write_virtual_qword(i->seg(), eaddr, BX_XMM_REG_HI_QWORD(i->nnn()));
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 66 0F F7 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MASKMOVDQU_VdqUdq(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MASKMOVDQU_VdqUdq(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   bx_address rdi = RDI & i->asize_mask();
@@ -613,7 +634,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MASKMOVDQU_VdqUdq(bxInstruction_c *i)
   read_virtual_dqword(i->seg(), rdi, (Bit8u *) &temp);
 
   /* no data will be written to memory if mask is all 0s */
-  if ((mask.xmm64u(0) | mask.xmm64u(1)) == 0) return;
+  if ((mask.xmm64u(0) | mask.xmm64u(1)) == 0) {
+    BX_NEXT_INSTR(i);
+  }
 
   for(unsigned j=0; j<16; j++) {
     if(mask.xmmubyte(j) & 0x80) temp.xmmubyte(j) = op.xmmubyte(j);
@@ -622,88 +645,86 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MASKMOVDQU_VdqUdq(bxInstruction_c *i)
   /* and write result back to the memory */
   write_virtual_dqword(i->seg(), rdi, (Bit8u *) &temp);
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 0F 50 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVMSKPS_GdVRps(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVMSKPS_GdVRps(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op = BX_READ_XMM_REG(i->rm());
-  Bit32u val32 = 0;
-
-  if(op.xmm32u(0) & 0x80000000) val32 |= 0x1;
-  if(op.xmm32u(1) & 0x80000000) val32 |= 0x2;
-  if(op.xmm32u(2) & 0x80000000) val32 |= 0x4;
-  if(op.xmm32u(3) & 0x80000000) val32 |= 0x8;
-
-  BX_WRITE_32BIT_REGZ(i->nnn(), val32);
+  Bit32u mask = sse_pmovmskd(&BX_XMM_REG(i->rm()));
+  BX_WRITE_32BIT_REGZ(i->nnn(), mask);
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 66 0F 50 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVMSKPD_GdVRpd(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVMSKPD_GdVRpd(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op = BX_READ_XMM_REG(i->rm());
-  Bit32u val32 = 0;
-
-  if(op.xmm32u(1) & 0x80000000) val32 |= 0x1;
-  if(op.xmm32u(3) & 0x80000000) val32 |= 0x2;
-
-  BX_WRITE_32BIT_REGZ(i->nnn(), val32);
+  Bit32u mask = sse_pmovmskq(&BX_XMM_REG(i->rm()));
+  BX_WRITE_32BIT_REGZ(i->nnn(), mask);
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 66 0F 6E */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVD_VdqEdR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVD_VdqEdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister op;
   op.xmm64u(0) = (Bit64u) BX_READ_32BIT_REG(i->rm());
   op.xmm64u(1) = 0;
 
-  BX_WRITE_XMM_REG(i->nnn(), op);
+  BX_WRITE_XMM_REGZ(i->nnn(), op, i->getVL());
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 #if BX_SUPPORT_X86_64
 
 /* 66 0F 6E */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVQ_VdqEqR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVQ_VdqEqR(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister op;
   op.xmm64u(0) = BX_READ_64BIT_REG(i->rm());
   op.xmm64u(1) = 0;
 
-  BX_WRITE_XMM_REG(i->nnn(), op);
-#endif
+  BX_WRITE_XMM_REGZ(i->nnn(), op, i->getVL());
+
+  BX_NEXT_INSTR(i);
 }
 
 #endif
 
 /* 66 0F 7E */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVD_EdVdR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVD_EdVdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   BX_WRITE_32BIT_REGZ(i->rm(), BX_READ_XMM_REG_LO_DWORD(i->nnn()));
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 #if BX_SUPPORT_X86_64
 
 /* 66 0F 7E */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVQ_EqVqR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVQ_EqVqR(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 6
   BX_WRITE_64BIT_REG(i->rm(), BX_READ_XMM_REG_LO_QWORD(i->nnn()));
-#endif
+
+  BX_NEXT_INSTR(i);
 }
 
 #endif
 
 /* F3 0F 7E */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVQ_VqWqR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVQ_VqWqR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister op;
@@ -711,11 +732,13 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVQ_VqWqR(bxInstruction_c *i)
   op.xmm64u(0) = BX_READ_XMM_REG_LO_QWORD(i->rm());
   op.xmm64u(1) = 0; /* zero-extension to 128 bit */
 
-  BX_WRITE_XMM_REG(i->nnn(), op);
+  BX_WRITE_XMM_REGZ(i->nnn(), op, i->getVL());
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVQ_VqWqM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVQ_VqWqM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister op;
@@ -723,13 +746,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVQ_VqWqM(bxInstruction_c *i)
   op.xmm64u(0) = read_virtual_qword(i->seg(), eaddr);
   op.xmm64u(1) = 0; /* zero-extension to 128 bit */
 
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), op);
+  BX_WRITE_XMM_REGZ(i->nnn(), op, i->getVL());
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* F2 0F D6 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVDQ2Q_PqVRq(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVDQ2Q_PqVRq(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   BX_CPU_THIS_PTR FPU_check_pending_exceptions(); /* check floating point status word for a pending FPU exceptions */
@@ -740,52 +764,37 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVDQ2Q_PqVRq(bxInstruction_c *i)
 
   BX_WRITE_MMX_REG(i->nnn(), mm);
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* F3 0F D6 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVQ2DQ_VdqQq(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVQ2DQ_VdqQq(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   BX_CPU_THIS_PTR FPU_check_pending_exceptions(); /* check floating point status word for a pending FPU exceptions */
   BX_CPU_THIS_PTR prepareFPU2MMX();
 
   BxPackedXmmRegister op;
-  BxPackedMmxRegister mm = BX_READ_MMX_REG(i->rm());
 
-  op.xmm64u(0) = MMXUQ(mm);
+  op.xmm64u(0) = BX_MMX_REG(i->rm());
   op.xmm64u(1) = 0;
 
   BX_WRITE_XMM_REG(i->nnn(), op);
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 66 0F D7 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVMSKB_GdUdq(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVMSKB_GdUdq(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op = BX_READ_XMM_REG(i->rm());
-  Bit32u result = 0;
-
-  if(op.xmmubyte(0x0) & 0x80) result |= 0x0001;
-  if(op.xmmubyte(0x1) & 0x80) result |= 0x0002;
-  if(op.xmmubyte(0x2) & 0x80) result |= 0x0004;
-  if(op.xmmubyte(0x3) & 0x80) result |= 0x0008;
-  if(op.xmmubyte(0x4) & 0x80) result |= 0x0010;
-  if(op.xmmubyte(0x5) & 0x80) result |= 0x0020;
-  if(op.xmmubyte(0x6) & 0x80) result |= 0x0040;
-  if(op.xmmubyte(0x7) & 0x80) result |= 0x0080;
-  if(op.xmmubyte(0x8) & 0x80) result |= 0x0100;
-  if(op.xmmubyte(0x9) & 0x80) result |= 0x0200;
-  if(op.xmmubyte(0xA) & 0x80) result |= 0x0400;
-  if(op.xmmubyte(0xB) & 0x80) result |= 0x0800;
-  if(op.xmmubyte(0xC) & 0x80) result |= 0x1000;
-  if(op.xmmubyte(0xD) & 0x80) result |= 0x2000;
-  if(op.xmmubyte(0xE) & 0x80) result |= 0x4000;
-  if(op.xmmubyte(0xF) & 0x80) result |= 0x8000;
-
-  /* now write result back to destination */
-  BX_WRITE_32BIT_REGZ(i->nnn(), result);
+  Bit32u mask = sse_pmovmskb(&BX_XMM_REG(i->rm()));
+  BX_WRITE_32BIT_REGZ(i->nnn(), mask);
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /* **************************** */
@@ -793,35 +802,25 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVMSKB_GdUdq(bxInstruction_c *i)
 /* **************************** */
 
 /* 0F C3 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVNTI_MdGd(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVNTI_MdGd(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
   write_virtual_dword(i->seg(), eaddr, BX_READ_32BIT_REG(i->nnn()));
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
-#if BX_SUPPORT_X86_64
-
-/* 0F C3 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVNTI_MqGq(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-  write_virtual_qword_64(i->seg(), eaddr, BX_READ_64BIT_REG(i->nnn()));
-#endif
-}
-
-#endif
-
 /* ************************** */
 /* 3-BYTE-OPCODE INSTRUCTIONS */
 /* ************************** */
 
-/* 66 0F 38 20 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVSXBW_VdqWqR(bxInstruction_c *i)
-{
 #if BX_CPU_LEVEL >= 6
+
+/* 66 0F 38 20 */
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVSXBW_VdqWqR(bxInstruction_c *i)
+{
   BxPackedXmmRegister result;
   BxPackedMmxRegister op;
 
@@ -837,15 +836,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVSXBW_VdqWqR(bxInstruction_c *i)
   result.xmm16u(6) = MMXSB6(op);
   result.xmm16u(7) = MMXSB7(op);
 
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), result);
-#endif
+  BX_WRITE_XMM_REGZ(i->nnn(), result, i->getVL());
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 66 0F 38 21 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVSXBD_VdqWdR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVSXBD_VdqWdR(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister result;
   Bit32u val32 = BX_READ_XMM_REG_LO_DWORD(i->rm());
 
@@ -854,77 +852,75 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVSXBD_VdqWdR(bxInstruction_c *i)
   result.xmm32u(2) = (Bit8s) ((val32 >> 16) & 0xFF);
   result.xmm32u(3) = (Bit8s) (val32  >> 24);
 
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), result);
-#endif
+  BX_WRITE_XMM_REGZ(i->nnn(), result, i->getVL());
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 66 0F 38 22 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVSXBQ_VdqWwR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVSXBQ_VdqWwR(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister result;
   Bit16u val16 = BX_READ_XMM_REG_LO_WORD(i->rm());
 
   result.xmm64u(0) = (Bit8s) (val16 & 0xFF);
   result.xmm64u(1) = (Bit8s) (val16 >> 8);
 
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), result);
-#endif
+  BX_WRITE_XMM_REGZ(i->nnn(), result, i->getVL());
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 66 0F 38 23 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVSXWD_VdqWqR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVSXWD_VdqWqR(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister result;
-  Bit64u val64 = BX_READ_XMM_REG_LO_QWORD(i->rm());
+  BxPackedMmxRegister op;
 
-  result.xmm32u(0) = (Bit16s) (val64 & 0xFFFF);
-  result.xmm32u(1) = (Bit16s) ((val64 >> 16) & 0xFFFF);
-  result.xmm32u(2) = (Bit16s) ((val64 >> 32) & 0xFFFF);
-  result.xmm32u(3) = (Bit16s) (val64  >> 48);
+  // use MMX register as 64-bit value with convinient accessors
+  MMXUQ(op) = BX_READ_XMM_REG_LO_QWORD(i->rm());
 
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), result);
-#endif
+  result.xmm32u(0) = (Bit16s) MMXSW0(op);
+  result.xmm32u(1) = (Bit16s) MMXSW1(op);
+  result.xmm32u(2) = (Bit16s) MMXSW2(op);
+  result.xmm32u(3) = (Bit16s) MMXSW3(op);
+
+  BX_WRITE_XMM_REGZ(i->nnn(), result, i->getVL());
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 66 0F 38 24 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVSXWQ_VdqWdR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVSXWQ_VdqWdR(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister result;
   Bit32u val32 = BX_READ_XMM_REG_LO_DWORD(i->rm());
 
   result.xmm64u(0) = (Bit16s) (val32 & 0xFFFF);
   result.xmm64u(1) = (Bit16s) (val32 >> 16);
 
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), result);
-#endif
+  BX_WRITE_XMM_REGZ(i->nnn(), result, i->getVL());
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 66 0F 38 25 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVSXDQ_VdqWqR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVSXDQ_VdqWqR(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister result;
   Bit64u val64 = BX_READ_XMM_REG_LO_QWORD(i->rm());
 
   result.xmm64u(0) = (Bit32s) (val64 & 0xFFFFFFFF);
   result.xmm64u(1) = (Bit32s) (val64 >> 32);
 
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), result);
-#endif
+  BX_WRITE_XMM_REGZ(i->nnn(), result, i->getVL());
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 66 0F 38 30 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVZXBW_VdqWqR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVZXBW_VdqWqR(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister result;
   BxPackedMmxRegister op;
 
@@ -940,15 +936,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVZXBW_VdqWqR(bxInstruction_c *i)
   result.xmm16u(6) = MMXUB6(op);
   result.xmm16u(7) = MMXUB7(op);
 
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), result);
-#endif
+  BX_WRITE_XMM_REGZ(i->nnn(), result, i->getVL());
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 66 0F 38 31 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVZXBD_VdqWdR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVZXBD_VdqWdR(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister result;
   Bit32u val32 = BX_READ_XMM_REG_LO_DWORD(i->rm());
 
@@ -957,118 +952,82 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVZXBD_VdqWdR(bxInstruction_c *i)
   result.xmm32u(2) = (val32 >> 16) & 0xFF;
   result.xmm32u(3) = val32  >> 24;
 
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), result);
-#endif
+  BX_WRITE_XMM_REGZ(i->nnn(), result, i->getVL());
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 66 0F 38 32 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVZXBQ_VdqWwR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVZXBQ_VdqWwR(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister result;
   Bit16u val16 = BX_READ_XMM_REG_LO_WORD(i->rm());
 
   result.xmm64u(0) = val16 & 0xFF;
   result.xmm64u(1) = val16 >> 8;
 
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), result);
-#endif
+  BX_WRITE_XMM_REGZ(i->nnn(), result, i->getVL());
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 66 0F 38 33 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVZXWD_VdqWqR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVZXWD_VdqWqR(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister result;
-  Bit64u val64 = BX_READ_XMM_REG_LO_QWORD(i->rm());
+  BxPackedMmxRegister op;
 
-  result.xmm32u(0) = val64 & 0xFFFF;
-  result.xmm32u(1) = (val64 >> 16) & 0xFFFF;
-  result.xmm32u(2) = (val64 >> 32) & 0xFFFF;
-  result.xmm32u(3) = val64  >> 48;
+  // use MMX register as 64-bit value with convinient accessors
+  MMXUQ(op) = BX_READ_XMM_REG_LO_QWORD(i->rm());
 
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), result);
-#endif
+  result.xmm32u(0) = MMXUW0(op);
+  result.xmm32u(1) = MMXUW1(op);
+  result.xmm32u(2) = MMXUW2(op);
+  result.xmm32u(3) = MMXUW3(op);
+
+  BX_WRITE_XMM_REGZ(i->nnn(), result, i->getVL());
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 66 0F 38 34 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVZXWQ_VdqWdR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVZXWQ_VdqWdR(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister result;
   Bit32u val32 = BX_READ_XMM_REG_LO_DWORD(i->rm());
 
   result.xmm64u(0) = val32 & 0xFFFF;
   result.xmm64u(1) = val32 >> 16;
 
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), result);
-#endif
+  BX_WRITE_XMM_REGZ(i->nnn(), result, i->getVL());
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 66 0F 38 35 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVZXDQ_VdqWqR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVZXDQ_VdqWqR(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister result;
   Bit64u val64 = BX_READ_XMM_REG_LO_QWORD(i->rm());
 
   result.xmm64u(0) = val64 & 0xFFFFFFFF;
   result.xmm64u(1) = val64 >> 32;
 
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), result);
-#endif
+  BX_WRITE_XMM_REGZ(i->nnn(), result, i->getVL());
+
+  BX_NEXT_INSTR(i);
 }
 
 /* 66 0F 3A 0F */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::PALIGNR_VdqWdqIbR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PALIGNR_VdqWdqIbR(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->nnn());
-  BxPackedXmmRegister op2 = BX_READ_XMM_REG(i->rm()), result;
+  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->nnn()), op2 = BX_READ_XMM_REG(i->rm());
 
-  unsigned shift = i->Ib() * 8;
+  sse_palignr(&op2, &op1, i->Ib());
 
-  if(shift == 0) {
-    result.xmm64u(0) = op2.xmm64u(0);
-    result.xmm64u(1) = op2.xmm64u(1);
-  }
-  else if(shift < 64) {
-    result.xmm64u(0) = (op2.xmm64u(0) >> shift) | (op2.xmm64u(1) << (64-shift));
-    result.xmm64u(1) = (op2.xmm64u(1) >> shift) | (op1.xmm64u(0) << (64-shift));
-  }
-  else if(shift == 64) {
-    result.xmm64u(0) = op2.xmm64u(1);
-    result.xmm64u(1) = op1.xmm64u(0);
-  }
-  else if(shift < 128) {
-    shift -= 64;
-    result.xmm64u(0) = (op2.xmm64u(1) >> shift) | (op1.xmm64u(0) << (64-shift));
-    result.xmm64u(1) = (op1.xmm64u(0) >> shift) | (op1.xmm64u(1) << (64-shift));
-  }
-  else if(shift == 128) {
-    result.xmm64u(0) = op1.xmm64u(0);
-    result.xmm64u(1) = op1.xmm64u(1);
-  }
-  else if(shift < 192) {
-    shift -= 128;
-    result.xmm64u(0) = (op1.xmm64u(0) >> shift) | (op1.xmm64u(1) << (64-shift));
-    result.xmm64u(1) = (op1.xmm64u(1) >> shift);
-  }
-  else if(shift < 256) {
-    result.xmm64u(0) = op1.xmm64u(1) >> (shift - 192);
-    result.xmm64u(1) = 0;
-  }
-  else {
-    result.xmm64u(0) = 0;
-    result.xmm64u(1) = 0;
-  }
+  BX_WRITE_XMM_REG(i->nnn(), op2);
 
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), result);
-#endif
+  BX_NEXT_INSTR(i);
 }
+
+#endif
