@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: sse_rcp.cc,v 1.27 2010/12/25 07:59:15 sshwarts Exp $
+// $Id: sse_rcp.cc 10451 2011-07-06 20:01:18Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2003-2010 Stanislav Shwartsman
+//   Copyright (c) 2003-2011 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -295,7 +295,7 @@ static Bit16u rcp_table[2048] = {
 };
 
 // approximate reciprocal of scalar single precision FP
-static float32 approximate_rcp(float32 op)
+float32 approximate_rcp(float32 op)
 {
   float_class_t op_class = float32_class(op);
 
@@ -344,7 +344,7 @@ static float32 approximate_rcp(float32 op)
  * Approximate reciprocals of packed single precision FP values from XMM2/MEM.
  * Possible floating point exceptions: -
  */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCPPS_VpsWpsR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCPPS_VpsWpsR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister op = BX_READ_XMM_REG(i->rm());
@@ -356,6 +356,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCPPS_VpsWpsR(bxInstruction_c *i)
 
   BX_WRITE_XMM_REG(i->nnn(), op);
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /*
@@ -363,13 +365,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCPPS_VpsWpsR(bxInstruction_c *i)
  * Approximate reciprocal of scalar single precision FP value from XMM2/MEM32.
  * Possible floating point exceptions: -
  */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCPSS_VssWssR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCPSS_VssWssR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   float32 op = BX_READ_XMM_REG_LO_DWORD(i->rm());
   op = approximate_rcp(op);
   BX_WRITE_XMM_REG_LO_DWORD(i->nnn(), op);
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 #if BX_CPU_LEVEL >= 6
@@ -640,7 +644,7 @@ Bit16u rsqrt_table1[1024] =
 
 
 // approximate reciprocal sqrt of scalar single precision FP
-static float32 approximate_rsqrt(float32 op)
+float32 approximate_rsqrt(float32 op)
 {
   float_class_t op_class = float32_class(op);
 
@@ -697,13 +701,15 @@ static float32 approximate_rsqrt(float32 op)
  * from XMM2/MEM32.
  * Possible floating point exceptions: -
  */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::RSQRTSS_VssWssR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RSQRTSS_VssWssR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   float32 op = BX_READ_XMM_REG_LO_DWORD(i->rm());
   op = approximate_rsqrt(op);
   BX_WRITE_XMM_REG_LO_DWORD(i->nnn(), op);
 #endif
+
+  BX_NEXT_INSTR(i);
 }
 
 /*
@@ -712,7 +718,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RSQRTSS_VssWssR(bxInstruction_c *i)
  * from XMM2/MEM.
  * Possible floating point exceptions: -
  */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::RSQRTPS_VpsWpsR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RSQRTPS_VpsWpsR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister op = BX_READ_XMM_REG(i->rm());
@@ -724,4 +730,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RSQRTPS_VpsWpsR(bxInstruction_c *i)
 
   BX_WRITE_XMM_REG(i->nnn(), op);
 #endif
+
+  BX_NEXT_INSTR(i);
 }
