@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: extplugin.h,v 1.5 2011/01/23 20:21:21 vruppert Exp $
+// $Id: extplugin.h 10897 2011-12-30 09:10:11Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 // extplugin.h
@@ -20,7 +20,7 @@
 #if BX_PLUGINS
 #if BX_HAVE_LTDL
 #include <ltdl.h>
-#else
+#elif !defined(_MSC_VER)
 #include "ltdl.h"
 #endif
 #endif
@@ -34,17 +34,24 @@ enum plugintype_t {
 
 #define MAX_ARGC 10
 
+typedef int (*plugin_init_t)(struct _plugin_t *plugin, plugintype_t type, int argc, char *argv[]);
+typedef void (*plugin_fini_t)(void);
+
 typedef struct _plugin_t
 {
     plugintype_t type;
     int  initialized;
 #if BX_PLUGINS
+#if defined(_MSC_VER)
+    HINSTANCE handle;
+#else
     lt_dlhandle handle;
+#endif
 #endif
     int  argc;
     char *name, *args, *argv[MAX_ARGC];
-    int  (*plugin_init)(struct _plugin_t *plugin, plugintype_t type, int argc, char *argv[]);
-    void (*plugin_fini)(void);
+    plugin_init_t plugin_init;
+    plugin_fini_t plugin_fini;
 
     struct _plugin_t *next;
 } plugin_t;
