@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: nogui.cc,v 1.31 2010/02/26 14:18:18 sshwarts Exp $
+// $Id: nogui.cc 10600 2011-08-17 22:41:03Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2009  The Bochs Project
@@ -25,6 +25,7 @@
 #define BX_PLUGGABLE
 
 #include "bochs.h"
+#include "plugin.h"
 #include "param_names.h"
 
 #if BX_WITH_NOGUI
@@ -48,7 +49,7 @@ IMPLEMENT_GUI_PLUGIN_CODE(nogui)
 // which there is no support for your native GUI, or if you want to compile
 // bochs without any native GUI support (no output window or
 // keyboard input will be possible).
-// Look in 'x.cc', 'beos.cc', and 'win32.cc' for specific
+// Look in 'x.cc', 'carbon.cc', and 'win32.cc' for specific
 // implementations of this interface.  -Kevin
 
 
@@ -56,10 +57,10 @@ IMPLEMENT_GUI_PLUGIN_CODE(nogui)
 // ::SPECIFIC_INIT()
 //
 // Called from gui.cc, once upon program startup, to allow for the
-// specific GUI code (X11, BeOS, ...) to be initialized.
+// specific GUI code (X11, Win32, ...) to be initialized.
 //
 // argc, argv: these arguments can be used to initialize the GUI with
-//     specific options (X11 options, BeOS options,...)
+//     specific options (X11 options, Win32 options,...)
 //
 // tilewidth, tileheight: for optimization, graphics_tile_update() passes
 //     only updated regions of the screen to the gui code to be redrawn.
