@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////
-// $Id: wx.cc,v 1.105 2010/05/18 15:33:41 vruppert Exp $
+// $Id: wx.cc 10897 2011-12-30 09:10:11Z vruppert $
 /////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2009  The Bochs Project
@@ -974,7 +974,7 @@ void bx_wx_gui_c::handle_events(void)
         switch (event_queue[i].u.toolbar.button) {
           case BX_TOOLBAR_FLOPPYA: floppyA_handler(); break;
           case BX_TOOLBAR_FLOPPYB: floppyB_handler(); break;
-          case BX_TOOLBAR_CDROMD: cdromD_handler(); break;
+          case BX_TOOLBAR_CDROM1: cdrom1_handler(); break;
           case BX_TOOLBAR_RESET: reset_handler(); break;
           case BX_TOOLBAR_POWER: power_handler(); break;
           case BX_TOOLBAR_SAVE_RESTORE: save_restore_handler(); break;
@@ -1083,17 +1083,7 @@ void bx_wx_gui_c::statusbar_setitem(int element, bx_bool active, bx_bool w)
   wxMutexGuiEnter();
   if (element < 0) {
     for (unsigned i = 0; i < statusitem_count; i++) {
-      if (active) {
-#if defined(__WXMSW__)
-        status_text[0] = 9;
-        strcpy(status_text+1, statusitem_text[i]);
-        theFrame->SetStatusText(status_text, i+1);
-#else
-        theFrame->SetStatusText(wxString(statusitem_text[i], wxConvUTF8), i+1);
-#endif
-      } else {
-        theFrame->SetStatusText(wxT(""), i+1);
-      }
+      theFrame->SetStatusText(wxT(""), i+1);
     }
   } else if ((unsigned)element < statusitem_count) {
     if (active) {
@@ -1676,7 +1666,7 @@ void bx_wx_gui_c::show_ips(Bit32u ips_count)
   bx_bool is_main_thread = wxThread::IsMain();
   bx_bool needmutex = !is_main_thread && SIM->is_sim_thread();
   if (needmutex) wxMutexGuiEnter();
-  sprintf(ips_text, "IPS: %9u", ips_count);
+  sprintf(ips_text, "IPS: %3.3fM", ips_count / 1000000.0);
   theFrame->SetStatusText(wxString(ips_text, wxConvUTF8), 0);
   if (needmutex) wxMutexGuiLeave();
 }
