@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////
-// $Id: wxdialog.h,v 1.78 2010/07/03 05:34:27 vruppert Exp $
+// $Id: wxdialog.h 10716 2011-10-01 12:48:48Z vruppert $
 ////////////////////////////////////////////////////////////////////
   
 // wxWidgets dialogs for Bochs
@@ -138,8 +138,8 @@ private:
 "from a particular source.  For example if you are having problems with\n"    \
 "the keyboard, you could ask for debug and info events from the keyboard\n"   \
 "to be reported.")
-#define ADVLOG_OPTS_TYPE_NAMES { wxT("Debug"), wxT("Info"), wxT("Error"), wxT("Panic"), wxT("Pass") }
-#define ADVLOG_OPTS_N_TYPES 5
+#define ADVLOG_OPTS_TYPE_NAMES { wxT("Debug"), wxT("Info"), wxT("Error"), wxT("Panic") }
+#define ADVLOG_OPTS_N_TYPES 4
 #define ADVLOG_DEFAULTS wxT("Use defaults for all devices")
   void Init();  // called automatically by ShowModal()
   void ShowHelp();
