@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: x.cc,v 1.133 2011/02/10 23:00:56 vruppert Exp $
+// $Id: x.cc 10897 2011-12-30 09:10:11Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002-2009  The Bochs Project
+//  Copyright (C) 2001-2011  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -90,27 +90,29 @@ static unsigned long white_pixel=0, black_pixel=0;
 
 static char *progname; /* name this program was invoked by */
 
+// text display
 static unsigned int text_rows=25, text_cols=80;
+static unsigned font_width, font_height;
 static Bit8u h_panning = 0, v_panning = 0;
 static Bit16u line_compare = 1023;
+static unsigned prev_cursor_x=0;
+static unsigned prev_cursor_y=0;
 
+// graphics display
 static Window win;
 static GC gc, gc_inv, gc_headerbar, gc_headerbar_inv;
-static unsigned font_width, font_height;
 static unsigned dimension_x=0, dimension_y=0;
 static unsigned vga_bpp=8;
 
 static XImage *ximage = NULL;
 static unsigned imDepth, imWide, imBPP;
 
-// current cursor coordinates
+// mouse cursor
+static bx_bool mouse_captured = 0;
 static int prev_x=-1, prev_y=-1;
 static int current_x=-1, current_y=-1, current_z=0;
 static unsigned mouse_button_state = 0;
 
-static unsigned prev_cursor_x=0;
-static unsigned prev_cursor_y=0;
-
 static int warp_home_x = 200;
 static int warp_home_y = 200;
 static int mouse_enable_x = 0;
@@ -122,9 +124,11 @@ static void warp_cursor(int dx, int dy);
 static void disable_cursor();
 static void enable_cursor();
 
+// keyboard
+static bx_bool x11_nokeyrepeat = 0;
 static Bit32u convertStringToXKeysym (const char *string);
 
-static bx_bool x_init_done = false;
+static bx_bool x_init_done = 0;
 
 static Pixmap vgafont[256];
 
@@ -621,7 +625,7 @@ void bx_x_gui_c::specific_init(int argc, char **argv, unsigned tilewidth, unsign
   }
   sprintf(bx_status_info_text, "%s enables mouse", get_toggle_info());
 
-  x_init_done = true;
+  x_init_done = 1;
   }
 
   curr_background = 0;
@@ -646,21 +650,21 @@ void bx_x_gui_c::specific_init(int argc, char **argv, unsigned tilewidth, unsign
   // parse x11 specific options
   if (argc > 1) {
     for (i = 1; i < argc; i++) {
+      if (!strcmp(argv[i], "nokeyrepeat")) {
+        BX_INFO(("disabled host keyboard repeat"));
+        x11_nokeyrepeat = 1;
 #if BX_DEBUGGER && BX_DEBUGGER_GUI
-      if (!strcmp(argv[i], "gui_debug")) {
+      } else if (!strcmp(argv[i], "gui_debug")) {
         void InitDebugDialog();
         SIM->set_debug_gui(1);
         InitDebugDialog();
-      }
-      else
 #endif
 #if BX_SHOW_IPS
-      if (!strcmp(argv[i], "hideIPS")) {
+      } else if (!strcmp(argv[i], "hideIPS")) {
+        BX_INFO(("hide IPS display in status bar"));
         x11_hide_ips = 1;
-      }
-      else
 #endif
-      {
+      } else {
         BX_PANIC(("Unknown x11 option '%s'", argv[i]));
       }
     }
@@ -708,7 +712,7 @@ void bx_x_gui_c::statusbar_setitem(int element, bx_bool active, bx_bool w)
 {
   if (element < 0) {
     for (unsigned i = 0; i < statusitem_count; i++) {
-      set_status_text(i+1, statusitem_text[i], active, w);
+      set_status_text(i+1, statusitem_text[i], 0, 0);
     }
   } else if ((unsigned)element < statusitem_count) {
     set_status_text(element+1, statusitem_text[element], active, w);
@@ -719,11 +723,11 @@ void bx_x_gui_c::statusbar_setitem(int element, bx_bool active, bx_bool w)
 // can change because of a gui event such as clicking on the mouse-enable
 // bitmap or pressing the middle button, or from the configuration interface.
 // In all those cases, setting the parameter value will get you here.
-void bx_x_gui_c::mouse_enabled_changed_specific (bx_bool val)
+void bx_x_gui_c::mouse_enabled_changed_specific(bx_bool val)
 {
-  BX_DEBUG (("mouse_enabled=%d, x11 specific code", val?1:0));
+  mouse_captured = val;
   if (val) {
-    BX_INFO(("[x] Mouse on"));
+    BX_INFO(("Mouse capture on"));
     sprintf(bx_status_info_text, "%s disables mouse", get_toggle_info());
     set_status_text(0, bx_status_info_text, 0);
     mouse_enable_x = current_x;
@@ -732,7 +736,7 @@ void bx_x_gui_c::mouse_enabled_changed_specific (bx_bool val)
     // Move the cursor to a 'safe' place
     warp_cursor(warp_home_x-current_x, warp_home_y-current_y);
   } else {
-    BX_INFO(("[x] Mouse off"));
+    BX_INFO(("Mouse capture off"));
     sprintf(bx_status_info_text, "%s enables mouse", get_toggle_info());
     set_status_text(0, bx_status_info_text, 0);
     enable_cursor();
@@ -757,6 +761,27 @@ void create_internal_vga_font(void)
   }
 }
 
+/* Check to see if this is a repeated key. (grabbed from SDL 1.2)
+   (idea shamelessly lifted from GII -- thanks guys! :)
+ */
+static int X11_KeyRepeat(Display *display, XEvent *event)
+{
+  XEvent peekevent;
+  int repeated;
+
+  repeated = 0;
+  if (XPending(display)) {
+    XPeekEvent(display, &peekevent);
+    if ((peekevent.type == KeyPress) &&
+        (peekevent.xkey.keycode == event->xkey.keycode) &&
+        ((peekevent.xkey.time-event->xkey.time) < 2)) {
+      repeated = 1;
+      XNextEvent(display, &peekevent);
+    }
+  }
+  return repeated;
+}
+
 void bx_x_gui_c::handle_events(void)
 {
   XEvent report;
@@ -782,6 +807,11 @@ void bx_x_gui_c::handle_events(void)
   while (XPending(bx_x_display) > 0)  {
     XNextEvent(bx_x_display, &report);
     current_z = 0;
+
+    if (x11_nokeyrepeat && (report.type == KeyRelease) && X11_KeyRepeat(bx_x_display, &report)) {
+      return;
+    }
+
     switch  (report.type) {
 
     case Expose:
@@ -983,22 +1013,14 @@ void send_keyboard_mouse_status(void)
     dz = current_z;
     warp_cursor(warp_home_x-current_x, warp_home_y-current_y);
 
-    DEV_mouse_motion_ext (dx, dy, dz, mouse_button_state);
-    //if (warped) {
-    //  prev_x = current_x = -1;
-    //  prev_y = current_y = -1;
-    //}
-    //else {
-      prev_x = current_x;
-      prev_y = current_y;
-    //}
-  }
-  else {
+    DEV_mouse_motion_ext(dx, dy, dz, mouse_button_state);
+    prev_x = current_x;
+    prev_y = current_y;
+  } else {
     if ((current_x!=-1) && (current_y!=-1)) {
       prev_x = current_x;
       prev_y = current_y;
-    }
-    else {
+    } else {
       prev_x = current_x = -1;
       prev_y = current_y = -1;
     }
@@ -1148,7 +1170,13 @@ void xkeypress(KeySym keysym, int press_release)
       case XK_Control_R:   key_event = BX_KEY_CTRL_R; break;
 #endif
       case XK_Shift_L:     key_event = BX_KEY_SHIFT_L; break;
+#ifdef XK_ISO_Prev_Group
+      case XK_ISO_Prev_Group: key_event = BX_KEY_SHIFT_L; break;
+#endif
       case XK_Shift_R:     key_event = BX_KEY_SHIFT_R; break;
+#ifdef XK_ISO_Next_Group
+      case XK_ISO_Next_Group: key_event = BX_KEY_SHIFT_R; break;
+#endif
       case XK_Alt_L:       key_event = BX_KEY_ALT_L; break;
 #ifdef XK_Alt_R
       case XK_Alt_R:       key_event = BX_KEY_ALT_R; break;
@@ -1164,7 +1192,9 @@ void xkeypress(KeySym keysym, int press_release)
 #ifdef XK_Pause
       case XK_Pause:       key_event = BX_KEY_PAUSE; break;
 #endif
-
+#ifdef XK_Break
+      case XK_Break:       key_event = BX_KEY_CTRL_BREAK; break;
+#endif
       case XK_Insert:      key_event = BX_KEY_INSERT; break;
       case XK_Home:        key_event = BX_KEY_HOME; break;
       case XK_End:         key_event = BX_KEY_END; break;
@@ -1822,9 +1852,7 @@ void bx_x_gui_c::exit(void)
 
 static void warp_cursor (int dx, int dy)
 {
-  if (SIM->get_param_bool(BXPN_MOUSE_ENABLED)->get() &&
-     (warp_dx || warp_dy || dx || dy)
-     ) {
+  if (mouse_captured && (warp_dx || warp_dy || dx || dy)) {
      warp_dx = dx;
      warp_dy = dy;
      XWarpPointer(bx_x_display, None, None, 0, 0, 0, 0, dx, dy);
@@ -1988,14 +2016,23 @@ void bx_x_gui_c::get_capabilities(Bit16u *xres, Bit16u *yres, Bit16u *bpp)
 {
   int num_sizes;
   Rotation original_rotation;
+  int event_base, error_base;
 
   Display *dpy = XOpenDisplay(NULL);
   Window root = RootWindow(dpy, 0);
-  XRRScreenSize *xrrs = XRRSizes(dpy, 0, &num_sizes);
-  XRRScreenConfiguration *conf = XRRGetScreenInfo(dpy, root);
-  SizeID original_size_id = XRRConfigCurrentConfiguration(conf, &original_rotation);
-  *xres = xrrs[original_size_id].width;
-  *yres = xrrs[original_size_id].height;
+
+  if (XRRQueryExtension (dpy, &event_base, &error_base)) {
+    XRRScreenSize *xrrs = XRRSizes(dpy, 0, &num_sizes);
+    XRRScreenConfiguration *conf = XRRGetScreenInfo(dpy, root);
+    SizeID original_size_id = XRRConfigCurrentConfiguration(conf, &original_rotation);
+    *xres = xrrs[original_size_id].width;
+    *yres = xrrs[original_size_id].height;
+  }
+  else {
+    int screen = DefaultScreen(dpy);
+    *xres = DisplayWidth(dpy, screen);
+    *yres = DisplayHeight(dpy, screen);
+   }
   XCloseDisplay(dpy);
   // always return 32 bit depth
   *bpp = 32;
@@ -2006,7 +2043,7 @@ void bx_x_gui_c::show_ips(Bit32u ips_count)
 {
   if (x11_mouse_msg_counter == 0) {
     if (!x11_ips_update && !x11_hide_ips) {
-      sprintf(x11_ips_text, "IPS: %9u", ips_count);
+      sprintf(x11_ips_text, "IPS: %3.3fM", ips_count / 1000000.0);
       x11_ips_update = 1;
     }
   } else {
@@ -2015,21 +2052,169 @@ void bx_x_gui_c::show_ips(Bit32u ips_count)
 }
 #endif
 
-// X11 dialog box functions
+// X11 control class
 
-typedef struct {
-  Window dialog;
-  GC gc, gc_inv;
-} x11_dialog_t;
+enum {
+  XDC_BUTTON = 0,
+  XDC_EDIT,
+  XDC_CHECKBOX
+};
 
-typedef struct {
-  int xmin, xmax, ymin, ymax;
-} x11_control_t;
+class x11_control_c {
+public:
+  x11_control_c(int type, int x, int y, unsigned int w,
+                unsigned int h, const char *text);
+  virtual ~x11_control_c();
 
-void x11_create_dialog(x11_dialog_t *xdlg, char *name, int width, int height)
+  void draw(Display *display, Window win, GC gc);
+  void draw_rect(Display *display, Window win, GC gc);
+  void draw_text(Display *display, Window win, GC gc, const char *text);
+  int  test(XButtonEvent *bev);
+  int  get_type() {return type;}
+  const char* get_text() {return text;}
+  // checkbox
+  int  get_status() {return status;}
+  void set_status(bx_bool new_stat) {status = new_stat;}
+  // edit
+  void set_maxlen(unsigned int max);
+  int process_input(KeySym key, const char *str);
+  const char* get_value() {return value;}
+private:
+  unsigned int width, height;
+  int type, xmin, xmax, ymin, ymax;
+  const char *text;
+  // checkbox
+  bx_bool status;
+  // edit
+  char *value;
+  char editstr[27];
+  unsigned int len, pos, max;
+};
+
+x11_control_c::x11_control_c(int _type, int x, int y, unsigned int w,
+                             unsigned int h, const char *_text)
 {
-  Window dialog;
+  type = _type;
+  xmin = x;
+  ymin = y;
+  width = w;
+  height = h;
+  xmax = x + width;
+  ymax = y + height;
+  if (type == XDC_EDIT) {
+    len = strlen(_text);
+    max = len;
+    value = (char*)malloc(max+1);
+    strcpy(value, _text);
+    pos = (len < 25) ? 0 : (len - 24);
+    strncpy(editstr, value+pos, 24);
+    editstr[len-pos] = 0;
+    text = editstr;
+  } else {
+    text = _text;
+    value = NULL;
+    if (type == XDC_CHECKBOX) {
+      status = !strcmp(text, "X");
+    }
+  }
+}
+
+x11_control_c::~x11_control_c()
+{
+  if (value != NULL) free(value);
+}
+
+void x11_control_c::draw(Display *display, Window win, GC gc)
+{
+  int ty;
+
+  XDrawRectangle(display, win, gc, xmin, ymin, width, height);
+  ty = ymin + height - (height - 12) / 2;
+  XDrawImageString(display, win, gc, xmin+4, ty, (char *)text, strlen(text));
+}
+
+void x11_control_c::draw_rect(Display *display, Window win, GC gc)
+{
+  XDrawRectangle(display, win, gc, xmin - 2, ymin - 2, width + 4, height + 4);
+}
+
+void x11_control_c::draw_text(Display *display, Window win, GC gc, const char *_text)
+{
+  int ty;
+
+  ty = ymin + height - (height - 12) / 2;
+  XDrawImageString(display, win, gc, xmin+4, ty, (char *)_text, strlen(_text));
+}
+
+int x11_control_c::test(XButtonEvent *bev)
+{
+  if ((bev->y > ymin) && (bev->y < ymax)) {
+    if ((bev->x > xmin) && (bev->x < xmax)) {
+      return 1;
+    }
+  }
+  return 0;
+}
+
+void x11_control_c::set_maxlen(unsigned int _max)
+{
+  max = _max;
+  value = (char*)realloc(value, max+1);
+}
+
+int x11_control_c::process_input(KeySym key, const char *str)
+{
+  int ret = 0;
+
+  if (key == XK_BackSpace) {
+    if (len > 0) {
+      value[--len] = 0;
+      if (pos > 0) pos--;
+      ret = 1;
+    }
+  } else if ((key == 0) && (len < max)) {
+    strcat(value, str);
+    len = strlen(value);
+    if (len > 24) pos++;
+    ret = 1;
+  }
+  strncpy(editstr, value+pos, 24);
+  editstr[len-pos] = 0;
+  return ret;
+}
+
+// X11 static text structure
+
+typedef struct _x11_static_t {
+  char *text;
+  int x, y;
+  struct _x11_static_t *next;
+} x11_static_t;
+
+// X11 dialog class
+
+class x11_dialog_c {
+public:
+  x11_dialog_c(char *name, int width, int height, int num_ctrls);
+  virtual ~x11_dialog_c();
+
+  x11_control_c* add_control(int type, int x, int y, unsigned int width,
+                             unsigned int height, const char *text);
+  void draw_controls(Display *display);
+  void add_static_text(int x, int y, const char *text, int length);
+  void draw_text(Display *display, int x, int y, const char *text, int length);
+  int run(int start_ctrl, int ok, int cancel);
+private:
+  Window dlgwin;
   GC gc, gc_inv;
+  int ctrl_cnt, cur_ctrl, old_ctrl;
+  x11_control_c **controls;
+  x11_static_t *static_items;
+};
+
+x11_dialog_c::x11_dialog_c(char *name, int width, int height, int num_ctrls)
+{
+  Window dialogw;
   XSizeHints hint;
   unsigned long black_pixel, white_pixel;
 
@@ -2040,19 +2225,19 @@ void x11_create_dialog(x11_dialog_t *xdlg, char *name, int width, int height)
   hint.height = hint.min_height = hint.max_height = height;
   black_pixel = BlackPixel(bx_x_display, bx_x_screen_num);
   white_pixel = WhitePixel(bx_x_display, bx_x_screen_num);
-  dialog = XCreateSimpleWindow(bx_x_display, RootWindow(bx_x_display,bx_x_screen_num),
+  dialogw = XCreateSimpleWindow(bx_x_display, RootWindow(bx_x_display,bx_x_screen_num),
     hint.x, hint.y, hint.width, hint.height, 4, black_pixel, white_pixel);
-  XSetStandardProperties(bx_x_display, dialog, name, name, None, NULL, 0, &hint);
+  XSetStandardProperties(bx_x_display, dialogw, name, name, None, NULL, 0, &hint);
   Atom wm_delete = XInternAtom(bx_x_display, "WM_DELETE_WINDOW", 1);
-  XSetWMProtocols(bx_x_display, dialog, &wm_delete, 1);
+  XSetWMProtocols(bx_x_display, dialogw, &wm_delete, 1);
 
-  gc = XCreateGC(bx_x_display, dialog, 0, 0);
-  gc_inv = XCreateGC(bx_x_display, dialog, 0, 0);
+  gc = XCreateGC(bx_x_display, dialogw, 0, 0);
+  gc_inv = XCreateGC(bx_x_display, dialogw, 0, 0);
   XSetState(bx_x_display, gc_inv, white_pixel, black_pixel, GXcopy, AllPlanes);
-  XSetBackground(bx_x_display,gc,WhitePixel(bx_x_display, bx_x_screen_num));
-  XSetForeground(bx_x_display,gc,BlackPixel(bx_x_display, bx_x_screen_num));
+  XSetBackground(bx_x_display, gc, WhitePixel(bx_x_display, bx_x_screen_num));
+  XSetForeground(bx_x_display, gc, BlackPixel(bx_x_display, bx_x_screen_num));
 
-  XSelectInput(bx_x_display, dialog, ButtonPressMask
+  XSelectInput(bx_x_display, dialogw, ButtonPressMask
                | ButtonReleaseMask
                | KeyPressMask
                | KeyReleaseMask
@@ -2060,115 +2245,125 @@ void x11_create_dialog(x11_dialog_t *xdlg, char *name, int width, int height)
                | PointerMotionMask
                | EnterWindowMask
                | LeaveWindowMask);
-  XMapWindow(bx_x_display, dialog);
+  XMapWindow(bx_x_display, dialogw);
   XFlush(bx_x_display);
-  xdlg->dialog = dialog;
-  xdlg->gc = gc;
-  xdlg->gc_inv = gc_inv;
+  dlgwin = dialogw;
+  ctrl_cnt = num_ctrls;
+  controls = new x11_control_c* [num_ctrls];
+  static_items = NULL;
+  cur_ctrl = 0;
+  old_ctrl = -1;
 }
 
-void x11_create_button(x11_control_t *xbutton, Display *display, x11_dialog_t *xdlg,
-                       int x, int y, unsigned int width, unsigned int height,
-                       const char *text)
+x11_dialog_c::~x11_dialog_c()
 {
-  xbutton->xmin = x;
-  xbutton->ymin = y;
-  xbutton->xmax = x + width;
-  xbutton->ymax = y + height;
-  XDrawRectangle(display, xdlg->dialog, xdlg->gc, x, y, width, height);
-  XDrawImageString(display, xdlg->dialog, xdlg->gc, x+4, y+14, (char *)text, strlen(text));
-}
+  x11_static_t *temp;
 
-int x11_test_control(x11_control_t *xbutton, XButtonEvent *bev)
-{
-  if ((bev->y > xbutton->ymin) && (bev->y < xbutton->ymax)) {
-    if ((bev->x > xbutton->xmin) && (bev->x < xbutton->xmax)) {
-      return 1;
-    }
+  for (int i = 0; i < ctrl_cnt; i++) {
+    delete controls[i];
   }
-  return 0;
+  delete [] controls;
+  while (static_items != NULL) {
+    temp = static_items;
+    static_items = temp->next;
+    delete [] temp->text;
+    delete temp;
+  }
+  XFreeGC(bx_x_display, gc);
+  XFreeGC(bx_x_display, gc_inv);
+  XDestroyWindow(bx_x_display, dlgwin);
 }
 
-int x11_ask_dialog(BxEvent *event)
+x11_control_c* x11_dialog_c::add_control(int type, int x, int y, unsigned int width,
+                                         unsigned int height, const char *text)
+{
+  x11_control_c *xctrl = new x11_control_c(type, x, y, width, height, text);
+  if (cur_ctrl < ctrl_cnt) {
+    controls[cur_ctrl++] = xctrl;
+  }
+  return xctrl;
+}
+
+void x11_dialog_c::draw_controls(Display *display)
+{
+  for (int i = 0; i < ctrl_cnt; i++) {
+    controls[i]->draw(display, dlgwin, gc);
+  }
+}
+
+void x11_dialog_c::add_static_text(int x, int y, const char *text, int length)
+{
+  x11_static_t *static_item, *temp;
+
+  static_item = (x11_static_t*)malloc(sizeof(x11_static_t));
+  static_item->x = x;
+  static_item->y = y;
+  static_item->text = new char[length+1];
+  strncpy(static_item->text, text, length);
+  static_item->text[length] = 0;
+  static_item->next = NULL;
+  if (static_items == NULL) {
+    static_items = static_item;
+  } else {
+    temp = static_items;
+    while (temp->next) {
+      temp = temp->next;
+    }
+    temp->next = static_item;
+  }
+}
+
+void x11_dialog_c::draw_text(Display *display, int x, int y, const char *text, int length)
+{
+  XDrawImageString(display, dlgwin, gc, x, y, text, length);
+}
+
+int x11_dialog_c::run(int start_ctrl, int ok, int cancel)
 {
-#if BX_DEBUGGER || BX_GDBSTUB
-  const int button_x[4] = { 36, 121, 206, 291 };
-  const int ask_code[4] = { BX_LOG_ASK_CHOICE_CONTINUE,
-                            BX_LOG_ASK_CHOICE_CONTINUE_ALWAYS,
-                            BX_LOG_ASK_CHOICE_ENTER_DEBUG,
-                            BX_LOG_ASK_CHOICE_DIE };
-  const int num_ctrls = 4;
-  x11_control_t xbtn_cont, xbtn_acont, xbtn_debug, xbtn_quit;
-#else
-  const int button_x[3] = { 81, 166, 251 };
-  const int ask_code[3] = { BX_LOG_ASK_CHOICE_CONTINUE,
-                            BX_LOG_ASK_CHOICE_CONTINUE_ALWAYS,
-                            BX_LOG_ASK_CHOICE_DIE };
-  const int num_ctrls = 3;
-  x11_control_t xbtn_cont, xbtn_acont, xbtn_quit;
-#endif
-  x11_dialog_t xdlg;
   XEvent xevent;
   KeySym key;
-  int done, i, level, cpos;
-  int retcode = -1;
-  int valid = 0, control = num_ctrls - 1, oldctrl = -1;
-  char name[16], text[10], device[16], message[512];
+  bx_bool init = 0, done = 0, valid = 0, status;
+  int i;
+  char text[10], editstr[27];
+  x11_static_t *temp;
 
-  level = event->u.logmsg.level;
-  strcpy(name, SIM->get_log_level_name(level));
-  sprintf(device, "Device: %s", event->u.logmsg.prefix);
-  sprintf(message, "Message: %s", event->u.logmsg.msg);
-  x11_create_dialog(&xdlg, name, 400, 115);
-  done = 0;
+  if (start_ctrl < 0) {
+    cur_ctrl = ctrl_cnt - 1;
+  } else {
+    cur_ctrl = start_ctrl;
+  }
   while (!done) {
     XNextEvent(bx_x_display, &xevent);
     switch (xevent.type) {
       case Expose:
         if (xevent.xexpose.count == 0) {
-          XDrawImageString(xevent.xexpose.display, xdlg.dialog,
-                           xdlg.gc, 20, 25, device, strlen(device));
-          if (strlen(message) > 62) {
-            cpos = 62;
-            while ((cpos > 0) && (!isspace(message[cpos]))) cpos--;
-            XDrawImageString(xevent.xexpose.display, xdlg.dialog,
-                             xdlg.gc, 20, 45, message, cpos);
-            XDrawImageString(xevent.xexpose.display, xdlg.dialog,
-                             xdlg.gc, 74, 63, message+cpos+1, strlen(message)-cpos-1);
-          } else {
-            XDrawImageString(xevent.xexpose.display, xdlg.dialog,
-                             xdlg.gc, 20, 45, message, strlen(message));
+          temp = static_items;
+          while (temp != NULL) {
+            draw_text(xevent.xexpose.display, temp->x, temp->y, temp->text, strlen(temp->text));
+            temp = temp->next;
           }
-          x11_create_button(&xbtn_cont, xevent.xexpose.display, &xdlg,
-                            button_x[0] + 2, 80, 65, 20, "Continue");
-          x11_create_button(&xbtn_acont, xevent.xexpose.display, &xdlg,
-                            button_x[1] + 2, 80, 65, 20, "Alwayscont");
-#if BX_DEBUGGER || BX_GDBSTUB
-          x11_create_button(&xbtn_debug, xevent.xexpose.display, &xdlg,
-                            button_x[2] + 2, 80, 65, 20, "Debugger");
-#endif
-          x11_create_button(&xbtn_quit, xevent.xexpose.display, &xdlg,
-                            button_x[num_ctrls-1] + 2, 80, 65, 20, "Quit");
-          oldctrl = control - 1;
-          if (oldctrl < 0) oldctrl = 1;
+          for (i = 0; i < ctrl_cnt; i++) {
+            controls[i]->draw(xevent.xexpose.display, dlgwin, gc);
+          }
+          old_ctrl = cur_ctrl - 1;
+          if (old_ctrl < 0) old_ctrl = 1;
+          init = 1;
         }
         break;
       case ButtonPress:
         if (xevent.xbutton.button == Button1) {
-          if (x11_test_control(&xbtn_cont, &xevent.xbutton)) {
-            control = 0;
-            valid = 1;
-          } else if (x11_test_control(&xbtn_acont, &xevent.xbutton)) {
-            control = 1;
-            valid = 1;
-          } else if (x11_test_control(&xbtn_quit, &xevent.xbutton)) {
-            control = num_ctrls - 1;
-            valid = 1;
-#if BX_DEBUGGER || BX_GDBSTUB
-          } else if (x11_test_control(&xbtn_debug, &xevent.xbutton)) {
-            control = 2;
-            valid = 1;
-#endif
+          for (i = 0; i < ctrl_cnt; i++) {
+            if (controls[i]->test(&xevent.xbutton)) {
+              cur_ctrl = i;
+              if (controls[cur_ctrl]->get_type() == XDC_BUTTON) {
+                valid = 1;
+              } else if (controls[cur_ctrl]->get_type() == XDC_CHECKBOX) {
+                status = !controls[cur_ctrl]->get_status();
+                controls[cur_ctrl]->set_status(status);
+                controls[cur_ctrl]->draw_text(bx_x_display, dlgwin, gc, status ? "X":" ");
+              }
+              break;
+            }
           }
         }
         break;
@@ -2180,18 +2375,40 @@ int x11_ask_dialog(BxEvent *event)
       case KeyPress:
         i = XLookupString((XKeyEvent *)&xevent, text, 10, &key, 0);
         if (key == XK_Tab) {
-          control++;
-          if (control >= num_ctrls) control = 0;
+          cur_ctrl++;
+          if (cur_ctrl >= ctrl_cnt) cur_ctrl = 0;
         } else if (key == XK_Escape) {
-          control = num_ctrls - 1;
+          cur_ctrl = cancel;
           done = 1;
+        } else if (controls[cur_ctrl]->get_type() == XDC_EDIT) {
+          if (key == XK_Return) {
+            cur_ctrl = ok;
+            done = 1;
+          } else if (key == XK_BackSpace) {
+            if (controls[cur_ctrl]->process_input(key, "")) {
+              old_ctrl = cur_ctrl ^ 1;
+            }
+          } else if (i == 1) {
+            if (controls[cur_ctrl]->process_input(0, text)) {
+              old_ctrl = cur_ctrl ^ 1;
+            }
+          }
+        } else if (controls[cur_ctrl]->get_type() == XDC_CHECKBOX) {
+          if (key == XK_space) {
+            status = !controls[cur_ctrl]->get_status();
+            controls[cur_ctrl]->set_status(status);
+            controls[cur_ctrl]->draw_text(bx_x_display, dlgwin, gc, status ? "X":" ");
+          } else if (key == XK_Return) {
+            cur_ctrl = ok;
+            done = 1;
+          }
         } else if ((key == XK_space) || (key == XK_Return)) {
           done = 1;
         }
         break;
       case ClientMessage:
         if (!strcmp(XGetAtomName(bx_x_display, xevent.xclient.message_type), "WM_PROTOCOLS")) {
-          control = num_ctrls - 1;
+          cur_ctrl = cancel;
           done = 1;
         }
         break;
@@ -2199,29 +2416,83 @@ int x11_ask_dialog(BxEvent *event)
         valid = 0;
         break;
     }
-    if (control != oldctrl) {
-      XDrawRectangle(bx_x_display, xdlg.dialog, xdlg.gc_inv, button_x[oldctrl], 78, 69, 24);
-      XDrawRectangle(bx_x_display, xdlg.dialog, xdlg.gc, button_x[control], 78, 69, 24);
-      oldctrl = control;
+    if (init && (cur_ctrl != old_ctrl)) {
+      if (controls[old_ctrl]->get_type() == XDC_EDIT) {
+        sprintf(editstr, "%s ", controls[old_ctrl]->get_text());
+        controls[old_ctrl]->draw_text(bx_x_display, dlgwin, gc, editstr);
+      } else {
+        controls[old_ctrl]->draw_rect(bx_x_display, dlgwin, gc_inv);
+      }
+      if (controls[cur_ctrl]->get_type() == XDC_EDIT) {
+        sprintf(editstr, "%s_ ", controls[cur_ctrl]->get_text());
+        controls[cur_ctrl]->draw_text(bx_x_display, dlgwin, gc, editstr);
+      } else {
+        controls[cur_ctrl]->draw_rect(bx_x_display, dlgwin, gc);
+      }
+      old_ctrl = cur_ctrl;
     }
   }
+  return cur_ctrl;
+}
+
+// X11 dialog box functions
+
+int x11_ask_dialog(BxEvent *event)
+{
+  x11_control_c *xbtn_cont, *xbtn_acont, *xbtn_quit;
+#if BX_DEBUGGER || BX_GDBSTUB
+  x11_control_c *xbtn_debug;
+  const int button_x[4] = { 36, 121, 206, 291 };
+  const int ask_code[4] = { BX_LOG_ASK_CHOICE_CONTINUE,
+                            BX_LOG_ASK_CHOICE_CONTINUE_ALWAYS,
+                            BX_LOG_ASK_CHOICE_ENTER_DEBUG,
+                            BX_LOG_ASK_CHOICE_DIE };
+  const int num_ctrls = 4;
+#else
+  const int button_x[3] = { 81, 166, 251 };
+  const int ask_code[3] = { BX_LOG_ASK_CHOICE_CONTINUE,
+                            BX_LOG_ASK_CHOICE_CONTINUE_ALWAYS,
+                            BX_LOG_ASK_CHOICE_DIE };
+  const int num_ctrls = 3;
+#endif
+  int level, cpos;
+  int retcode = -1;
+  int control = num_ctrls - 1;
+  char name[16], device[16], message[512];
+
+  level = event->u.logmsg.level;
+  strcpy(name, SIM->get_log_level_name(level));
+  sprintf(device, "Device: %s", event->u.logmsg.prefix);
+  sprintf(message, "Message: %s", event->u.logmsg.msg);
+  x11_dialog_c *xdlg = new x11_dialog_c(name, 400, 115, num_ctrls);
+  xdlg->add_static_text(20, 25, device, strlen(device));
+  if (strlen(message) > 62) {
+    cpos = 62;
+    while ((cpos > 0) && (!isspace(message[cpos]))) cpos--;
+    xdlg->add_static_text(20, 45, message, cpos);
+    xdlg->add_static_text(74, 63, message+cpos+1, strlen(message)-cpos-1);
+  } else {
+    xdlg->add_static_text(20, 45, message, strlen(message));
+  }
+  xbtn_cont = xdlg->add_control(XDC_BUTTON, button_x[0] + 2, 80, 65, 20, "Continue");
+  xbtn_acont = xdlg->add_control(XDC_BUTTON, button_x[1] + 2, 80, 65, 20, "Alwayscont");
+#if BX_DEBUGGER || BX_GDBSTUB
+  xbtn_debug = xdlg->add_control(XDC_BUTTON, button_x[2] + 2, 80, 65, 20, "Debugger");
+#endif
+  xbtn_quit = xdlg->add_control(XDC_BUTTON, button_x[num_ctrls-1] + 2, 80, 65, 20, "Quit");
+  control = xdlg->run(num_ctrls-1, 0, num_ctrls-1);
   retcode = ask_code[control];
-  XFreeGC(bx_x_display, xdlg.gc);
-  XFreeGC(bx_x_display, xdlg.gc_inv);
-  XDestroyWindow(bx_x_display, xdlg.dialog);
+  delete xdlg;
   return retcode;
 }
 
 int x11_string_dialog(bx_param_string_c *param, bx_param_bool_c *param2)
 {
-  x11_dialog_t xdlg;
-  x11_control_t xctl_edit, xbtn_ok, xbtn_cancel, xbtn_status;
-  XEvent xevent;
-  KeySym key;
-  int valid = 0, control = 0, oldctrl = -1, num_ctrls;
-  int done, h, i, ok_button, status = 0;
-  unsigned int len, max, pos = 0;
-  char editstr[26], name[80], text[10], value[BX_PATHNAME_LEN];
+  x11_control_c *xctl_edit, *xbtn_ok, *xbtn_cancel, *xbtn_status = NULL;
+  int control = 0, h, num_ctrls, ok_button;
+  bx_bool status = 0;
+  char name[80], text[10];
+  const char *value;
 
   if (param2 != NULL) {
     strcpy(name, "First CD-ROM image/device");
@@ -2239,164 +2510,23 @@ int x11_string_dialog(bx_param_string_c *param, bx_param_bool_c *param2)
     ok_button = 1;
     num_ctrls = 3;
   }
-  strcpy(value, param->getptr());
-  len = strlen(value);
-  max = param->get_maxsize();
-  x11_create_dialog(&xdlg, name, 250, h);
-  done = 0;
-  while (!done) {
-    XNextEvent(bx_x_display, &xevent);
-    switch (xevent.type) {
-      case Expose:
-        if (xevent.xexpose.count == 0) {
-          if (len < 25) {
-            sprintf(editstr, "%s%s", value, "_ ");
-          } else {
-            pos = len - 24;
-            strncpy(editstr, value+pos, 24);
-            editstr[24] = 0;
-            strcat(editstr, "_");
-          }
-          x11_create_button(&xctl_edit, xevent.xexpose.display, &xdlg, 45, 20, 160, 20,
-                            editstr);
-          x11_create_button(&xbtn_ok, xevent.xexpose.display, &xdlg, 55, h - 30, 65, 20,
-                            "OK");
-          x11_create_button(&xbtn_cancel, xevent.xexpose.display, &xdlg, 130, h - 30, 65, 20,
-                            "Cancel");
-          if (param2 != NULL) {
-            if (status == 1) {
-              strcpy(text, "X");
-            } else {
-              strcpy(text, " ");
-            }
-            x11_create_button(&xbtn_status, xevent.xexpose.display, &xdlg, 45, 50, 15, 16,
-                              text);
-            XDrawImageString(bx_x_display, xdlg.dialog, xdlg.gc, 70, 62, "Inserted", 8);
-          }
-          oldctrl = control - 1;
-          if (oldctrl < 0) oldctrl = 1;
-        }
-        break;
-      case ButtonPress:
-        if (xevent.xbutton.button == Button1) {
-          if (x11_test_control(&xctl_edit, &xevent.xbutton)) {
-            control = 0;
-            valid = 1;
-          } else if (x11_test_control(&xbtn_status, &xevent.xbutton)) {
-            control = 1;
-            valid = 1;
-            status ^= 1;
-            if (status == 1) {
-              strcpy(text, "X");
-            } else {
-              strcpy(text, " ");
-            }
-            XDrawImageString(bx_x_display, xdlg.dialog, xdlg.gc, 49, 64, text, 1);
-          } else if (x11_test_control(&xbtn_ok, &xevent.xbutton)) {
-            control = ok_button;
-            valid = 1;
-          } else if (x11_test_control(&xbtn_cancel, &xevent.xbutton)) {
-            control = num_ctrls - 1;
-            valid = 1;
-          }
-        }
-        break;
-      case ButtonRelease:
-        if ((xevent.xbutton.button == Button1) && (valid == 1)) {
-          if (control >= ok_button) {
-            done = 1;
-          }
-        }
-        break;
-      case KeyPress:
-        i = XLookupString((XKeyEvent *)&xevent, text, 10, &key, 0);
-        if (key == XK_Tab) {
-          control++;
-          if (control >= num_ctrls) control = 0;
-        } else if (key == XK_Escape) {
-          control = num_ctrls - 1;
-          done = 1;
-        } else if (control == 0) {
-          if (key == XK_Return) {
-            control = ok_button;
-            done = 1;
-          } else if (key == XK_BackSpace) {
-            if (len > 0) {
-              value[--len] = 0;
-              if (pos > 0) pos--;
-              oldctrl = -1;
-            }
-          } else if ((i == 1) && (len < max)) {
-            strcat(value, text);
-            len = strlen(value);
-            if (len > 24) pos++;
-            oldctrl = -1;
-          }
-        } else if (control >= ok_button) {
-          if ((key == XK_space) || (key == XK_Return)) {
-            done = 1;
-          }
-        } else {
-          if (key == XK_space) {
-            status ^= 1;
-            if (status == 1) {
-              strcpy(text, "X");
-            } else {
-              strcpy(text, " ");
-            }
-            XDrawImageString(bx_x_display, xdlg.dialog, xdlg.gc, 49, 64, text, 1);
-          } else if (key == XK_Return) {
-            control = ok_button;
-            done = 1;
-          }
-        }
-        break;
-      case ClientMessage:
-        if (!strcmp(XGetAtomName(bx_x_display, xevent.xclient.message_type), "WM_PROTOCOLS")) {
-          control = 2;
-          done = 1;
-        }
-        break;
-      case LeaveNotify:
-        valid = 0;
-        break;
-    }
-    if (control != oldctrl) {
-      if (oldctrl >= ok_button) {
-        XDrawRectangle(bx_x_display, xdlg.dialog, xdlg.gc_inv, oldctrl==ok_button?53:128, h - 32, 69, 24);
-      } else if (oldctrl == 1) {
-        XDrawRectangle(bx_x_display, xdlg.dialog, xdlg.gc_inv, 43, 48, 19, 20);
-      } else if (oldctrl == 0) {
-        if (len < 25) {
-          sprintf(editstr, "%s%s", value, " ");
-        } else {
-          strncpy(editstr, value+pos, 24);
-          editstr[24] = 0;
-          strcat(editstr, " ");
-        }
-        XDrawImageString(bx_x_display, xdlg.dialog, xdlg.gc, 49, 34, editstr, strlen(editstr));
-      }
-      if (control >= ok_button) {
-        XDrawRectangle(bx_x_display, xdlg.dialog, xdlg.gc, control==ok_button?53:128, h - 32, 69, 24);
-      } else if (control == 1) {
-        XDrawRectangle(bx_x_display, xdlg.dialog, xdlg.gc, 43, 48, 19, 20);
-      } else if (control == 0) {
-        if (len < 25) {
-          sprintf(editstr, "%s%s", value, "_ ");
-        } else {
-          strncpy(editstr, value+pos, 24);
-          editstr[24] = 0;
-          strcat(editstr, "_");
-        }
-        XDrawImageString(bx_x_display, xdlg.dialog, xdlg.gc, 49, 34, editstr, strlen(editstr));
-      }
-      oldctrl = control;
-    }
+  x11_dialog_c *xdlg = new x11_dialog_c(name, 250, h, num_ctrls);
+  xctl_edit = xdlg->add_control(XDC_EDIT, 45, 20, 160, 20, param->getptr());
+  xctl_edit->set_maxlen(param->get_maxsize());
+  if (param2 != NULL) {
+    strcpy(text, status ? "X":" ");
+    xbtn_status = xdlg->add_control(XDC_CHECKBOX, 45, 50, 15, 16, text);
+    xdlg->add_static_text(70, 62, "Inserted", 8);
   }
+  xbtn_ok = xdlg->add_control(XDC_BUTTON, 55, h - 30, 65, 20, "OK");
+  xbtn_cancel = xdlg->add_control(XDC_BUTTON, 130, h - 30, 65, 20, "Cancel");
+  control = xdlg->run(0, ok_button, num_ctrls-1);
   if (control == ok_button) {
+    value = xctl_edit->get_value();
     if (param2 != NULL) {
+      status = xbtn_status->get_status();
       if (status == 1) {
-        if (len > 0) {
+        if (strlen(value) > 0) {
           param->set(value);
           param2->set(1);
         } else {
@@ -2409,9 +2539,7 @@ int x11_string_dialog(bx_param_string_c *param, bx_param_bool_c *param2)
       param->set(value);
     }
   }
-  XFreeGC(bx_x_display, xdlg.gc);
-  XFreeGC(bx_x_display, xdlg.gc_inv);
-  XDestroyWindow(bx_x_display, xdlg.dialog);
+  delete xdlg;
   if (control == ok_button) {
     return 1;
   } else {
@@ -2421,15 +2549,11 @@ int x11_string_dialog(bx_param_string_c *param, bx_param_bool_c *param2)
 
 int x11_yesno_dialog(bx_param_bool_c *param)
 {
-  x11_dialog_t xdlg;
-  x11_control_t xbtn_yes, xbtn_no;
-  XEvent xevent;
-  KeySym key;
+  x11_control_c *xbtn_yes, *xbtn_no;
   int button_x[2], size_x, size_y;
-  int valid = 0, oldctrl = -1;
-  int control, done, i, ypos;
+  int control, ypos;
   unsigned int cpos1, cpos2, len, maxlen, lines;
-  char name[80], text[10], message[512];
+  char name[80], message[512];
 
   if (param->get_label() != NULL) {
     strcpy(name, param->get_label());
@@ -2439,7 +2563,6 @@ int x11_yesno_dialog(bx_param_bool_c *param)
   strcpy(message, param->get_description());
   cpos1 = 0;
   cpos2 = 0;
-  ypos = 34;
   lines = 0;
   maxlen = 0;
   while (cpos2 < strlen(message)) {
@@ -2447,8 +2570,8 @@ int x11_yesno_dialog(bx_param_bool_c *param)
     while ((cpos2 < strlen(message)) && (message[cpos2] != 0x0a)) cpos2++;
     len = cpos2 - cpos1;
     if (len > maxlen) maxlen = len;
-    cpos1 = cpos2 + 1;
     cpos2++;
+    cpos1 = cpos2;
   }
   if (maxlen < 36) {
     size_x = 250;
@@ -2465,81 +2588,23 @@ int x11_yesno_dialog(bx_param_bool_c *param)
     size_y = 60 + lines * 15;
   }
   control = 1 - param->get();
-  x11_create_dialog(&xdlg, name, size_x, size_y);
-  done = 0;
-  while (!done) {
-    XNextEvent(bx_x_display, &xevent);
-    switch (xevent.type) {
-      case Expose:
-        if (xevent.xexpose.count == 0) {
-          cpos1 = 0;
-          cpos2 = 0;
-          ypos = 34;
-          while (cpos2 < strlen(message)) {
-            while ((cpos2 < strlen(message)) && (message[cpos2] != 0x0a)) cpos2++;
-            len = cpos2 - cpos1;
-            XDrawImageString(bx_x_display, xdlg.dialog,
-                             xdlg.gc, 20, ypos, message+cpos1, len);
-            cpos1 = cpos2 + 1;
-            cpos2++;
-            ypos += 15;
-          }
-          x11_create_button(&xbtn_yes, xevent.xexpose.display, &xdlg, button_x[0], size_y - 30, 65, 20,
-                            "Yes");
-          x11_create_button(&xbtn_no, xevent.xexpose.display, &xdlg, button_x[1], size_y - 30, 65, 20,
-                            "No");
-          oldctrl = control - 1;
-          if (oldctrl < 0) oldctrl = 1;
-        }
-        break;
-      case ButtonPress:
-        if (xevent.xbutton.button == Button1) {
-          if (x11_test_control(&xbtn_yes, &xevent.xbutton)) {
-            control = 0;
-            valid = 1;
-          } else if (x11_test_control(&xbtn_no, &xevent.xbutton)) {
-            control = 1;
-            valid = 1;
-          }
-        }
-        break;
-      case ButtonRelease:
-        if ((xevent.xbutton.button == Button1) && (valid == 1)) {
-          done = 1;
-        }
-        break;
-      case KeyPress:
-        i = XLookupString((XKeyEvent *)&xevent, text, 10, &key, 0);
-        if (key == XK_Tab) {
-          control++;
-          if (control > 1) control = 0;
-        } else if (key == XK_Escape) {
-          control = 1;
-          done = 1;
-        } else if ((key == XK_space) || (key == XK_Return)) {
-          done = 1;
-        }
-        break;
-      case ClientMessage:
-        if (!strcmp(XGetAtomName(bx_x_display, xevent.xclient.message_type), "WM_PROTOCOLS")) {
-          control = 1;
-          done = 1;
-        }
-        break;
-      case LeaveNotify:
-        valid = 0;
-        break;
-    }
-    if (control != oldctrl) {
-      XDrawRectangle(bx_x_display, xdlg.dialog, xdlg.gc_inv, button_x[oldctrl] - 2, size_y - 32, 69, 24);
-      XDrawRectangle(bx_x_display, xdlg.dialog, xdlg.gc, button_x[control] - 2, size_y - 32, 69, 24);
-      oldctrl = control;
-    }
+  x11_dialog_c *xdlg = new x11_dialog_c(name, size_x, size_y, 2);
+  cpos1 = 0;
+  cpos2 = 0;
+  ypos = 34;
+  while (cpos2 < strlen(message)) {
+    while ((cpos2 < strlen(message)) && (message[cpos2] != 0x0a)) cpos2++;
+    len = cpos2 - cpos1;
+    xdlg->add_static_text(20, ypos, message+cpos1, len);
+    cpos2++;
+    cpos1 = cpos2;
+    ypos += 15;
   }
+  xbtn_yes = xdlg->add_control(XDC_BUTTON, button_x[0], size_y - 30, 65, 20, "Yes");
+  xbtn_no = xdlg->add_control(XDC_BUTTON, button_x[1], size_y - 30, 65, 20, "No");
+  control = xdlg->run(control, 0, 1);
   param->set(1 - control);
-  XFreeGC(bx_x_display, xdlg.gc);
-  XFreeGC(bx_x_display, xdlg.gc_inv);
-  XDestroyWindow(bx_x_display, xdlg.dialog);
+  delete xdlg;
   return control;
 }
 
