@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: instrument.h,v 1.36 2009/10/14 20:45:29 sshwarts Exp $
+// $Id: instrument.h 10690 2011-09-25 17:40:41Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2006-2009 Stanislav Shwartsman
@@ -19,38 +19,13 @@
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-
-// possible types passed to BX_INSTR_TLB_CNTRL()
-#define BX_INSTR_MOV_CR3      10
-#define BX_INSTR_INVLPG       11
-#define BX_INSTR_TASKSWITCH   12
-
-// possible types passed to BX_INSTR_CACHE_CNTRL()
-#define BX_INSTR_INVD         20
-#define BX_INSTR_WBINVD       21
-
-// possible types passed to BX_INSTR_FAR_BRANCH()
-#define BX_INSTR_IS_CALL      10
-#define BX_INSTR_IS_RET       11
-#define BX_INSTR_IS_IRET      12
-#define BX_INSTR_IS_JMP       13
-#define BX_INSTR_IS_INT       14
-#define BX_INSTR_IS_SYSCALL   15
-#define BX_INSTR_IS_SYSRET    16
-#define BX_INSTR_IS_SYSENTER  17
-#define BX_INSTR_IS_SYSEXIT   18
-
-// possible types passed to BX_INSTR_PREFETCH_HINT()
-#define BX_INSTR_PREFETCH_NTA 0
-#define BX_INSTR_PREFETCH_T0  1
-#define BX_INSTR_PREFETCH_T1  2
-#define BX_INSTR_PREFETCH_T2  3
-
-
 #if BX_INSTRUMENTATION
 
 class bxInstruction_c;
 
+// define if you want to store instruction opcode bytes in bxInstruction_c
+#define BX_INSTR_STORE_OPCODE_BYTES
+
 void bx_instr_init_env(void);
 void bx_instr_exit_env(void);
 
@@ -58,14 +33,14 @@ void bx_instr_exit_env(void);
 
 void bx_instr_initialize(unsigned cpu);
 void bx_instr_reset(unsigned cpu, unsigned type);
-void bx_instr_new_instruction(unsigned cpu);
 
 void bx_instr_cnear_branch_taken(unsigned cpu, bx_address new_eip);
 void bx_instr_cnear_branch_not_taken(unsigned cpu);
 void bx_instr_ucnear_branch(unsigned cpu, unsigned what, bx_address new_eip);
 void bx_instr_far_branch(unsigned cpu, unsigned what, Bit16u new_cs, bx_address new_eip);
 
-void bx_instr_opcode(unsigned cpu, const Bit8u *opcode, unsigned len, bx_bool is32, bx_bool is64);
+void bx_instr_before_execution(unsigned cpu, bxInstruction_c *i);
+void bx_instr_after_execution(unsigned cpu, bxInstruction_c *i);
 
 void bx_instr_interrupt(unsigned cpu, unsigned vector);
 void bx_instr_exception(unsigned cpu, unsigned vector, unsigned error_code);
@@ -83,7 +58,6 @@ void bx_instr_mem_data_access(unsigned cpu, unsigned seg, bx_address offset, uns
 #define BX_INSTR_RESET(cpu_id, type)     bx_instr_reset(cpu_id, type)
 #define BX_INSTR_HLT(cpu_id)
 #define BX_INSTR_MWAIT(cpu_id, addr, len, flags)
-#define BX_INSTR_NEW_INSTRUCTION(cpu_id) bx_instr_new_instruction(cpu_id)
 
 /* called from command line debugger */
 #define BX_INSTR_DEBUG_PROMPT()
@@ -96,8 +70,7 @@ void bx_instr_mem_data_access(unsigned cpu, unsigned seg, bx_address offset, uns
 #define BX_INSTR_FAR_BRANCH(cpu_id, what, new_cs, new_eip) bx_instr_far_branch(cpu_id, what, new_cs, new_eip)
 
 /* decoding completed */
-#define BX_INSTR_OPCODE(cpu_id, opcode, len, is32, is64) \
-                       bx_instr_opcode(cpu_id, opcode, len, is32, is64)
+#define BX_INSTR_OPCODE(cpu_id, i, opcode, len, is32, is64)
 
 /* exceptional case and interrupt */
 #define BX_INSTR_EXCEPTION(cpu_id, vector, error_code) \
@@ -113,8 +86,8 @@ void bx_instr_mem_data_access(unsigned cpu, unsigned seg, bx_address offset, uns
 #define BX_INSTR_PREFETCH_HINT(cpu_id, what, seg, offset)
 
 /* execution */
-#define BX_INSTR_BEFORE_EXECUTION(cpu_id, i)
-#define BX_INSTR_AFTER_EXECUTION(cpu_id, i)
+#define BX_INSTR_BEFORE_EXECUTION(cpu_id, i) bx_instr_before_execution(cpu_id, i)
+#define BX_INSTR_AFTER_EXECUTION(cpu_id, i) bx_instr_after_execution(cpu_id, i)
 #define BX_INSTR_REPEAT_ITERATION(cpu_id, i)
 
 /* memory access */
@@ -148,7 +121,6 @@ void bx_instr_mem_data_access(unsigned cpu, unsigned seg, bx_address offset, uns
 #define BX_INSTR_RESET(cpu_id, type)
 #define BX_INSTR_HLT(cpu_id)
 #define BX_INSTR_MWAIT(cpu_id, addr, len, flags)
-#define BX_INSTR_NEW_INSTRUCTION(cpu_id)
 
 /* called from command line debugger */
 #define BX_INSTR_DEBUG_PROMPT()
@@ -161,7 +133,7 @@ void bx_instr_mem_data_access(unsigned cpu, unsigned seg, bx_address offset, uns
 #define BX_INSTR_FAR_BRANCH(cpu_id, what, new_cs, new_eip)
 
 /* decoding completed */
-#define BX_INSTR_OPCODE(cpu_id, opcode, len, is32, is64)
+#define BX_INSTR_OPCODE(cpu_id, i, opcode, len, is32, is64)
 
 /* exceptional case and interrupt */
 #define BX_INSTR_EXCEPTION(cpu_id, vector, error_code)
