@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: instrument.h,v 1.35 2009/10/14 20:45:29 sshwarts Exp $
+// $Id: instrument.h 10690 2011-09-25 17:40:41Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2006-2009 Stanislav Shwartsman
@@ -19,44 +19,19 @@
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-
-// possible types passed to BX_INSTR_TLB_CNTRL()
-#define BX_INSTR_MOV_CR3      10
-#define BX_INSTR_INVLPG       11
-#define BX_INSTR_TASKSWITCH   12
-
-// possible types passed to BX_INSTR_CACHE_CNTRL()
-#define BX_INSTR_INVD         20
-#define BX_INSTR_WBINVD       21
-
-// possible types passed to BX_INSTR_FAR_BRANCH()
-#define BX_INSTR_IS_CALL      10
-#define BX_INSTR_IS_RET       11
-#define BX_INSTR_IS_IRET      12
-#define BX_INSTR_IS_JMP       13
-#define BX_INSTR_IS_INT       14
-#define BX_INSTR_IS_SYSCALL   15
-#define BX_INSTR_IS_SYSRET    16
-#define BX_INSTR_IS_SYSENTER  17
-#define BX_INSTR_IS_SYSEXIT   18
-
-// possible types passed to BX_INSTR_PREFETCH_HINT()
-#define BX_INSTR_PREFETCH_NTA 0
-#define BX_INSTR_PREFETCH_T0  1
-#define BX_INSTR_PREFETCH_T1  2
-#define BX_INSTR_PREFETCH_T2  3
-
-
 #if BX_INSTRUMENTATION
 
 class bxInstruction_c;
 
+// define if you want to store instruction opcode bytes in bxInstruction_c
+#define BX_INSTR_STORE_OPCODE_BYTES
+
 void bx_instr_init_env(void);
 void bx_instr_exit_env(void);
 void bx_instr_initialize(unsigned cpu);
 
 // maximum size of an instruction
-#define MAX_OPCODE_SIZE 16
+#define MAX_OPCODE_LENGTH 16
 
 // maximum physical addresses an instruction can generate
 #define MAX_DATA_ACCESSES 1024
@@ -64,14 +39,14 @@ void bx_instr_initialize(unsigned cpu);
 class bxInstrumentation {
 public:
 
-  bx_bool  valid;        // is current instruction valid
-  bx_bool active;        // is active
+  bx_bool ready;          // is current instruction ready to be printed
+  bx_bool active;
 
   unsigned cpu_id;
 
   /* decoding */
-  unsigned opcode_size;
-  Bit8u    opcode[MAX_OPCODE_SIZE];
+  unsigned opcode_length;
+  Bit8u    opcode[MAX_OPCODE_LENGTH];
   bx_bool  is32, is64;
 
   /* memory accesses */
@@ -80,7 +55,7 @@ public:
     bx_address laddr;     // linear address
     bx_phy_address paddr; // physical address
     unsigned op;          // BX_READ, BX_WRITE or BX_RW
-    unsigned size;        // 1 .. 8
+    unsigned size;        // 1 .. 32
   } data_access[MAX_DATA_ACCESSES];
 
   /* branch resolution and target */
@@ -89,7 +64,7 @@ public:
   bx_address target_linear;
 
 public:
-  bxInstrumentation(): valid(0), active(0) {}
+  bxInstrumentation(): ready(0), active(0) {}
 
   void set_cpu_id(unsigned cpu) { cpu_id = cpu; }
 
@@ -99,14 +74,14 @@ public:
   bx_bool is_active() const { return active; }
 
   void bx_instr_reset(unsigned type);
-  void bx_instr_new_instruction();
 
   void bx_instr_cnear_branch_taken(bx_address new_eip);
   void bx_instr_cnear_branch_not_taken();
   void bx_instr_ucnear_branch(unsigned what, bx_address new_eip);
   void bx_instr_far_branch(unsigned what, Bit16u new_cs, bx_address new_eip);
 
-  void bx_instr_opcode(const Bit8u *opcode, unsigned len, bx_bool is32, bx_bool is64);
+  void bx_instr_before_execution(bxInstruction_c *i);
+  void bx_instr_after_execution(bxInstruction_c *i);
 
   void bx_instr_interrupt(unsigned vector);
   void bx_instr_exception(unsigned vector, unsigned error_code);
@@ -116,6 +91,8 @@ public:
 
 private:
   void branch_taken(bx_address new_eip);
+
+  void bx_print_instruction(void);
 };
 
 void bx_instr_init(unsigned cpu);
@@ -127,14 +104,12 @@ extern bxInstrumentation *icpu;
 #define BX_INSTR_EXIT_ENV() bx_instr_exit_env()
 
 /* simulation init, shutdown, reset */
-#define BX_INSTR_INITIALIZE(cpu_id)	   bx_instr_initialize(cpu_id);
+#define BX_INSTR_INITIALIZE(cpu_id)	 bx_instr_initialize(cpu_id)
 #define BX_INSTR_EXIT(cpu_id)
 #define BX_INSTR_RESET(cpu_id, type)     icpu[cpu_id].bx_instr_reset(type)
 #define BX_INSTR_HLT(cpu_id)
 #define BX_INSTR_MWAIT(cpu_id, addr, len, flags)
 
-#define BX_INSTR_NEW_INSTRUCTION(cpu_id) icpu[cpu_id].bx_instr_new_instruction()
-
 /* called from command line debugger */
 #define BX_INSTR_DEBUG_PROMPT()
 #define BX_INSTR_DEBUG_CMD(cmd)
@@ -146,8 +121,7 @@ extern bxInstrumentation *icpu;
 #define BX_INSTR_FAR_BRANCH(cpu_id, what, new_cs, new_eip) icpu[cpu_id].bx_instr_far_branch(what, new_cs, new_eip)
 
 /* decoding completed */
-#define BX_INSTR_OPCODE(cpu_id, opcode, len, is32, is64) \
-                       icpu[cpu_id].bx_instr_opcode(opcode, len, is32, is64)
+#define BX_INSTR_OPCODE(cpu_id, i, opcode, len, is32, is64)
 
 /* exceptional case and interrupt */
 #define BX_INSTR_EXCEPTION(cpu_id, vector, error_code) \
@@ -163,8 +137,8 @@ extern bxInstrumentation *icpu;
 #define BX_INSTR_PREFETCH_HINT(cpu_id, what, seg, offset)
 
 /* execution */
-#define BX_INSTR_BEFORE_EXECUTION(cpu_id, i)
-#define BX_INSTR_AFTER_EXECUTION(cpu_id, i)
+#define BX_INSTR_BEFORE_EXECUTION(cpu_id, i) icpu[cpu_id].bx_instr_before_execution(i)
+#define BX_INSTR_AFTER_EXECUTION(cpu_id, i) icpu[cpu_id].bx_instr_after_execution(i)
 #define BX_INSTR_REPEAT_ITERATION(cpu_id, i)
 
 /* memory access */
@@ -197,7 +171,6 @@ extern bxInstrumentation *icpu;
 #define BX_INSTR_RESET(cpu_id, type)
 #define BX_INSTR_HLT(cpu_id)
 #define BX_INSTR_MWAIT(cpu_id, addr, len, flags)
-#define BX_INSTR_NEW_INSTRUCTION(cpu_id)
 
 /* called from command line debugger */
 #define BX_INSTR_DEBUG_PROMPT()
@@ -210,7 +183,7 @@ extern bxInstrumentation *icpu;
 #define BX_INSTR_FAR_BRANCH(cpu_id, what, new_cs, new_eip)
 
 /* decoding completed */
-#define BX_INSTR_OPCODE(cpu_id, opcode, len, is32, is64)
+#define BX_INSTR_OPCODE(cpu_id, i, opcode, len, is32, is64)
 
 /* exceptional case and interrupt */
 #define BX_INSTR_EXCEPTION(cpu_id, vector, error_code)
