@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: instrument.h,v 1.2 2010/01/09 15:11:32 sshwarts Exp $
+// $Id: instrument.h 10690 2011-09-25 17:40:41Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2009 Stanislav Shwartsman
@@ -19,37 +19,13 @@
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-
-// possible types passed to BX_INSTR_TLB_CNTRL()
-#define BX_INSTR_MOV_CR3      10
-#define BX_INSTR_INVLPG       11
-#define BX_INSTR_TASKSWITCH   12
-
-// possible types passed to BX_INSTR_CACHE_CNTRL()
-#define BX_INSTR_INVD         20
-#define BX_INSTR_WBINVD       21
-
-// possible types passed to BX_INSTR_FAR_BRANCH()
-#define BX_INSTR_IS_CALL      10
-#define BX_INSTR_IS_RET       11
-#define BX_INSTR_IS_IRET      12
-#define BX_INSTR_IS_JMP       13
-#define BX_INSTR_IS_INT       14
-#define BX_INSTR_IS_SYSCALL   15
-#define BX_INSTR_IS_SYSRET    16
-#define BX_INSTR_IS_SYSENTER  17
-#define BX_INSTR_IS_SYSEXIT   18
-
-// possible types passed to BX_INSTR_PREFETCH_HINT()
-#define BX_INSTR_PREFETCH_NTA 0
-#define BX_INSTR_PREFETCH_T0  1
-#define BX_INSTR_PREFETCH_T1  2
-#define BX_INSTR_PREFETCH_T2  3
-
 #if BX_INSTRUMENTATION
 
 class bxInstruction_c;
 
+// define if you want to store instruction opcode bytes in bxInstruction_c
+//#define BX_INSTR_STORE_OPCODE_BYTES
+
 // called from the CPU core
 
 void bx_instr_initialize(unsigned cpu);
@@ -84,7 +60,7 @@ void bx_instr_before_execution(unsigned cpu, bxInstruction_c *i);
 #define BX_INSTR_FAR_BRANCH(cpu_id, what, new_cs, new_eip)
 
 /* decoding completed */
-#define BX_INSTR_OPCODE(cpu_id, opcode, len, is32, is64)
+#define BX_INSTR_OPCODE(cpu_id, i, opcode, len, is32, is64)
 
 /* exceptional case and interrupt */
 #define BX_INSTR_EXCEPTION(cpu_id, vector, error_code) \
@@ -148,7 +124,7 @@ void bx_instr_before_execution(unsigned cpu, bxInstruction_c *i);
 #define BX_INSTR_FAR_BRANCH(cpu_id, what, new_cs, new_eip)
 
 /* decoding completed */
-#define BX_INSTR_OPCODE(cpu_id, opcode, len, is32, is64)
+#define BX_INSTR_OPCODE(cpu_id, i, opcode, len, is32, is64)
 
 /* exceptional case and interrupt */
 #define BX_INSTR_EXCEPTION(cpu_id, vector, error_code)
