@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: eth_vnet.cc,v 1.31 2011/01/24 20:35:51 vruppert Exp $
+// $Id: eth_vnet.cc 10582 2011-08-16 17:27:27Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2011  The Bochs Project
@@ -34,11 +34,10 @@
 #define BX_PLUGGABLE
 
 #include "iodev.h"
+#include "netmod.h"
 
 #if BX_NETWORKING
 
-#include "eth.h"
-
 #define LOG_THIS netdev->
 
 #define BX_ETH_VNET_LOGGING 1
@@ -64,7 +63,6 @@ static const Bit8u broadcast_ipv4addr[3][4] =
 
 #define ICMP_ECHO_PACKET_MAX  128
 #define LAYER4_LISTEN_MAX  128
-#define DEFAULT_LEASE_TIME 28800
 
 static Bit8u    packet_buffer[BX_PACKET_BUFSIZE];
 static unsigned packet_len;
@@ -102,46 +100,6 @@ typedef void (*layer4_handler_t)(
 
 #define TFTP_BUFFER_SIZE 512
 
-#define BOOTREQUEST 1
-#define BOOTREPLY 2
-
-#define BOOTPOPT_PADDING 0
-#define BOOTPOPT_END 255
-#define BOOTPOPT_SUBNETMASK 1
-#define BOOTPOPT_TIMEOFFSET 2
-#define BOOTPOPT_ROUTER_OPTION 3
-#define BOOTPOPT_DOMAIN_NAMESERVER 6
-#define BOOTPOPT_HOST_NAME 12
-#define BOOTPOPT_DOMAIN_NAME 15
-#define BOOTPOPT_MAX_DATAGRAM_SIZE 22
-#define BOOTPOPT_DEFAULT_IP_TTL 23
-#define BOOTPOPT_BROADCAST_ADDRESS 28
-#define BOOTPOPT_ARPCACHE_TIMEOUT 35
-#define BOOTPOPT_DEFAULT_TCP_TTL 37
-#define BOOTPOPT_NTP_SERVER 42
-#define BOOTPOPT_NETBIOS_NAMESERVER 44
-#define BOOTPOPT_X_FONTSERVER 48
-#define BOOTPOPT_REQUESTED_IP_ADDRESS 50
-#define BOOTPOPT_IP_ADDRESS_LEASE_TIME 51
-#define BOOTPOPT_OPTION_OVRLOAD 52
-#define BOOTPOPT_DHCP_MESSAGETYPE 53
-#define BOOTPOPT_SERVER_IDENTIFIER 54
-#define BOOTPOPT_PARAMETER_REQUEST_LIST 55
-#define BOOTPOPT_MAX_DHCP_MESSAGE_SIZE 57
-#define BOOTPOPT_RENEWAL_TIME 58
-#define BOOTPOPT_REBINDING_TIME 59
-#define BOOTPOPT_CLASS_IDENTIFIER 60
-#define BOOTPOPT_CLIENT_IDENTIFIER 61
-
-#define DHCPDISCOVER 1
-#define DHCPOFFER    2
-#define DHCPREQUEST  3
-#define DHCPDECLINE  4
-#define DHCPACK      5
-#define DHCPNAK      6
-#define DHCPRELEASE  7
-#define DHCPINFORM   8
-
 class bx_vnet_pktmover_c : public eth_pktmover_c {
 public:
   bx_vnet_pktmover_c();
@@ -221,10 +179,7 @@ private:
   bx_bool tftp_write;
   Bit16u tftp_tid;
 
-  Bit8u host_macaddr[6];
-  Bit8u guest_macaddr[6];
-  Bit8u host_ipv4addr[4];
-  Bit8u guest_ipv4addr[4];
+  dhcp_cfg_t dhcp;
 
   struct {
     unsigned ipprotocol;
@@ -263,53 +218,6 @@ protected:
   }
 } bx_vnet_match;
 
-static void put_net2(Bit8u *buf,Bit16u data)
-{
-  *buf = (Bit8u)(data >> 8);
-  *(buf+1) = (Bit8u)(data & 0xff);
-}
-
-static void put_net4(Bit8u *buf,Bit32u data)
-{
-  *buf = (Bit8u)((data >> 24) & 0xff);
-  *(buf+1) = (Bit8u)((data >> 16) & 0xff);
-  *(buf+2) = (Bit8u)((data >> 8) & 0xff);
-  *(buf+3) = (Bit8u)(data & 0xff);
-}
-
-static Bit16u get_net2(const Bit8u *buf)
-{
-  return (((Bit16u)*buf) << 8) |
-         ((Bit16u)*(buf+1));
-}
-
-static Bit32u get_net4(const Bit8u *buf)
-{
-  return (((Bit32u)*buf) << 24) |
-         (((Bit32u)*(buf+1)) << 16) |
-         (((Bit32u)*(buf+2)) << 8) |
-         ((Bit32u)*(buf+3));
-}
-
-static Bit16u ip_checksum(const Bit8u *buf, unsigned buf_len)
-{
-  Bit32u sum = 0;
-  unsigned n;
-
-  for (n = 0; n < buf_len; n++) {
-    if (n & 1) {
-      sum += (Bit32u)(*buf++);
-    } else {
-      sum += (Bit32u)(*buf++) << 8;
-    }
-  }
-  while (sum > 0xffff) {
-    sum = (sum >> 16) + (sum & 0xffff);
-  }
-
-  return (Bit16u)sum;
-}
-
 
 // duplicate the part of tftp_send_data() that constructs the filename
 // but ignore errors since tftp_send_data() will respond for us
@@ -348,12 +256,14 @@ void bx_vnet_pktmover_c::pktmover_init(
   this->tftp_tid = 0;
   this->tftp_write = 0;
 
-  memcpy(&host_macaddr[0], macaddr, 6);
-  memcpy(&guest_macaddr[0], macaddr, 6);
-  host_macaddr[5] = (host_macaddr[5] & (~0x01)) ^ 0x02;
+  memcpy(&dhcp.host_macaddr[0], macaddr, 6);
+  memcpy(&dhcp.guest_macaddr[0], macaddr, 6);
+  dhcp.host_macaddr[5] ^= 0x03;
 
-  memcpy(&host_ipv4addr[0], &default_host_ipv4addr[0], 4);
-  memcpy(&guest_ipv4addr[0], &broadcast_ipv4addr[1][0], 4);
+  memcpy(&dhcp.host_ipv4addr[0], &default_host_ipv4addr[0], 4);
+  memcpy(&dhcp.guest_ipv4addr[0], &broadcast_ipv4addr[1][0], 4);
+  dhcp.default_guest_ipv4addr = default_guest_ipv4addr;
+  memcpy(&dhcp.dns_ipv4addr[0], &broadcast_ipv4addr[0][0], 4);
 
   l4data_used = 0;
 
@@ -372,10 +282,10 @@ void bx_vnet_pktmover_c::pktmover_init(
   fprintf(pktlog_txt, "host MAC address = ");
   int i;
   for (i=0; i<6; i++)
-    fprintf(pktlog_txt, "%02x%s", 0xff & host_macaddr[i], i<5?":" : "\n");
+    fprintf(pktlog_txt, "%02x%s", 0xff & dhcp.host_macaddr[i], i<5?":" : "\n");
   fprintf(pktlog_txt, "guest MAC address = ");
   for (i=0; i<6; i++)
-    fprintf(pktlog_txt, "%02x%s", 0xff & guest_macaddr[i], i<5?":" : "\n");
+    fprintf(pktlog_txt, "%02x%s", 0xff & dhcp.guest_macaddr[i], i<5?":" : "\n");
   fprintf(pktlog_txt, "--\n");
   fflush(pktlog_txt);
 #endif
@@ -410,8 +320,8 @@ void bx_vnet_pktmover_c::guest_to_host(const Bit8u *buf, unsigned io_len)
 
   this->tx_time = (64 + 96 + 4 * 8 + io_len * 8) / 10;
   if ((io_len >= 14) &&
-      (!memcmp(&buf[6],&this->guest_macaddr[0],6)) &&
-      (!memcmp(&buf[0],&this->host_macaddr[0],6) ||
+      (!memcmp(&buf[6],&dhcp.guest_macaddr[0],6)) &&
+      (!memcmp(&buf[0],&dhcp.host_macaddr[0],6) ||
        !memcmp(&buf[0],&broadcast_macaddr[0],6))) {
     switch (get_net2(&buf[12])) {
     case 0x0800: // IPv4.
@@ -499,16 +409,16 @@ void bx_vnet_pktmover_c::process_arp(const Bit8u *buf, unsigned io_len)
     if (buf[19] == 0x04) {
       switch (opcode) {
       case 0x0001: // ARP REQUEST
-        if (!memcmp(&buf[22],&this->guest_macaddr[0],6)) {
-          memcpy(&this->guest_ipv4addr[0],&buf[28],4);
-          if (!memcmp(&buf[38],&this->host_ipv4addr[0],4)) {
+        if (!memcmp(&buf[22],&dhcp.guest_macaddr[0],6)) {
+          memcpy(&dhcp.guest_ipv4addr[0],&buf[28],4);
+          if (!memcmp(&buf[38],&dhcp.host_ipv4addr[0],4)) {
             memcpy(&replybuf[14],&buf[14],6);
             replybuf[20]=0x00;
             replybuf[21]=0x02;
-            memcpy(&replybuf[22],&this->host_macaddr[0],6);
-            memcpy(&replybuf[28],&this->host_ipv4addr[0],4);
-            memcpy(&replybuf[32],&this->guest_macaddr[0],6);
-            memcpy(&replybuf[38],&this->guest_ipv4addr[0],4);
+            memcpy(&replybuf[22],&dhcp.host_macaddr[0],6);
+            memcpy(&replybuf[28],&dhcp.host_ipv4addr[0],4);
+            memcpy(&replybuf[32],&dhcp.guest_macaddr[0],6);
+            memcpy(&replybuf[38],&dhcp.guest_ipv4addr[0],4);
 
             host_to_guest_arp(replybuf,60);
           }
@@ -541,8 +451,8 @@ void bx_vnet_pktmover_c::process_arp(const Bit8u *buf, unsigned io_len)
 
 void bx_vnet_pktmover_c::host_to_guest_arp(Bit8u *buf, unsigned io_len)
 {
-  memcpy(&buf[0],&this->guest_macaddr[0],6);
-  memcpy(&buf[6],&this->host_macaddr[0],6);
+  memcpy(&buf[0],&dhcp.guest_macaddr[0],6);
+  memcpy(&buf[6],&dhcp.host_macaddr[0],6);
   buf[12]=0x08;
   buf[13]=0x06;
   host_to_guest(buf,io_len);
@@ -587,7 +497,7 @@ void bx_vnet_pktmover_c::process_ipv4(const Bit8u *buf, unsigned io_len)
   // Ignore this check to tolerant some cases
   //if (io_len > (14U+total_len)) return;
 
-  if (memcmp(&buf[14+16],host_ipv4addr,4) &&
+  if (memcmp(&buf[14+16],dhcp.host_ipv4addr,4) &&
       memcmp(&buf[14+16],broadcast_ipv4addr[0],4) &&
       memcmp(&buf[14+16],broadcast_ipv4addr[1],4) &&
       memcmp(&buf[14+16],broadcast_ipv4addr[2],4))
@@ -631,14 +541,14 @@ void bx_vnet_pktmover_c::host_to_guest_ipv4(Bit8u *buf, unsigned io_len)
 {
   unsigned l3header_len;
 
-  memcpy(&buf[0],&this->guest_macaddr[0],6);
-  memcpy(&buf[6],&this->host_macaddr[0],6);
+  memcpy(&buf[0],&dhcp.guest_macaddr[0],6);
+  memcpy(&buf[6],&dhcp.host_macaddr[0],6);
   buf[12]=0x08;
   buf[13]=0x00;
   buf[14+0] = (buf[14+0] & 0x0f) | 0x40;
   l3header_len = ((unsigned)(buf[14+0] & 0x0f) << 2);
-  memcpy(&buf[14+12],&this->host_ipv4addr[0],4);
-  memcpy(&buf[14+16],&this->guest_ipv4addr[0],4);
+  memcpy(&buf[14+12],&dhcp.host_ipv4addr[0],4);
+  memcpy(&buf[14+16],&dhcp.guest_ipv4addr[0],4);
   put_net2(&buf[14+10], 0);
   put_net2(&buf[14+10], ip_checksum(&buf[14],l3header_len) ^ (Bit16u)0xffff);
 
@@ -782,8 +692,8 @@ void bx_vnet_pktmover_c::host_to_guest_udpipv4_packet(
   ipbuf[34U-12U]=0;
   ipbuf[34U-11U]=0x11; // UDP
   put_net2(&ipbuf[34U-10U],8U+udpdata_len);
-  memcpy(&ipbuf[34U-8U],host_ipv4addr,4);
-  memcpy(&ipbuf[34U-4U],guest_ipv4addr,4);
+  memcpy(&ipbuf[34U-8U],dhcp.host_ipv4addr,4);
+  memcpy(&ipbuf[34U-4U],dhcp.guest_ipv4addr,4);
   // udp header
   put_net2(&ipbuf[34U+0],source_port);
   put_net2(&ipbuf[34U+2],target_port);
@@ -850,330 +760,13 @@ void bx_vnet_pktmover_c::udpipv4_dhcp_handler_ns(
     unsigned sourceport, unsigned targetport,
     const Bit8u *data, unsigned data_len)
 {
-  const Bit8u *opts;
-  unsigned opts_len;
-  unsigned extcode;
-  unsigned extlen;
-  const Bit8u *extdata;
-  unsigned dhcpmsgtype = 0;
-  bx_bool found_serverid = false;
-  bx_bool found_leasetime = false;
-  bx_bool found_guest_ipaddr = false;
-  bx_bool found_host_name = false;
-  Bit32u leasetime = BX_MAX_BIT32U;
-  const Bit8u *dhcpreqparams = NULL;
-  unsigned dhcpreqparams_len = 0;
-  Bit8u dhcpreqparam_default[8];
-  bx_bool dhcpreqparam_default_validflag = false;
-  unsigned dhcpreqparams_default_len = 0;
-  Bit8u *replyopts;
   Bit8u replybuf[576];
-  char *hostname = NULL;
-  unsigned hostname_len = 0;
+  unsigned opts_len;
 
-  if (data_len < (236U+4U)) return;
-  if (data[0] != BOOTREQUEST) return;
-  if (data[1] != 1 || data[2] != 6) return;
-  if (memcmp(&data[28U],guest_macaddr,6)) return;
-  if (data[236] != 0x63 || data[237] != 0x82 ||
-      data[238] != 0x53 || data[239] != 0x63) return;
-
-  opts = &data[240];
-  opts_len = data_len - 240U;
-
-  while (1) {
-    if (opts_len < 1) {
-      BX_ERROR(("dhcp: invalid request"));
-      return;
-    }
-    extcode = *opts++;
-    opts_len--;
-
-    if (extcode == BOOTPOPT_PADDING) continue;
-    if (extcode == BOOTPOPT_END) break;
-    if (opts_len < 1) {
-      BX_ERROR(("dhcp: invalid request"));
-      return;
-    }
-    extlen = *opts++;
-    opts_len--;
-    if (opts_len < extlen) {
-      BX_ERROR(("dhcp: invalid request"));
-      return;
-    }
-    extdata = opts;
-    opts += extlen;
-    opts_len -= extlen;
-
-    switch (extcode)
-    {
-    case BOOTPOPT_DHCP_MESSAGETYPE:
-      if (extlen != 1)
-        break;
-      dhcpmsgtype = *extdata;
-      break;
-    case BOOTPOPT_PARAMETER_REQUEST_LIST:
-      if (extlen < 1)
-        break;
-      dhcpreqparams = extdata;
-      dhcpreqparams_len = extlen;
-      break;
-    case BOOTPOPT_SERVER_IDENTIFIER:
-      if (extlen != 4)
-        break;
-      if (memcmp(extdata,host_ipv4addr,4)) {
-        BX_INFO(("dhcp: request to another server"));
-        return;
-      }
-      found_serverid = true;
-      break;
-    case BOOTPOPT_IP_ADDRESS_LEASE_TIME:
-      if (extlen != 4)
-        break;
-      leasetime = get_net4(&extdata[0]);
-      found_leasetime = true;
-      break;
-    case BOOTPOPT_REQUESTED_IP_ADDRESS:
-      if (extlen != 4)
-        break;
-      if (!memcmp(extdata,default_guest_ipv4addr,4)) {
-        found_guest_ipaddr = true;
-        memcpy(guest_ipv4addr,default_guest_ipv4addr,4);
-      }
-      break;
-    case BOOTPOPT_HOST_NAME:
-      if (extlen < 1)
-        break;
-      hostname = (char*)malloc(extlen);
-      memcpy(hostname, extdata, extlen);
-      hostname_len = extlen;
-      found_host_name = true;
-      break;
-    default:
-      BX_ERROR(("extcode %d not supported yet", extcode));
-      break;
-    }
+  opts_len = process_dhcp(netdev, data, data_len, replybuf, &dhcp);
+  if (opts_len > 0) {
+    host_to_guest_udpipv4_packet(sourceport, targetport, replybuf, opts_len);
   }
-
-  memset(&dhcpreqparam_default,0,sizeof(dhcpreqparam_default));
-  memset(&replybuf[0],0,sizeof(replybuf));
-  replybuf[0] = BOOTREPLY;
-  replybuf[1] = 1;
-  replybuf[2] = 6;
-  memcpy(&replybuf[4],&data[4],4);
-  memcpy(&replybuf[16],default_guest_ipv4addr,4);
-  memcpy(&replybuf[20],host_ipv4addr,4);
-  memcpy(&replybuf[28],&data[28],6);
-  memcpy(&replybuf[44],"vnet",4);
-  memcpy(&replybuf[108],"pxelinux.0",10);
-  replybuf[236] = 0x63;
-  replybuf[237] = 0x82;
-  replybuf[238] = 0x53;
-  replybuf[239] = 0x63;
-  replyopts = &replybuf[240];
-  opts_len = sizeof(replybuf)/sizeof(replybuf[0])-240;
-  switch (dhcpmsgtype) {
-  case DHCPDISCOVER:
-    BX_INFO(("dhcp server: DHCPDISCOVER"));
-    // reset guest address; answer must be broadcasted to unconfigured IP
-    memcpy(guest_ipv4addr,broadcast_ipv4addr[1],4);
-    *replyopts ++ = BOOTPOPT_DHCP_MESSAGETYPE;
-    *replyopts ++ = 1;
-    *replyopts ++ = DHCPOFFER;
-    opts_len -= 3;
-    dhcpreqparam_default[0] = BOOTPOPT_IP_ADDRESS_LEASE_TIME;
-    dhcpreqparam_default[1] = BOOTPOPT_SERVER_IDENTIFIER;
-    if (found_host_name) {
-      dhcpreqparam_default[2] = BOOTPOPT_HOST_NAME;
-    }
-    dhcpreqparam_default_validflag = true;
-    break;
-  case DHCPREQUEST:
-    BX_INFO(("dhcp server: DHCPREQUEST"));
-    // check ciaddr.
-    if (found_serverid || found_guest_ipaddr || (!memcmp(&data[12],default_guest_ipv4addr,4))) {
-      *replyopts ++ = BOOTPOPT_DHCP_MESSAGETYPE;
-      *replyopts ++ = 1;
-      *replyopts ++ = DHCPACK;
-      opts_len -= 3;
-      dhcpreqparam_default[0] = BOOTPOPT_IP_ADDRESS_LEASE_TIME;
-      if (!found_serverid) {
-        dhcpreqparam_default[1] = BOOTPOPT_SERVER_IDENTIFIER;
-      }
-      dhcpreqparam_default_validflag = true;
-    } else {
-      *replyopts ++ = BOOTPOPT_DHCP_MESSAGETYPE;
-      *replyopts ++ = 1;
-      *replyopts ++ = DHCPNAK;
-      opts_len -= 3;
-      if (found_leasetime) {
-        dhcpreqparam_default[dhcpreqparams_default_len++] = BOOTPOPT_IP_ADDRESS_LEASE_TIME;
-        dhcpreqparam_default_validflag = true;
-      }
-      if (!found_serverid) {
-        dhcpreqparam_default[dhcpreqparams_default_len++] = BOOTPOPT_SERVER_IDENTIFIER;
-        dhcpreqparam_default_validflag = true;
-      }
-    }
-    break;
-  default:
-    BX_ERROR(("dhcp server: unsupported message type %u",dhcpmsgtype));
-    return;
-  }
-
-  while (1) {
-    while (dhcpreqparams_len-- > 0) {
-      switch (*dhcpreqparams++) {
-      case BOOTPOPT_SUBNETMASK:
-        BX_INFO(("provide BOOTPOPT_SUBNETMASK"));
-        if (opts_len < 6) {
-          BX_ERROR(("option buffer is insufficient"));
-          return;
-        }
-        opts_len -= 6;
-        *replyopts ++ = BOOTPOPT_SUBNETMASK;
-        *replyopts ++ = 4;
-        memcpy(replyopts,subnetmask_ipv4addr,4);
-        replyopts += 4;
-        break;
-      case BOOTPOPT_ROUTER_OPTION:
-        BX_INFO(("provide BOOTPOPT_ROUTER_OPTION"));
-        if (opts_len < 6) {
-          BX_ERROR(("option buffer is insufficient"));
-          return;
-        }
-        opts_len -= 6;
-        *replyopts ++ = BOOTPOPT_ROUTER_OPTION;
-        *replyopts ++ = 4;
-        memcpy(replyopts,host_ipv4addr,4);
-        replyopts += 4;
-        break;
-#if 0 // DNS is not implemented.
-      case BOOTPOPT_DOMAIN_NAMESERVER:
-        BX_INFO(("provide BOOTPOPT_DOMAIN_NAMESERVER"));
-        if (opts_len < 6) {
-          BX_ERROR(("option buffer is insufficient"));
-          return;
-        }
-        opts_len -= 6;
-        *replyopts ++ = BOOTPOPT_DOMAIN_NAMESERVER;
-        *replyopts ++ = 4;
-        memcpy(replyopts,host_ipv4addr,4);
-        replyopts += 4;
-        break;
-#endif
-      case BOOTPOPT_BROADCAST_ADDRESS:
-        BX_INFO(("provide BOOTPOPT_BROADCAST_ADDRESS"));
-        if (opts_len < 6) {
-          BX_ERROR(("option buffer is insufficient"));
-          return;
-        }
-        opts_len -= 6;
-        *replyopts ++ = BOOTPOPT_BROADCAST_ADDRESS;
-        *replyopts ++ = 4;
-        memcpy(replyopts,broadcast_ipv4addr[2],4);
-        replyopts += 4;
-        break;
-      case BOOTPOPT_IP_ADDRESS_LEASE_TIME:
-        BX_INFO(("provide BOOTPOPT_IP_ADDRESS_LEASE_TIME"));
-        if (opts_len < 6) {
-          BX_ERROR(("option buffer is insufficient"));
-          return;
-        }
-        opts_len -= 6;
-        *replyopts ++ = BOOTPOPT_IP_ADDRESS_LEASE_TIME;
-        *replyopts ++ = 4;
-        if (leasetime < DEFAULT_LEASE_TIME) {
-          put_net4(replyopts, leasetime);
-        } else {
-          put_net4(replyopts, DEFAULT_LEASE_TIME);
-        }
-        replyopts += 4;
-        break;
-      case BOOTPOPT_SERVER_IDENTIFIER:
-        BX_INFO(("provide BOOTPOPT_SERVER_IDENTIFIER"));
-        if (opts_len < 6) {
-          BX_ERROR(("option buffer is insufficient"));
-          return;
-        }
-        opts_len -= 6;
-        *replyopts ++ = BOOTPOPT_SERVER_IDENTIFIER;
-        *replyopts ++ = 4;
-        memcpy(replyopts,host_ipv4addr,4);
-        replyopts += 4;
-        break;
-      case BOOTPOPT_RENEWAL_TIME:
-        BX_INFO(("provide BOOTPOPT_RENEWAL_TIME"));
-        if (opts_len < 6) {
-          BX_ERROR(("option buffer is insufficient"));
-          return;
-        }
-        opts_len -= 6;
-        *replyopts ++ = BOOTPOPT_RENEWAL_TIME;
-        *replyopts ++ = 4;
-        put_net4(replyopts, 600);
-        replyopts += 4;
-        break;
-      case BOOTPOPT_REBINDING_TIME:
-        BX_INFO(("provide BOOTPOPT_REBINDING_TIME"));
-        if (opts_len < 6) {
-          BX_ERROR(("option buffer is insufficient"));
-          return;
-        }
-        opts_len -= 6;
-        *replyopts ++ = BOOTPOPT_REBINDING_TIME;
-        *replyopts ++ = 4;
-        put_net4(replyopts, 1800);
-        replyopts += 4;
-        break;
-      case BOOTPOPT_HOST_NAME:
-        if (hostname != NULL) {
-          BX_INFO(("provide BOOTPOPT_HOST_NAME"));
-          if (opts_len < (hostname_len + 2)) {
-            free(hostname);
-            BX_ERROR(("option buffer is insufficient"));
-            return;
-          }
-          opts_len -= (hostname_len + 2);
-          *replyopts ++ = BOOTPOPT_HOST_NAME;
-          *replyopts ++ = hostname_len;
-          memcpy(replyopts, hostname, hostname_len);
-          *replyopts += hostname_len;
-          free(hostname);
-          hostname = NULL;
-          break;
-        }
-      default:
-        if (*(dhcpreqparams-1) != 0) {
-          BX_ERROR(("dhcp server: requested parameter %u not supported yet",*(dhcpreqparams-1)));
-        }
-        break;
-      }
-    }
-
-    if (!dhcpreqparam_default_validflag) break;
-    dhcpreqparams = &dhcpreqparam_default[0];
-    dhcpreqparams_len = sizeof(dhcpreqparam_default);
-    dhcpreqparam_default_validflag = false;
-  }
-
-  if (opts_len < 1) {
-    BX_ERROR(("option buffer is insufficient"));
-    return;
-  }
-  opts_len -= 2;
-  *replyopts ++ = BOOTPOPT_END;
-
-  opts_len = replyopts - &replybuf[0];
-  if (opts_len < (236U+64U)) {
-    opts_len = (236U+64U); // BOOTP
-  }
-  if (opts_len < (548U)) {
-    opts_len = 548U; // DHCP
-  }
-  host_to_guest_udpipv4_packet(
-    sourceport, targetport,
-    replybuf, opts_len);
 }
 
 void bx_vnet_pktmover_c::udpipv4_tftp_handler(
