@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: hdimage.cc,v 1.31 2011/01/24 20:35:51 vruppert Exp $
+// $Id: hdimage.cc 10793 2011-11-26 15:09:00Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002-2011  The Bochs Project
@@ -25,6 +25,7 @@
 #define BX_PLUGGABLE
 
 #include "iodev.h"
+#include "cdrom.h"
 #include "hdimage.h"
 #include "vmware3.h"
 #include "vmware4.h"
@@ -33,6 +34,10 @@
 #if BX_HAVE_SYS_MMAN_H
 #include <sys/mman.h>
 #endif
+#ifdef linux
+#include <linux/fs.h>
+#include <sys/ioctl.h>
+#endif
 
 #define LOG_THIS theHDImageCtl->
 
@@ -106,22 +111,6 @@ device_image_t* bx_hdimage_ctl_c::init_image(Bit8u image_mode, Bit64u disk_size,
       hdimage = new volatile_image_t(journal);
       break;
 
-#if BX_COMPRESSED_HD_SUPPORT
-    case BX_HDIMAGE_MODE_Z_UNDOABLE:
-      BX_PANIC(("z-undoable disk support not implemented"));
-#if 0
-      hdimage = new z_undoable_image_t(disk_size, journal);
-#endif
-      break;
-
-    case BX_HDIMAGE_MODE_Z_VOLATILE:
-      BX_PANIC(("z-volatile disk support not implemented"));
-#if 0
-      hdimage = new z_volatile_image_t(disk_size, "journal");
-#endif
-      break;
-#endif //BX_COMPRESSED_HD_SUPPORT
-
     case BX_HDIMAGE_MODE_VVFAT:
       hdimage = new vvfat_image_t(disk_size, journal);
       break;
@@ -133,6 +122,13 @@ device_image_t* bx_hdimage_ctl_c::init_image(Bit8u image_mode, Bit64u disk_size,
   return hdimage;
 }
 
+#ifdef LOWLEVEL_CDROM
+LOWLEVEL_CDROM* bx_hdimage_ctl_c::init_cdrom(const char *dev)
+{
+  return new LOWLEVEL_CDROM(dev);
+}
+#endif
+
 /*** base class device_image_t ***/
 
 device_image_t::device_image_t()
@@ -183,16 +179,22 @@ int default_image_t::open(const char* pathname, int flags)
 #ifndef WIN32
   /* look at size of image file to calculate disk geometry */
   struct stat stat_buf;
-  int ret = fstat(fd, &stat_buf);
-  if (ret) {
+  if (fstat(fd, &stat_buf)) {
     BX_PANIC(("fstat() returns error!"));
   }
-  hd_size = (Bit64u)stat_buf.st_size;
-#endif
-  if ((hd_size % 512) != 0) {
-    BX_PANIC(("size of disk image must be multiple of 512 bytes"));
+#ifdef linux
+  if (stat_buf.st_rdev) { // Is this a special device file (e.g. /dev/sde) ?
+    ioctl(fd, BLKGETSIZE64, &hd_size); // yes it's!
   }
-
+  else
+#endif
+  {
+    hd_size = (Bit64u)stat_buf.st_size; // standard unix procedure to get size of regular files
+  }
+#endif
+  BX_INFO(("hd_size: "FMT_LL"u", hd_size));
+  if (hd_size <= 0) BX_PANIC(("size of disk image not detected / invalid"));
+  if ((hd_size % 512) != 0) BX_PANIC(("size of disk image must be multiple of 512 bytes"));
   return fd;
 }
 
@@ -374,6 +376,7 @@ sparse_image_t::sparse_image_t ()
   mmap_header = NULL;
 #endif
   pagetable = NULL;
+  parent_image = NULL;
 }
 
 /*
@@ -442,7 +445,7 @@ void sparse_image_t::read_header()
 
 #ifdef _POSIX_MAPPED_FILES
 // Try to memory map from the beginning of the file (0 is trivially a page multiple)
- void * mmap_header = mmap(NULL, preamble_size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
+ void *mmap_header = mmap(NULL, preamble_size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
  if (mmap_header == MAP_FAILED)
  {
    BX_INFO(("failed to mmap sparse disk file - using conventional file access"));
@@ -1636,262 +1639,3 @@ ssize_t volatile_image_t::write(const void* buf, size_t count)
   }
   return (ret < 0) ? ret : count;
 }
-
-#if BX_COMPRESSED_HD_SUPPORT
-
-/*** z_ro_image_t function definitions ***/
-
-z_ro_image_t::z_ro_image_t()
-{
-  offset = (Bit64s)0;
-}
-
-int z_ro_image_t::open(const char* pathname)
-{
-  fd = ::open(pathname, O_RDONLY
-#ifdef O_BINARY
-              | O_BINARY
-#endif
-             );
-
-  if (fd < 0)
-  {
-    BX_PANIC(("Could not open '%s' file", pathname));
-    return fd;
-  }
-
-  gzfile = gzdopen(fd, "rb");
-  return 0;
-}
-
-void z_ro_image_t::close()
-{
-  if (fd > -1) {
-    gzclose(gzfile);
-    // ::close(fd);
-  }
-}
-
-Bit64s z_ro_image_t::lseek(Bit64s _offset, int whence)
-{
-  // Only SEEK_SET supported
-  if (whence != SEEK_SET)
-  {
-    BX_PANIC(("lseek on compressed images : only SEEK_SET supported"));
-  }
-
-  // Seeking is expensive on compressed files, so we do it
-  // only when necessary, at the latest moment
-  offset = _offset;
-
-  return offset;
-}
-
-ssize_t z_ro_image_t::read(void* buf, size_t count)
-{
-  gzseek(gzfile, offset, SEEK_SET);
-  return gzread(gzfile, buf, count);
-}
-
-ssize_t z_ro_image_t::write(const void* buf, size_t count)
-{
-  BX_PANIC(("z_ro_image: write not supported"));
-  return 0;
-}
-
-
-/*** z_undoable_image_t function definitions ***/
-
-z_undoable_image_t::z_undoable_image_t(Bit64u _size, const char* _redolog_name)
-{
-  redolog = new redolog_t();
-  ro_disk = new z_ro_image_t();
-  size = _size;
-
-  redolog_name = NULL;
-  if (_redolog_name != NULL) {
-    if (strcmp(_redolog_name,"") != 0) {
-      redolog_name = strdup(_redolog_name);
-    }
-  }
-}
-
-z_undoable_image_t::~z_undoable_image_t()
-{
-  delete redolog;
-  delete ro_disk;
-}
-
-int z_undoable_image_t::open(const char* pathname)
-{
-  char *logname=NULL;
-
-  if (ro_disk->open(pathname)<0)
-    return -1;
-
-  // If redolog name was set
-  if (redolog_name != NULL) {
-    if (strcmp(redolog_name, "") != 0) {
-      logname = (char*)malloc(strlen(redolog_name) + 1);
-      strcpy (logname, redolog_name);
-    }
-  }
-
-  // Otherwise we make up the redolog filename from the pathname
-  if (logname == NULL) {
-    logname = (char*)malloc(strlen(pathname) + UNDOABLE_REDOLOG_EXTENSION_LENGTH + 1);
-    sprintf (logname, "%s%s", pathname, UNDOABLE_REDOLOG_EXTENSION);
-  }
-
-  if (redolog->open(logname, REDOLOG_SUBTYPE_UNDOABLE) < 0)
-  {
-    if (redolog->create(logname, REDOLOG_SUBTYPE_UNDOABLE, size) < 0)
-    {
-      BX_PANIC(("Can't open or create redolog '%s'",logname));
-      return -1;
-    }
-  }
-
-  BX_INFO(("'z-undoable' disk opened, z-ro-file is '%s', redolog is '%s'", pathname, logname));
-  free(logname);
-
-  return 0;
-}
-
-void z_undoable_image_t::close()
-{
-  redolog->close();
-  ro_disk->close();
-
-  if (redolog_name!=NULL)
-    free(redolog_name);
-}
-
-Bit64s z_undoable_image_t::lseek(Bit64s offset, int whence)
-{
-  redolog->lseek(offset, whence);
-  return ro_disk->lseek(offset, whence);
-}
-
-ssize_t z_undoable_image_t::read(void* buf, size_t count)
-{
-  // This should be fixed if count != 512
-  if (redolog->read((char*) buf, count) != count)
-    return ro_disk->read((char*) buf, count);
-  else
-    return count;
-}
-
-ssize_t z_undoable_image_t::write(const void* buf, size_t count)
-{
-  return redolog->write((char*) buf, count);
-}
-
-
-/*** z_volatile_image_t function definitions ***/
-
-z_volatile_image_t::z_volatile_image_t(Bit64u _size, const char* _redolog_name)
-{
-  redolog = new redolog_t();
-  ro_disk = new z_ro_image_t();
-  size = _size;
-
-  redolog_temp = NULL;
-  redolog_name = NULL;
-  if (_redolog_name != NULL) {
-    if (strcmp(_redolog_name,"") != 0) {
-      redolog_name = strdup(_redolog_name);
-    }
-  }
-}
-
-z_volatile_image_t::~z_volatile_image_t()
-{
-  delete redolog;
-  delete ro_disk;
-}
-
-int z_volatile_image_t::open(const char* pathname)
-{
-  int filedes;
-  const char *logname=NULL;
-
-  if (ro_disk->open(pathname)<0)
-    return -1;
-
-  // if redolog name was set
-  if (redolog_name != NULL) {
-    if (strcmp(redolog_name, "") != 0) {
-      logname = redolog_name;
-    }
-  }
-
-  // otherwise use pathname as template
-  if (logname == NULL) {
-    logname = pathname;
-  }
-
-  redolog_temp = (char*)malloc(strlen(logname) + VOLATILE_REDOLOG_EXTENSION_LENGTH + 1);
-  sprintf (redolog_temp, "%s%s", logname, VOLATILE_REDOLOG_EXTENSION);
-
-  filedes = mkstemp (redolog_temp);
-
-  if (filedes < 0)
-  {
-    BX_PANIC(("Can't create volatile redolog '%s'", redolog_temp));
-    return -1;
-  }
-  if (redolog->create(filedes, REDOLOG_SUBTYPE_VOLATILE, size) < 0)
-  {
-    BX_PANIC(("Can't create volatile redolog '%s'", redolog_temp));
-    return -1;
-  }
-
-#if (!defined(WIN32)) && !BX_WITH_MACOS
-  // on unix it is legal to delete an open file
-  unlink(redolog_temp);
-#endif
-
-  BX_INFO(("'z-volatile' disk opened: z-ro-file is '%s', redolog is '%s'", pathname, redolog_temp));
-
-  return 0;
-}
-
-void z_volatile_image_t::close ()
-{
-  redolog->close();
-  ro_disk->close();
-
-#if defined(WIN32) || BX_WITH_MACOS
-  // on non-unix we have to wait till the file is closed to delete it
-  unlink(redolog_temp);
-#endif
-
-  if (redolog_temp!=NULL)
-    free(redolog_temp);
-
-  if (redolog_name!=NULL)
-    free(redolog_name);
-}
-
-Bit64s z_volatile_image_t::lseek(Bit64s offset, int whence)
-{
-  redolog->lseek(offset, whence);
-  return ro_disk->lseek(offset, whence);
-}
-
-ssize_t z_volatile_image_t::read (void* buf, size_t count)
-{
-  // This should be fixed if count != 512
-  if (redolog->read((char*) buf, count) != count)
-    return ro_disk->read((char*) buf, count);
-  else
-    return count;
-}
-
-ssize_t z_volatile_image_t::write (const void* buf, size_t count)
-{
-  return redolog->write((char*) buf, count);
-}
-
-#endif
