@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pcipnic.cc,v 1.37 2009/04/23 15:52:53 vruppert Exp $
+// $Id: pcipnic.cc 10897 2011-12-30 09:10:11Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2003  Fen Systems Ltd.
@@ -30,7 +30,7 @@
 #if BX_SUPPORT_PCI && BX_SUPPORT_PCIPNIC
 
 #include "pci.h"
-#include "eth.h"
+#include "netmod.h"
 #include "pcipnic.h"
 
 #define LOG_THIS thePNICDevice->
@@ -74,33 +74,13 @@ void bx_pcipnic_c::init(void)
                             "Experimental PCI Pseudo NIC");
 
   for (unsigned i=0; i<256; i++) {
-    BX_PNIC_THIS s.pci_conf[i] = 0x0;
+    BX_PNIC_THIS pci_conf[i] = 0x0;
   }
 
-  // This code ripped wholesale from ne2k.cc:
-  // Attach to the simulated ethernet dev
-  const char *ethmod = SIM->get_param_enum("ethmod", base)->get_selected();
-  BX_PNIC_THIS ethdev = eth_locator_c::create(ethmod,
-                                              SIM->get_param_string("ethdev", base)->getptr(),
-                                              (const char *) SIM->get_param_string("macaddr", base)->getptr(),
-                                              rx_handler,
-                                              this,
-                                              SIM->get_param_string("script", base)->getptr());
+  // Attach to the selected ethernet module
+  BX_PNIC_THIS ethdev = DEV_net_init_module(base, rx_handler, this);
 
-  if (BX_PNIC_THIS ethdev == NULL) {
-    BX_PANIC(("could not find eth module %s", ethmod));
-    // if they continue, use null.
-    BX_INFO(("could not find eth module %s - using null instead", ethmod));
-
-    BX_PNIC_THIS ethdev = eth_locator_c::create("null", NULL,
-                                                (const char *) SIM->get_param_string("macaddr", base)->getptr(),
-                                                rx_handler,
-                                                this, "");
-    if (BX_PNIC_THIS ethdev == NULL)
-      BX_PANIC(("could not locate null module"));
-  }
-
-  BX_PNIC_THIS s.base_ioaddr = 0;
+  BX_PNIC_THIS pci_base_address[4] = 0;
 
   BX_INFO(("PCI Pseudo NIC initialized - I/O base and IRQ assigned by PCI BIOS"));
 }
@@ -117,8 +97,8 @@ void bx_pcipnic_c::reset(unsigned type)
     { 0x01, PNIC_PCI_VENDOR >> 8 },
     { 0x02, PNIC_PCI_DEVICE & 0xff },
     { 0x03, PNIC_PCI_DEVICE >> 8 },
-    { 0x04, 0x05 }, { 0x05, 0x00 },	// command_io
-    { 0x06, 0x80 }, { 0x07, 0x02 },	// status
+    { 0x04, 0x01 }, { 0x05, 0x00 }, // command_io
+    { 0x06, 0x00 }, { 0x07, 0x00 }, // status
     { 0x08, 0x01 },                 // revision number
     { 0x09, 0x00 },                 // interface
     { 0x0a, 0x00 },                 // class_sub
@@ -135,7 +115,7 @@ void bx_pcipnic_c::reset(unsigned type)
 
   };
   for (i = 0; i < sizeof(reset_vals) / sizeof(*reset_vals); ++i) {
-      BX_PNIC_THIS s.pci_conf[reset_vals[i].addr] = reset_vals[i].val;
+      BX_PNIC_THIS pci_conf[reset_vals[i].addr] = reset_vals[i].val;
   }
 
   // Set up initial register values
@@ -171,22 +151,22 @@ void bx_pcipnic_c::register_state(void)
   }
   new bx_shadow_data_c(list, "rData", BX_PNIC_THIS s.rData, PNIC_DATA_SIZE);
   new bx_shadow_data_c(list, "recvRing", (Bit8u*)BX_PNIC_THIS s.recvRing, PNIC_RECV_RINGS*PNIC_DATA_SIZE);
-  register_pci_state(list, BX_PNIC_THIS s.pci_conf);
+  register_pci_state(list);
 }
 
 void bx_pcipnic_c::after_restore_state(void)
 {
   if (DEV_pci_set_base_io(BX_PNIC_THIS_PTR, read_handler, write_handler,
-                          &BX_PNIC_THIS s.base_ioaddr,
-                          &BX_PNIC_THIS s.pci_conf[0x10],
+                          &BX_PNIC_THIS pci_base_address[4],
+                          &BX_PNIC_THIS pci_conf[0x20],
                           16, &pnic_iomask[0], "PNIC")) {
-    BX_INFO(("new base address: 0x%04x", BX_PNIC_THIS s.base_ioaddr));
+    BX_INFO(("new base address: 0x%04x", BX_PNIC_THIS pci_base_address[4]));
   }
 }
 
 void bx_pcipnic_c::set_irq_level(bx_bool level)
 {
-  DEV_pci_set_irq(BX_PNIC_THIS s.devfunc, BX_PNIC_THIS s.pci_conf[0x3d], level);
+  DEV_pci_set_irq(BX_PNIC_THIS s.devfunc, BX_PNIC_THIS pci_conf[0x3d], level);
 }
 
 // static IO port read callback handler
@@ -209,7 +189,7 @@ Bit32u bx_pcipnic_c::read(Bit32u address, unsigned io_len)
 
   BX_DEBUG(("register read from address 0x%04x - ", (unsigned) address));
 
-  offset = address - BX_PNIC_THIS s.base_ioaddr;
+  offset = address - BX_PNIC_THIS pci_base_address[4];
 
   switch (offset) {
   case PNIC_REG_STAT:
@@ -258,7 +238,7 @@ void bx_pcipnic_c::write(Bit32u address, Bit32u value, unsigned io_len)
 
   BX_DEBUG(("register write to address 0x%04x - ", (unsigned) address));
 
-  offset = address - BX_PNIC_THIS s.base_ioaddr;
+  offset = address - BX_PNIC_THIS pci_base_address[4];
 
   switch (offset) {
   case PNIC_REG_CMD:
@@ -306,7 +286,7 @@ Bit32u bx_pcipnic_c::pci_read_handler(Bit8u address, unsigned io_len)
   Bit32u value = 0;
 
   for (unsigned i=0; i<io_len; i++) {
-    value |= (BX_PNIC_THIS s.pci_conf[address+i] << (i*8));
+    value |= (BX_PNIC_THIS pci_conf[address+i] << (i*8));
   }
 
   if (io_len == 1)
@@ -332,16 +312,14 @@ void bx_pcipnic_c::pci_write_handler(Bit8u address, Bit32u value, unsigned io_le
 
   for (unsigned i=0; i<io_len; i++) {
     value8 = (value >> (i*8)) & 0xFF;
-    oldval = BX_PNIC_THIS s.pci_conf[address+i];
+    oldval = BX_PNIC_THIS pci_conf[address+i];
     switch (address+i) {
-      case 0x3d: //
-      case 0x05: // disallowing write to command hi-byte
-      case 0x06: // disallowing write to status lo-byte (is that expected?)
+      case 0x04:
+        value8 &= 0x01;
         break;
       case 0x3c:
         if (value8 != oldval) {
           BX_INFO(("new irq line = %d", value8));
-          BX_PNIC_THIS s.pci_conf[address+i] = value8;
         }
         break;
       case 0x20:
@@ -350,16 +328,18 @@ void bx_pcipnic_c::pci_write_handler(Bit8u address, Bit32u value, unsigned io_le
       case 0x22:
       case 0x23:
         baseaddr_change = (value8 != oldval);
+        break;
       default:
-        BX_PNIC_THIS s.pci_conf[address+i] = value8;
+        value8 = oldval;
     }
+    BX_PNIC_THIS pci_conf[address+i] = value8;
   }
   if (baseaddr_change) {
     if (DEV_pci_set_base_io(BX_PNIC_THIS_PTR, read_handler, write_handler,
-                            &BX_PNIC_THIS s.base_ioaddr,
-                            &BX_PNIC_THIS s.pci_conf[0x20],
+                            &BX_PNIC_THIS pci_base_address[4],
+                            &BX_PNIC_THIS pci_conf[0x20],
                             16, &pnic_iomask[0], "PNIC")) {
-      BX_INFO(("new base address: 0x%04x", BX_PNIC_THIS s.base_ioaddr));
+      BX_INFO(("new base address: 0x%04x", BX_PNIC_THIS pci_base_address[4]));
     }
   }
 
