@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pcivga.cc,v 1.30 2009/04/23 15:52:53 vruppert Exp $
+// $Id: pcivga.cc 10897 2011-12-30 09:10:11Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002,2003 Mike Nordell
@@ -35,7 +35,7 @@
 
 #include "iodev.h"
 
-#if BX_SUPPORT_PCI && BX_SUPPORT_PCIVGA
+#if BX_SUPPORT_PCI
 
 #include "pci.h"
 #include "pcivga.h"
@@ -77,7 +77,7 @@ void bx_pcivga_c::init(void)
                             "Experimental PCI VGA");
 
   for (i=0; i<256; i++) {
-    BX_PCIVGA_THIS s.pci_conf[i] = 0x0;
+    BX_PCIVGA_THIS pci_conf[i] = 0x0;
   }
 
   // readonly registers
@@ -95,10 +95,15 @@ void bx_pcivga_c::init(void)
     { 0x0e, 0x00 }                  // header_type_generic
   };
   for (i = 0; i < sizeof(init_vals) / sizeof(*init_vals); ++i) {
-    BX_PCIVGA_THIS s.pci_conf[init_vals[i].addr] = init_vals[i].val;
+    BX_PCIVGA_THIS pci_conf[init_vals[i].addr] = init_vals[i].val;
   }
-  WriteHostDWordToLittleEndian(&BX_PCIVGA_THIS s.pci_conf[0x10], 0x08);
-  BX_PCIVGA_THIS s.base_address = 0;
+  BX_PCIVGA_THIS vbe_present = !strcmp(SIM->get_param_string(BXPN_VGA_EXTENSION)->getptr(), "vbe");
+  if (BX_PCIVGA_THIS vbe_present) {
+    WriteHostDWordToLittleEndian(&BX_PCIVGA_THIS pci_conf[0x10], 0x08);
+    BX_PCIVGA_THIS pci_base_address[0] = 0;
+  }
+  BX_PCIVGA_THIS pci_rom_address = 0;
+  BX_PCIVGA_THIS load_pci_rom(SIM->get_param_string(BXPN_VGA_ROM_PATH)->getptr());
 }
 
 void bx_pcivga_c::reset(unsigned type)
@@ -111,22 +116,64 @@ void bx_pcivga_c::reset(unsigned type)
       { 0x06, 0x00 }, { 0x07, 0x02 }  // status_devsel_medium
   };
   for (unsigned i = 0; i < sizeof(reset_vals) / sizeof(*reset_vals); ++i) {
-    BX_PCIVGA_THIS s.pci_conf[reset_vals[i].addr] = reset_vals[i].val;
+    BX_PCIVGA_THIS pci_conf[reset_vals[i].addr] = reset_vals[i].val;
   }
 }
 
 void bx_pcivga_c::register_state(void)
 {
   bx_list_c *list = new bx_list_c(SIM->get_bochs_root(), "pcivga", "PCI VGA Adapter State", 1);
-  register_pci_state(list, BX_PCIVGA_THIS s.pci_conf);
+  register_pci_state(list);
 }
 
 void bx_pcivga_c::after_restore_state(void)
 {
-  if (DEV_vbe_set_base_addr(&BX_PCIVGA_THIS s.base_address,
-                            &BX_PCIVGA_THIS s.pci_conf[0x10])) {
-    BX_INFO(("new base address: 0x%08x", BX_PCIVGA_THIS s.base_address));
+  if (BX_PCIVGA_THIS vbe_present) {
+    if (DEV_vbe_set_base_addr(&BX_PCIVGA_THIS pci_base_address[0],
+                              &BX_PCIVGA_THIS pci_conf[0x10])) {
+      BX_INFO(("new base address: 0x%08x", BX_PCIVGA_THIS pci_base_address[0]));
+    }
   }
+  if (DEV_pci_set_base_mem(this, mem_read_handler, mem_write_handler,
+                           &BX_PCIVGA_THIS pci_rom_address,
+                           &BX_PCIVGA_THIS pci_conf[0x30],
+                           BX_PCIVGA_THIS pci_rom_size)) {
+    BX_INFO(("new ROM address: 0x%08x", BX_PCIVGA_THIS pci_rom_address));
+  }
+}
+
+bx_bool bx_pcivga_c::mem_read_handler(bx_phy_address addr, unsigned len, void *data, void *param)
+{
+  Bit8u *data_ptr;
+
+  Bit32u mask = (BX_PCIVGA_THIS pci_rom_size - 1);
+#ifdef BX_LITTLE_ENDIAN
+  data_ptr = (Bit8u *) data;
+#else
+  data_ptr = (Bit8u *) data + (len - 1);
+#endif
+
+  for (unsigned i = 0; i < len; i++) {
+    if (BX_PCIVGA_THIS pci_conf[0x30] & 0x01) {
+      *data_ptr = BX_PCIVGA_THIS pci_rom[addr & mask];
+    } else {
+      *data_ptr = 0xff;
+    }
+    addr++;
+#ifdef BX_LITTLE_ENDIAN
+    data_ptr++;
+#else
+    data_ptr--;
+#endif
+  }
+
+  return 1;
+}
+
+bx_bool bx_pcivga_c::mem_write_handler(bx_phy_address addr, unsigned len, void *data, void *param)
+{
+  BX_INFO(("write to ROM ignored (addr=0x%08x len=%d)", (Bit32u)addr, len));
+  return 1;
 }
 
 // pci configuration space read callback handler
@@ -135,7 +182,7 @@ Bit32u bx_pcivga_c::pci_read_handler(Bit8u address, unsigned io_len)
   Bit32u value = 0;
 
   for (unsigned i=0; i<io_len; i++) {
-    value |= (BX_PCIVGA_THIS s.pci_conf[address+i] << (i*8));
+    value |= (BX_PCIVGA_THIS pci_conf[address+i] << (i*8));
   }
 
   if (io_len == 1)
@@ -152,37 +199,7 @@ Bit32u bx_pcivga_c::pci_read_handler(Bit8u address, unsigned io_len)
 // static pci configuration space write callback handler
 void bx_pcivga_c::pci_write_handler(Bit8u address, Bit32u value, unsigned io_len)
 {
-  unsigned i;
-  unsigned write_addr;
-  Bit8u new_value, old_value;
-  bx_bool baseaddr_change = 0;
-
-  if ((address >= 0x14) && (address < 0x34))
-    return;
-
-  for (i = 0; i < io_len; i++) {
-    write_addr = address + i;
-    old_value = BX_PCIVGA_THIS s.pci_conf[write_addr];
-    new_value = (Bit8u)(value & 0xff);
-    switch (write_addr) {
-      case 0x04: // disallowing write to command
-      case 0x06: // disallowing write to status lo-byte (is that expected?)
-        break;
-      case 0x10: // base address #0
-        new_value = (new_value & 0xf0) | (old_value & 0x0f);
-      case 0x11: case 0x12: case 0x13:
-        baseaddr_change |= (old_value != new_value);
-      default:
-        BX_PCIVGA_THIS s.pci_conf[write_addr] = new_value;
-    }
-    value >>= 8;
-  }
-  if (baseaddr_change) {
-    if (DEV_vbe_set_base_addr(&BX_PCIVGA_THIS s.base_address,
-                              &BX_PCIVGA_THIS s.pci_conf[0x10])) {
-      BX_INFO(("new base address: 0x%08x", BX_PCIVGA_THIS s.base_address));
-    }
-  }
+  bx_bool baseaddr_change = 0, romaddr_change = 0;
 
   if (io_len == 1)
     BX_DEBUG(("write PCI register 0x%02x value 0x%02x", address, value));
@@ -190,6 +207,51 @@ void bx_pcivga_c::pci_write_handler(Bit8u address, Bit32u value, unsigned io_len
     BX_DEBUG(("write PCI register 0x%02x value 0x%04x", address, value));
   else if (io_len == 4)
     BX_DEBUG(("write PCI register 0x%02x value 0x%08x", address, value));
+
+  if ((address >= 0x14) && (address < 0x30))
+    return;
+
+  if (address == 0x30) {
+    value = value & 0xfffffc01;
+    romaddr_change = 1;
+  }
+
+  for (unsigned i = 0; i < io_len; i++) {
+    unsigned write_addr = address + i;
+    Bit8u old_value = BX_PCIVGA_THIS pci_conf[write_addr];
+    Bit8u new_value = (Bit8u)(value & 0xff);
+    switch (write_addr) {
+      case 0x04: // disallowing write to command
+      case 0x06: // disallowing write to status lo-byte (is that expected?)
+        break;
+      case 0x10: // base address #0
+        new_value = (new_value & 0xf0) | (old_value & 0x0f);
+      case 0x11: case 0x12: case 0x13:
+        if (BX_PCIVGA_THIS vbe_present) {
+          baseaddr_change |= (old_value != new_value);
+        } else {
+          break;
+        }
+      default:
+        BX_PCIVGA_THIS pci_conf[write_addr] = new_value;
+    }
+    value >>= 8;
+  }
+
+  if (baseaddr_change) {
+    if (DEV_vbe_set_base_addr(&BX_PCIVGA_THIS pci_base_address[0],
+                              &BX_PCIVGA_THIS pci_conf[0x10])) {
+      BX_INFO(("new base address: 0x%08x", BX_PCIVGA_THIS pci_base_address[0]));
+    }
+  }
+  if (romaddr_change) {
+    if (DEV_pci_set_base_mem(this, mem_read_handler, mem_write_handler,
+                             &BX_PCIVGA_THIS pci_rom_address,
+                             &BX_PCIVGA_THIS pci_conf[0x30],
+                             BX_PCIVGA_THIS pci_rom_size)) {
+      BX_INFO(("new ROM address: 0x%08x", BX_PCIVGA_THIS pci_rom_address));
+    }
+  }
 }
 
-#endif // BX_SUPPORT_PCI && BX_SUPPORT_PCIVGA
+#endif // BX_SUPPORT_PCI
