@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: scsi_device.cc,v 1.21 2011/01/24 20:35:51 vruppert Exp $
+// $Id: scsi_device.cc 10414 2011-06-21 19:54:37Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2007-2011  The Bochs Project
@@ -34,7 +34,6 @@
 
 #define LOG_THIS
 
-#define BX_MIN(a,b) ( ((a)>(b))?(b):(a) )
 #define DEVICE_NAME "SCSI drive"
 
 static SCSIRequest *free_requests = NULL;
@@ -44,7 +43,9 @@ scsi_device_t::scsi_device_t(device_image_t *_hdimage, int _tcq,
                            scsi_completionfn _completion, void *_dev)
 {
   type = SCSIDEV_TYPE_DISK;
+#ifdef LOWLEVEL_CDROM
   cdrom = NULL;
+#endif
   hdimage = _hdimage;
   requests = NULL;
   sense = 0;
@@ -60,6 +61,7 @@ scsi_device_t::scsi_device_t(device_image_t *_hdimage, int _tcq,
   put("SCSID");
 }
 
+#ifdef LOWLEVEL_CDROM
 scsi_device_t::scsi_device_t(LOWLEVEL_CDROM *_cdrom, int _tcq,
                            scsi_completionfn _completion, void *_dev)
 {
@@ -79,6 +81,7 @@ scsi_device_t::scsi_device_t(LOWLEVEL_CDROM *_cdrom, int _tcq,
 
   put("SCSIC");
 }
+#endif
 
 scsi_device_t::~scsi_device_t(void)
 {
@@ -230,11 +233,15 @@ void scsi_device_t::scsi_read_data(Bit32u tag)
     n = SCSI_DMA_BUF_SIZE / (512 * cluster_size);
   r->buf_len = n * 512 * cluster_size;
   if (type == SCSIDEV_TYPE_CDROM) {
+#ifdef LOWLEVEL_CDROM
     if (!cdrom->read_block(r->dma_buf, (Bit32u)r->sector, 2048)) {
       scsi_command_complete(r, STATUS_CHECK_CONDITION, SENSE_HARDWARE_ERROR);
     } else {
       scsi_read_complete((void*)r, 0);
     }
+#else
+    scsi_command_complete(r, STATUS_CHECK_CONDITION, SENSE_HARDWARE_ERROR);
+#endif
   } else {
     ret = (int)hdimage->lseek(r->sector * 512, SEEK_SET);
     if (ret < 0) {
@@ -624,7 +631,9 @@ Bit32s scsi_device_t::scsi_send_command(Bit32u tag, Bit8u *buf, int lun)
       if (type == SCSIDEV_TYPE_CDROM && (buf[4] & 2)) {
         if (!(buf[4] & 1)) {
           // eject medium
+#ifdef LOWLEVEL_CDROM
           cdrom->eject_cdrom();
+#endif
           inserted = 0;
         }
       }
@@ -638,7 +647,11 @@ Bit32s scsi_device_t::scsi_send_command(Bit32u tag, Bit8u *buf, int lun)
       // The normal LEN field for this command is zero
       memset(outbuf, 0, 8);
       if (type == SCSIDEV_TYPE_CDROM) {
+#ifdef LOWLEVEL_CDROM
         nb_sectors = cdrom->capacity();
+#else
+        nb_sectors = 0;
+#endif
       } else {
         nb_sectors = hdimage->hd_size / 512;
       }
@@ -692,6 +705,7 @@ Bit32s scsi_device_t::scsi_send_command(Bit32u tag, Bit8u *buf, int lun)
           format = buf[2] & 0xf;
           start_track = buf[6];
           BX_DEBUG(("Read TOC (track %d format %d msf %d)", start_track, format, msf >> 1));
+#ifdef LOWLEVEL_CDROM
           cdrom->read_toc(outbuf, &toclen, msf, start_track, format);
           if (toclen > 0) {
             if (len > toclen)
@@ -699,6 +713,7 @@ Bit32s scsi_device_t::scsi_send_command(Bit32u tag, Bit8u *buf, int lun)
             r->buf_len = len;
             break;
           }
+#endif
           BX_ERROR(("Read TOC error"));
           goto fail;
         } else {
@@ -749,7 +764,11 @@ Bit32s scsi_device_t::scsi_send_command(Bit32u tag, Bit8u *buf, int lun)
 
       // Current/Max Cap Header
       if (type == SCSIDEV_TYPE_CDROM) {
+#ifdef LOWLEVEL_CDROM
         nb_sectors = cdrom->capacity();
+#else
+        nb_sectors = 0;
+#endif
       } else {
         nb_sectors = (hdimage->hd_size / 512);
       }
