@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: soundosx.h,v 1.7 2011/02/13 17:25:25 vruppert Exp $
+// $Id: soundosx.h 10365 2011-05-24 16:47:07Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2004-2011  The Bochs Project
@@ -31,11 +31,12 @@
 #else
 #define BX_SOUND_OSX_use_converter
 //#define BX_SOUND_OSX_use_quicktime
+#include <CoreServices/CoreServices.h>
 #endif
 
 #define BX_SOUND_OSX_NBUF     8   // number of buffers for digital output
 
-class bx_sound_osx_c : public bx_sound_output_c {
+class bx_sound_osx_c : public bx_sound_lowlevel_c {
 public:
     bx_sound_osx_c(logfunctions *dev);
     virtual ~bx_sound_osx_c();
@@ -48,7 +49,7 @@ public:
     virtual int    closemidioutput();
 
     virtual int    openwaveoutput(const char *wavedev);
-    virtual int    startwaveplayback(int frequency, int bits, int stereo, int format);
+    virtual int    startwaveplayback(int frequency, int bits, bx_bool stereo, int format);
     virtual int    sendwavepacket(int length, Bit8u data[]);
     virtual int    stopwaveplayback();
     virtual int    closewaveoutput();
@@ -56,16 +57,27 @@ public:
     void nextbuffer(int *outDataSize, void **outData);
 #endif
 
+  virtual int openwaveinput(const char *wavedev, sound_record_handler_t rh);
+  virtual int startwaverecord(int frequency, int bits, bx_bool stereo, int format);
+  virtual int getwavepacket(int length, Bit8u data[]);
+  virtual int stopwaverecord();
+  virtual int closewaveinput();
+
+  static void record_timer_handler(void *);
+  void record_timer(void);
 private:
     int MidiOpen;
     int WaveOpen;
 
-    Bit8u WaveData[BX_SOUND_OSX_NBUF][BX_SOUND_OUTPUT_WAVEPACKETSIZE];
+    Bit8u WaveData[BX_SOUND_OSX_NBUF][BX_SOUNDLOW_WAVEPACKETSIZE];
     int WaveLength[BX_SOUND_OSX_NBUF];
     int head, tail;  // buffer pointers
 
 #ifdef BX_SOUND_OSX_use_converter
     int WavePlaying;
+
+    OSStatus core_audio_pause();
+    OSStatus core_audio_resume();
 #endif
 };
 
