@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: soundwin.h,v 1.11 2011/02/13 17:25:25 vruppert Exp $
+// $Id: soundwin.h 10362 2011-05-13 21:08:33Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2011  The Bochs Project
@@ -31,11 +31,11 @@
 
 #define BX_SOUND_WINDOWS_MAXSYSEXLEN  256    // maximum supported length of a sysex message
 
-#define BX_SOUND_WINDOWS_NBUF 	64  // number of buffers for the output, must be power of 2 and >= 4
-#define BX_SOUND_WINDOWS_NMASK	(BX_SOUND_WINDOWS_NBUF - 1)
+#define BX_SOUND_WINDOWS_NBUF   64  // number of buffers for the output, must be power of 2 and >= 4
+#define BX_SOUND_WINDOWS_NMASK  (BX_SOUND_WINDOWS_NBUF - 1)
 
 #ifndef WAVEMAPPER
-#define WAVEMAPPER		-1
+#define WAVEMAPPER -1
 #endif
 
 // Definitions for WINMM.DLL, if not defined already
@@ -164,7 +164,7 @@ typedef struct {
 
 #endif
 
-class bx_sound_windows_c : public bx_sound_output_c {
+class bx_sound_windows_c : public bx_sound_lowlevel_c {
 public:
   bx_sound_windows_c(logfunctions *dev);
   virtual ~bx_sound_windows_c();
@@ -177,44 +177,59 @@ public:
   virtual int    closemidioutput();
 
   virtual int    openwaveoutput(const char *wavedev);
-  virtual int    startwaveplayback(int frequency, int bits, int stereo, int format);
+  virtual int    startwaveplayback(int frequency, int bits, bx_bool stereo, int format);
   virtual int    sendwavepacket(int length, Bit8u data[]);
   virtual int    stopwaveplayback();
   virtual int    closewaveoutput();
 
+  virtual int openwaveinput(const char *wavedev, sound_record_handler_t rh);
+  virtual int startwaverecord(int frequency, int bits, bx_bool stereo, int format);
+  virtual int getwavepacket(int length, Bit8u data[]);
+  virtual int stopwaverecord();
+  virtual int closewaveinput();
+
+  static void record_timer_handler(void *);
+  void record_timer(void);
+
 private:
-  struct bx_sb16_waveinfo_struct {
+  struct bx_sound_waveinfo_struct {
     int frequency;
     int bits;
-    int stereo;
+    bx_bool stereo;
     int format;
   };
 
   HMIDIOUT MidiOut;       // Midi output device
   int MidiOpen;           // is it open?
-  HWAVEOUT WaveOut;       // Wave output device
-  int WaveOpen;           // is it open?
+  HWAVEOUT hWaveOut;      // Wave output device
+  int WaveOutOpen;        // is it open?
+  HWAVEIN hWaveIn;        // Wave input device
+  int WaveInOpen;         // is it open?
 
   UINT WaveDevice;        // Wave device ID, for waveOutOpen
 
-	// some data for the wave buffers
+  // some data for the wave buffers
   HANDLE DataHandle;          // returned by GlobalAlloc()
   Bit8u *DataPointer;         // returned by GlobalLock()
 
   LPWAVEHDR WaveHeader[BX_SOUND_WINDOWS_NBUF];
   LPSTR WaveData[BX_SOUND_WINDOWS_NBUF];
+  LPWAVEHDR WaveInHdr;
+  LPSTR WaveInData;
+  bx_bool recording;
   int length[BX_SOUND_WINDOWS_NBUF];                // length of the data in the buffer
   int needreopen;                                   // if the format has changed
   int head,tailfull,tailplay;       // These are for three states of the buffers: empty, full, played
-  bx_sb16_waveinfo_struct WaveInfo;                 // format for the next buffer to be played
+  bx_sound_waveinfo_struct WaveInfo[2];             // format for the next buffer to be played
   int iswaveready;
 
-	// and the midi buffer for the SYSEX messages
+  // and the midi buffer for the SYSEX messages
   LPMIDIHDR MidiHeader;
   LPSTR MidiData;
   int ismidiready;
 
   int playnextbuffer();
+  int recordnextpacket();
   void checkmidiready();
   void checkwaveready();
 };
