@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: memory.h,v 1.75 2011/02/11 15:33:08 vruppert Exp $
+// $Id: memory.h 10888 2011-12-29 20:52:44Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2009  The Bochs Project
@@ -45,6 +45,9 @@ class BX_CPU_C;
 #define BIOS_MAP_LAST128K(addr) (((addr) | 0xfff00000) & BIOS_MASK)
 
 typedef bx_bool (*memory_handler_t)(bx_phy_address addr, unsigned len, void *data, void *param);
+// return a pointer to 4K region containing <addr> or NULL if direct access is not allowed
+// same format as getHostMemAddr method
+typedef Bit8u* (*memory_direct_access_handler_t)(bx_phy_address addr, unsigned rw, void *param);
 
 struct memory_handler_struct {
   struct memory_handler_struct *next;
@@ -53,6 +56,7 @@ struct memory_handler_struct {
   bx_phy_address end;
   memory_handler_t read_handler;
   memory_handler_t write_handler;
+  memory_direct_access_handler_t da_handler;
 };
 
 #define SMRAM_CODE  1
@@ -72,9 +76,17 @@ private:
   Bit8u  **blocks;
   Bit8u   *rom;      // 512k BIOS rom space + 128k expansion rom space
   Bit8u   *bogus;    // 4k for unexisting memory
-  unsigned used_blocks;
   bx_bool rom_present[65];
 
+  Bit32u used_blocks;
+#if BX_LARGE_RAMFILE
+  static Bit8u * const swapped_out; // NULL; // (NULL - sizeof(Bit8u));
+  Bit32u  next_swapout_idx;
+  FILE    *overflow_file;
+
+  BX_MEM_SMF void   read_block(Bit32u block);
+#endif
+
 public:
   BX_MEM_C();
  ~BX_MEM_C();
@@ -82,15 +94,25 @@ public:
   BX_MEM_SMF Bit8u*  get_vector(bx_phy_address addr);
   BX_MEM_SMF void    init_memory(Bit64u guest, Bit64u host);
   BX_MEM_SMF void    cleanup_memory(void);
+
   BX_MEM_SMF void    enable_smram(bx_bool enable, bx_bool restricted);
   BX_MEM_SMF void    disable_smram(void);
   BX_MEM_SMF bx_bool is_smram_accessible(void);
+
+  BX_MEM_SMF Bit8u*  getHostMemAddr(BX_CPU_C *cpu, bx_phy_address addr, unsigned rw);
+
+  // Note: accesses should always be contained within a single page
   BX_MEM_SMF void    readPhysicalPage(BX_CPU_C *cpu, bx_phy_address addr,
                                       unsigned len, void *data);
   BX_MEM_SMF void    writePhysicalPage(BX_CPU_C *cpu, bx_phy_address addr,
                                        unsigned len, void *data);
+
+  BX_MEM_SMF void    dmaReadPhysicalPage(bx_phy_address addr, unsigned len, Bit8u *data);
+  BX_MEM_SMF void    dmaWritePhysicalPage(bx_phy_address addr, unsigned len, Bit8u *data);
+
   BX_MEM_SMF void    load_ROM(const char *path, bx_phy_address romaddress, Bit8u type);
   BX_MEM_SMF void    load_RAM(const char *path, bx_phy_address romaddress, Bit8u type);
+
 #if (BX_DEBUGGER || BX_DISASM || BX_GDBSTUB)
   BX_MEM_SMF bx_bool dbg_fetch_mem(BX_CPU_C *cpu, bx_phy_address addr, unsigned len, Bit8u *buf);
 #endif
@@ -98,11 +120,18 @@ public:
   BX_MEM_SMF bx_bool dbg_set_mem(bx_phy_address addr, unsigned len, Bit8u *buf);
   BX_MEM_SMF bx_bool dbg_crc32(bx_phy_address addr1, bx_phy_address addr2, Bit32u *crc);
 #endif
-  BX_MEM_SMF Bit8u* getHostMemAddr(BX_CPU_C *cpu, bx_phy_address addr, unsigned rw);
+
   BX_MEM_SMF bx_bool registerMemoryHandlers(void *param, memory_handler_t read_handler,
-		  memory_handler_t write_handler, bx_phy_address begin_addr, bx_phy_address end_addr);
-  BX_MEM_SMF bx_bool unregisterMemoryHandlers(memory_handler_t read_handler, memory_handler_t write_handler,
-		  bx_phy_address begin_addr, bx_phy_address end_addr);
+                  memory_handler_t write_handler, memory_direct_access_handler_t da_handler,
+                  bx_phy_address begin_addr, bx_phy_address end_addr);
+  BX_MEM_SMF BX_CPP_INLINE bx_bool registerMemoryHandlers(void *param, memory_handler_t read_handler,
+                  memory_handler_t write_handler,
+                  bx_phy_address begin_addr, bx_phy_address end_addr)
+  {
+     return registerMemoryHandlers(param, read_handler, write_handler, NULL, begin_addr, end_addr);
+  }
+  BX_MEM_SMF bx_bool unregisterMemoryHandlers(void *param, bx_phy_address begin_addr, bx_phy_address end_addr);
+
   BX_MEM_SMF Bit64u  get_memory_len(void);
   BX_MEM_SMF void allocate_block(Bit32u index);
   BX_MEM_SMF Bit8u* alloc_vector_aligned(Bit32u bytes, Bit32u alignment);
@@ -114,6 +143,7 @@ public:
 
   void register_state(void);
 
+  friend void ramfile_save_handler(void *devptr, FILE *fp);
   friend Bit64s memory_param_save_handler(void *devptr, bx_param_c *param);
   friend void memory_param_restore_handler(void *devptr, bx_param_c *param, Bit64s val);
 };
@@ -133,7 +163,13 @@ BX_CPP_INLINE Bit8u* BX_MEM_C::get_vector(bx_phy_address addr)
 BX_CPP_INLINE Bit8u* BX_MEM_C::get_vector(bx_phy_address addr)
 {
   Bit32u block = (Bit32u)(addr / BX_MEM_BLOCK_LEN);
-  if (! BX_MEM_THIS blocks[block]) allocate_block(block);
+#if (BX_LARGE_RAMFILE)
+  if (!BX_MEM_THIS blocks[block] || (BX_MEM_THIS blocks[block] == BX_MEM_THIS swapped_out))
+#else
+  if (!BX_MEM_THIS blocks[block])
+#endif
+    allocate_block(block);
+
   return BX_MEM_THIS blocks[block] + (Bit32u)(addr & (BX_MEM_BLOCK_LEN-1));
 }
 
