@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: param_names.h,v 1.14 2011/02/12 17:50:48 vruppert Exp $
+// $Id: param_names.h 10890 2011-12-29 21:25:34Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2009  The Bochs Project
+//  Copyright (C) 2009-2011  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -35,27 +35,42 @@
 #define BXPN_CPU_NPROCESSORS             "cpu.n_processors"
 #define BXPN_CPU_NCORES                  "cpu.n_cores"
 #define BXPN_CPU_NTHREADS                "cpu.n_threads"
+#define BXPN_CPU_MODEL                   "cpu.model"
 #define BXPN_IPS                         "cpu.ips"
 #define BXPN_SMP_QUANTUM                 "cpu.quantum"
 #define BXPN_RESET_ON_TRIPLE_FAULT       "cpu.reset_on_triple_fault"
 #define BXPN_IGNORE_BAD_MSRS             "cpu.ignore_bad_msrs"
 #define BXPN_CONFIGURABLE_MSRS_PATH      "cpu.msrs"
+#define BXPN_CPUID_LIMIT_WINNT           "cpu.cpuid_limit_winnt"
 #define BXPN_VENDOR_STRING               "cpuid.vendor_string"
 #define BXPN_BRAND_STRING                "cpuid.brand_string"
+#define BXPN_CPUID_FAMILY                "cpuid.family"
+#define BXPN_CPUID_MODEL                 "cpuid.model"
 #define BXPN_CPUID_STEPPING              "cpuid.stepping"
-#define BXPN_CPUID_LIMIT_WINNT           "cpuid.cpuid_limit_winnt"
 #define BXPN_CPUID_MMX                   "cpuid.mmx"
 #define BXPN_CPUID_SSE                   "cpuid.sse"
+#define BXPN_CPUID_SSE4A                 "cpuid.sse4a"
 #define BXPN_CPUID_AES                   "cpuid.aes"
 #define BXPN_CPUID_MOVBE                 "cpuid.movbe"
 #define BXPN_CPUID_SEP                   "cpuid.sep"
 #define BXPN_CPUID_XSAVE                 "cpuid.xsave"
-#define BXPN_CPUID_XAPIC                 "cpuid.xapic"
+#define BXPN_CPUID_XSAVEOPT              "cpuid.xsaveopt"
+#define BXPN_CPUID_VMX                   "cpuid.vmx"
+#define BXPN_CPUID_AVX                   "cpuid.avx"
+#define BXPN_CPUID_AVX_F16CVT            "cpuid.avx_f16c"
+#define BXPN_CPUID_AVX_FMA               "cpuid.avx_fma"
+#define BXPN_CPUID_BMI                   "cpuid.bmi"
+#define BXPN_CPUID_XOP                   "cpuid.xop"
+#define BXPN_CPUID_TBM                   "cpuid.tbm"
+#define BXPN_CPUID_FMA4                  "cpuid.fma4"
+#define BXPN_CPUID_APIC                  "cpuid.apic"
 #define BXPN_CPUID_MWAIT                 "cpuid.mwait"
 #define BXPN_CPUID_MWAIT_IS_NOP          "cpuid.mwait_is_nop"
+#define BXPN_CPUID_X86_64                "cpuid.x86_64"
 #define BXPN_CPUID_1G_PAGES              "cpuid.1g_pages"
 #define BXPN_CPUID_PCID                  "cpuid.pcid"
 #define BXPN_CPUID_FSGSBASE              "cpuid.fsgsbase"
+#define BXPN_CPUID_SMEP                  "cpuid.smep"
 #define BXPN_MEM_SIZE                    "memory.standard.ram.size"
 #define BXPN_HOST_MEM_SIZE               "memory.standard.ram.host_size"
 #define BXPN_ROM_PATH                    "memory.standard.rom.path"
@@ -91,7 +106,7 @@
 #define BXPN_FULLSCREEN                  "display.fullscreen"
 #define BXPN_SCREENMODE                  "display.screenmode"
 #define BXPN_VGA_EXTENSION               "display.vga_extension"
-#define BXPN_VGA_UPDATE_INTERVAL         "display.vga_update_interval"
+#define BXPN_VGA_UPDATE_FREQUENCY        "display.vga_update_frequency"
 #define BXPN_KBD_TYPE                    "keyboard_mouse.keyboard.type"
 #define BXPN_KBD_SERIAL_DELAY            "keyboard_mouse.keyboard.serial_delay"
 #define BXPN_KBD_PASTE_DELAY             "keyboard_mouse.keyboard.paste_delay"
@@ -137,28 +152,24 @@
 #define BXPN_ATA3_SLAVE                  "ata.3.slave"
 #define BXPN_USB_UHCI                    "ports.usb.uhci"
 #define BXPN_UHCI_ENABLED                "ports.usb.uhci.enabled"
-#define BXPN_UHCI_PORT1_DEVICE           "ports.usb.uhci.port1.device"
-#define BXPN_UHCI_PORT1_OPTIONS          "ports.usb.uhci.port1.options"
-#define BXPN_UHCI_PORT2_DEVICE           "ports.usb.uhci.port2.device"
-#define BXPN_UHCI_PORT2_OPTIONS          "ports.usb.uhci.port2.options"
 #define BXPN_USB_OHCI                    "ports.usb.ohci"
 #define BXPN_OHCI_ENABLED                "ports.usb.ohci.enabled"
-#define BXPN_OHCI_PORT1_DEVICE           "ports.usb.ohci.port1.device"
-#define BXPN_OHCI_PORT1_OPTIONS          "ports.usb.ohci.port1.options"
-#define BXPN_OHCI_PORT2_DEVICE           "ports.usb.ohci.port2.device"
-#define BXPN_OHCI_PORT2_OPTIONS          "ports.usb.ohci.port2.options"
+#define BXPN_USB_XHCI                    "ports.usb.xhci"
+#define BXPN_XHCI_ENABLED                "ports.usb.xhci.enabled"
 #define BXPN_NE2K                        "network.ne2k"
 #define BXPN_NE2K_ENABLED                "network.ne2k.enabled"
 #define BXPN_PNIC                        "network.pnic"
 #define BXPN_PNIC_ENABLED                "network.pnic.enabled"
-#define BXPN_SB16                        "sound.sb16"
+#define BXPN_SOUND_SB16                  "sound.sb16"
 #define BXPN_SB16_ENABLED                "sound.sb16.enabled"
 #define BXPN_SB16_MIDIFILE               "sound.sb16.midifile"
 #define BXPN_SB16_WAVEFILE               "sound.sb16.wavefile"
 #define BXPN_SB16_DMATIMER               "sound.sb16.dmatimer"
 #define BXPN_SB16_LOGLEVEL               "sound.sb16.loglevel"
+#define BXPN_SOUND_ES1370                "sound.es1370"
+#define BXPN_ES1370_ENABLED              "sound.es1370.enabled"
+#define BXPN_ES1370_WAVEDEV              "sound.es1370.wavedev"
 #define BXPN_PORT_E9_HACK                "misc.port_e9_hack"
-#define BXPN_TEXT_SNAPSHOT_CHECK         "misc.text_snapshot_check"
 #define BXPN_GDBSTUB                     "misc.gdbstub"
 #define BXPN_PLUGIN_CTRL                 "misc.plugin_ctrl"
 #define BXPN_LOG_FILENAME                "log.filename"
